/********************************************************************************
** Form generated from reading UI file 'showtextdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SHOWTEXTDIALOG_H
#define UI_SHOWTEXTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>

QT_BEGIN_NAMESPACE

class Ui_ShowTextDialog
{
public:
    QGridLayout *gridLayout;
    QTextBrowser *textBrowser;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *okButton;

    void setupUi(QDialog *ShowTextDialog)
    {
        if (ShowTextDialog->objectName().isEmpty())
            ShowTextDialog->setObjectName(QString::fromUtf8("ShowTextDialog"));
        ShowTextDialog->resize(1116, 748);
        gridLayout = new QGridLayout(ShowTextDialog);
        gridLayout->setSpacing(6);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        textBrowser = new QTextBrowser(ShowTextDialog);
        textBrowser->setObjectName(QString::fromUtf8("textBrowser"));

        gridLayout->addWidget(textBrowser, 0, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(131, 31, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        okButton = new QPushButton(ShowTextDialog);
        okButton->setObjectName(QString::fromUtf8("okButton"));

        hboxLayout->addWidget(okButton);


        gridLayout->addLayout(hboxLayout, 1, 0, 1, 1);


        retranslateUi(ShowTextDialog);
        QObject::connect(okButton, SIGNAL(clicked()), ShowTextDialog, SLOT(accept()));

        QMetaObject::connectSlotsByName(ShowTextDialog);
    } // setupUi

    void retranslateUi(QDialog *ShowTextDialog)
    {
        ShowTextDialog->setWindowTitle(QCoreApplication::translate("ShowTextDialog", "Dialog", nullptr));
        okButton->setText(QCoreApplication::translate("ShowTextDialog", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ShowTextDialog: public Ui_ShowTextDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SHOWTEXTDIALOG_H
