/****************************************************************************
** Meta object code from reading C++ file 'texteditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/texteditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'texteditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TextEditor_t {
    QByteArrayData data[56];
    char stringdata0[692];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TextEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TextEditor_t qt_meta_stringdata_TextEditor = {
    {
QT_MOC_LITERAL(0, 0, 10), // "TextEditor"
QT_MOC_LITERAL(1, 11, 14), // "textHasChanged"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 7), // "VymText"
QT_MOC_LITERAL(4, 35, 2), // "vt"
QT_MOC_LITERAL(5, 38, 12), // "windowClosed"
QT_MOC_LITERAL(6, 51, 20), // "fontFamilyHasChanged"
QT_MOC_LITERAL(7, 72, 18), // "fontSizeHasChanged"
QT_MOC_LITERAL(8, 91, 13), // "editorChanged"
QT_MOC_LITERAL(9, 105, 11), // "setRichText"
QT_MOC_LITERAL(10, 117, 1), // "t"
QT_MOC_LITERAL(11, 119, 12), // "setPlainText"
QT_MOC_LITERAL(12, 132, 11), // "setTextAuto"
QT_MOC_LITERAL(13, 144, 10), // "setVymText"
QT_MOC_LITERAL(14, 155, 11), // "setInactive"
QT_MOC_LITERAL(15, 167, 11), // "editCopyAll"
QT_MOC_LITERAL(16, 179, 5), // "clear"
QT_MOC_LITERAL(17, 185, 9), // "deleteAll"
QT_MOC_LITERAL(18, 195, 8), // "textLoad"
QT_MOC_LITERAL(19, 204, 10), // "textSaveAs"
QT_MOC_LITERAL(20, 215, 8), // "textSave"
QT_MOC_LITERAL(21, 224, 17), // "textExportAsASCII"
QT_MOC_LITERAL(22, 242, 9), // "textPrint"
QT_MOC_LITERAL(23, 252, 12), // "textEditUndo"
QT_MOC_LITERAL(24, 265, 14), // "toggleFonthint"
QT_MOC_LITERAL(25, 280, 15), // "setRichTextMode"
QT_MOC_LITERAL(26, 296, 1), // "b"
QT_MOC_LITERAL(27, 298, 14), // "toggleRichText"
QT_MOC_LITERAL(28, 313, 12), // "setFixedFont"
QT_MOC_LITERAL(29, 326, 10), // "setVarFont"
QT_MOC_LITERAL(30, 337, 8), // "textBold"
QT_MOC_LITERAL(31, 346, 13), // "textUnderline"
QT_MOC_LITERAL(32, 360, 10), // "textItalic"
QT_MOC_LITERAL(33, 371, 10), // "textFamily"
QT_MOC_LITERAL(34, 382, 1), // "f"
QT_MOC_LITERAL(35, 384, 8), // "textSize"
QT_MOC_LITERAL(36, 393, 1), // "p"
QT_MOC_LITERAL(37, 395, 9), // "textColor"
QT_MOC_LITERAL(38, 405, 9), // "textAlign"
QT_MOC_LITERAL(39, 415, 8), // "QAction*"
QT_MOC_LITERAL(40, 424, 10), // "textVAlign"
QT_MOC_LITERAL(41, 435, 11), // "fontChanged"
QT_MOC_LITERAL(42, 447, 12), // "colorChanged"
QT_MOC_LITERAL(43, 460, 1), // "c"
QT_MOC_LITERAL(44, 462, 13), // "formatChanged"
QT_MOC_LITERAL(45, 476, 15), // "QTextCharFormat"
QT_MOC_LITERAL(46, 492, 16), // "alignmentChanged"
QT_MOC_LITERAL(47, 509, 1), // "a"
QT_MOC_LITERAL(48, 511, 24), // "verticalAlignmentChanged"
QT_MOC_LITERAL(49, 536, 34), // "QTextCharFormat::VerticalAlig..."
QT_MOC_LITERAL(50, 571, 13), // "updateActions"
QT_MOC_LITERAL(51, 585, 8), // "setState"
QT_MOC_LITERAL(52, 594, 11), // "EditorState"
QT_MOC_LITERAL(53, 606, 11), // "updateState"
QT_MOC_LITERAL(54, 618, 36), // "selectColorRichTextDefaultBac..."
QT_MOC_LITERAL(55, 655, 36) // "selectColorRichTextDefaultFor..."

    },
    "TextEditor\0textHasChanged\0\0VymText\0"
    "vt\0windowClosed\0fontFamilyHasChanged\0"
    "fontSizeHasChanged\0editorChanged\0"
    "setRichText\0t\0setPlainText\0setTextAuto\0"
    "setVymText\0setInactive\0editCopyAll\0"
    "clear\0deleteAll\0textLoad\0textSaveAs\0"
    "textSave\0textExportAsASCII\0textPrint\0"
    "textEditUndo\0toggleFonthint\0setRichTextMode\0"
    "b\0toggleRichText\0setFixedFont\0setVarFont\0"
    "textBold\0textUnderline\0textItalic\0"
    "textFamily\0f\0textSize\0p\0textColor\0"
    "textAlign\0QAction*\0textVAlign\0fontChanged\0"
    "colorChanged\0c\0formatChanged\0"
    "QTextCharFormat\0alignmentChanged\0a\0"
    "verticalAlignmentChanged\0"
    "QTextCharFormat::VerticalAlignment\0"
    "updateActions\0setState\0EditorState\0"
    "updateState\0selectColorRichTextDefaultBackground\0"
    "selectColorRichTextDefaultForeground"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TextEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      42,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  224,    2, 0x06 /* Public */,
       5,    0,  227,    2, 0x06 /* Public */,
       6,    0,  228,    2, 0x06 /* Public */,
       7,    0,  229,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0,  230,    2, 0x0a /* Public */,
       9,    1,  231,    2, 0x0a /* Public */,
      11,    1,  234,    2, 0x0a /* Public */,
      12,    1,  237,    2, 0x0a /* Public */,
      13,    1,  240,    2, 0x0a /* Public */,
      14,    0,  243,    2, 0x0a /* Public */,
      15,    0,  244,    2, 0x0a /* Public */,
      16,    0,  245,    2, 0x0a /* Public */,
      17,    0,  246,    2, 0x09 /* Protected */,
      18,    0,  247,    2, 0x08 /* Private */,
      19,    0,  248,    2, 0x08 /* Private */,
      20,    0,  249,    2, 0x08 /* Private */,
      21,    0,  250,    2, 0x08 /* Private */,
      22,    0,  251,    2, 0x08 /* Private */,
      23,    0,  252,    2, 0x08 /* Private */,
      24,    0,  253,    2, 0x08 /* Private */,
      25,    1,  254,    2, 0x08 /* Private */,
      27,    0,  257,    2, 0x08 /* Private */,
      28,    0,  258,    2, 0x08 /* Private */,
      29,    0,  259,    2, 0x08 /* Private */,
      30,    0,  260,    2, 0x08 /* Private */,
      31,    0,  261,    2, 0x08 /* Private */,
      32,    0,  262,    2, 0x08 /* Private */,
      33,    1,  263,    2, 0x08 /* Private */,
      35,    1,  266,    2, 0x08 /* Private */,
      37,    0,  269,    2, 0x08 /* Private */,
      38,    1,  270,    2, 0x08 /* Private */,
      40,    0,  273,    2, 0x08 /* Private */,
      41,    1,  274,    2, 0x08 /* Private */,
      42,    1,  277,    2, 0x08 /* Private */,
      44,    1,  280,    2, 0x08 /* Private */,
      46,    1,  283,    2, 0x08 /* Private */,
      48,    1,  286,    2, 0x08 /* Private */,
      50,    0,  289,    2, 0x08 /* Private */,
      51,    1,  290,    2, 0x08 /* Private */,
      53,    0,  293,    2, 0x08 /* Private */,
      54,    0,  294,    2, 0x08 /* Private */,
      55,    0,  295,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   34,
    QMetaType::Void, QMetaType::QString,   36,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 39,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QFont,   34,
    QMetaType::Void, QMetaType::QColor,   43,
    QMetaType::Void, 0x80000000 | 45,   34,
    QMetaType::Void, QMetaType::Int,   47,
    QMetaType::Void, 0x80000000 | 49,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 52,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void TextEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TextEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->textHasChanged((*reinterpret_cast< const VymText(*)>(_a[1]))); break;
        case 1: _t->windowClosed(); break;
        case 2: _t->fontFamilyHasChanged(); break;
        case 3: _t->fontSizeHasChanged(); break;
        case 4: _t->editorChanged(); break;
        case 5: _t->setRichText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->setPlainText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->setTextAuto((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->setVymText((*reinterpret_cast< const VymText(*)>(_a[1]))); break;
        case 9: _t->setInactive(); break;
        case 10: _t->editCopyAll(); break;
        case 11: _t->clear(); break;
        case 12: _t->deleteAll(); break;
        case 13: _t->textLoad(); break;
        case 14: _t->textSaveAs(); break;
        case 15: _t->textSave(); break;
        case 16: _t->textExportAsASCII(); break;
        case 17: _t->textPrint(); break;
        case 18: _t->textEditUndo(); break;
        case 19: _t->toggleFonthint(); break;
        case 20: _t->setRichTextMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->toggleRichText(); break;
        case 22: _t->setFixedFont(); break;
        case 23: _t->setVarFont(); break;
        case 24: _t->textBold(); break;
        case 25: _t->textUnderline(); break;
        case 26: _t->textItalic(); break;
        case 27: _t->textFamily((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 28: _t->textSize((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 29: _t->textColor(); break;
        case 30: _t->textAlign((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 31: _t->textVAlign(); break;
        case 32: _t->fontChanged((*reinterpret_cast< const QFont(*)>(_a[1]))); break;
        case 33: _t->colorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 34: _t->formatChanged((*reinterpret_cast< const QTextCharFormat(*)>(_a[1]))); break;
        case 35: _t->alignmentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 36: _t->verticalAlignmentChanged((*reinterpret_cast< QTextCharFormat::VerticalAlignment(*)>(_a[1]))); break;
        case 37: _t->updateActions(); break;
        case 38: _t->setState((*reinterpret_cast< EditorState(*)>(_a[1]))); break;
        case 39: _t->updateState(); break;
        case 40: _t->selectColorRichTextDefaultBackground(); break;
        case 41: _t->selectColorRichTextDefaultForeground(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TextEditor::*)(const VymText & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextEditor::textHasChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TextEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextEditor::windowClosed)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TextEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextEditor::fontFamilyHasChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TextEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextEditor::fontSizeHasChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TextEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_TextEditor.data,
    qt_meta_data_TextEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TextEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TextEditor.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int TextEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 42)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 42;
    }
    return _id;
}

// SIGNAL 0
void TextEditor::textHasChanged(const VymText & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TextEditor::windowClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TextEditor::fontFamilyHasChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TextEditor::fontSizeHasChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
