/****************************************************************************
** Meta object code from reading C++ file 'export-html-dialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/export-html-dialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'export-html-dialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ExportHTMLDialog_t {
    QByteArrayData data[30];
    char stringdata0[517];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ExportHTMLDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ExportHTMLDialog_t qt_meta_stringdata_ExportHTMLDialog = {
    {
QT_MOC_LITERAL(0, 0, 16), // "ExportHTMLDialog"
QT_MOC_LITERAL(1, 17, 12), // "readSettings"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 12), // "setDirectory"
QT_MOC_LITERAL(4, 44, 10), // "dirChanged"
QT_MOC_LITERAL(5, 55, 22), // "browseDirectoryPressed"
QT_MOC_LITERAL(6, 78, 20), // "imageCheckBoxPressed"
QT_MOC_LITERAL(7, 99, 1), // "b"
QT_MOC_LITERAL(8, 101, 28), // "includeImagesCheckBoxPressed"
QT_MOC_LITERAL(9, 130, 18), // "TOCCheckBoxPressed"
QT_MOC_LITERAL(10, 149, 24), // "numberingCheckBoxPressed"
QT_MOC_LITERAL(11, 174, 24), // "taskFlagsCheckBoxPressed"
QT_MOC_LITERAL(12, 199, 24), // "userFlagsCheckBoxPressed"
QT_MOC_LITERAL(13, 224, 24), // "textcolorCheckBoxPressed"
QT_MOC_LITERAL(14, 249, 32), // "saveSettingsInMapCheckBoxPressed"
QT_MOC_LITERAL(15, 282, 23), // "warningsCheckBoxPressed"
QT_MOC_LITERAL(16, 306, 21), // "outputCheckBoxPressed"
QT_MOC_LITERAL(17, 328, 13), // "cssSrcChanged"
QT_MOC_LITERAL(18, 342, 13), // "cssDstChanged"
QT_MOC_LITERAL(19, 356, 9), // "getCssSrc"
QT_MOC_LITERAL(20, 366, 9), // "getCssDst"
QT_MOC_LITERAL(21, 376, 14), // "copyCssPressed"
QT_MOC_LITERAL(22, 391, 19), // "browseCssSrcPressed"
QT_MOC_LITERAL(23, 411, 19), // "browseCssDstPressed"
QT_MOC_LITERAL(24, 431, 17), // "postscriptChanged"
QT_MOC_LITERAL(25, 449, 29), // "browsePostExportButtonPressed"
QT_MOC_LITERAL(26, 479, 12), // "saveSettings"
QT_MOC_LITERAL(27, 492, 11), // "setFilePath"
QT_MOC_LITERAL(28, 504, 1), // "s"
QT_MOC_LITERAL(29, 506, 10) // "setMapName"

    },
    "ExportHTMLDialog\0readSettings\0\0"
    "setDirectory\0dirChanged\0browseDirectoryPressed\0"
    "imageCheckBoxPressed\0b\0"
    "includeImagesCheckBoxPressed\0"
    "TOCCheckBoxPressed\0numberingCheckBoxPressed\0"
    "taskFlagsCheckBoxPressed\0"
    "userFlagsCheckBoxPressed\0"
    "textcolorCheckBoxPressed\0"
    "saveSettingsInMapCheckBoxPressed\0"
    "warningsCheckBoxPressed\0outputCheckBoxPressed\0"
    "cssSrcChanged\0cssDstChanged\0getCssSrc\0"
    "getCssDst\0copyCssPressed\0browseCssSrcPressed\0"
    "browseCssDstPressed\0postscriptChanged\0"
    "browsePostExportButtonPressed\0"
    "saveSettings\0setFilePath\0s\0setMapName"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ExportHTMLDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      26,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  144,    2, 0x0a /* Public */,
       3,    1,  145,    2, 0x0a /* Public */,
       4,    0,  148,    2, 0x0a /* Public */,
       5,    0,  149,    2, 0x0a /* Public */,
       6,    1,  150,    2, 0x0a /* Public */,
       8,    1,  153,    2, 0x0a /* Public */,
       9,    1,  156,    2, 0x0a /* Public */,
      10,    1,  159,    2, 0x0a /* Public */,
      11,    1,  162,    2, 0x0a /* Public */,
      12,    1,  165,    2, 0x0a /* Public */,
      13,    1,  168,    2, 0x0a /* Public */,
      14,    1,  171,    2, 0x0a /* Public */,
      15,    1,  174,    2, 0x0a /* Public */,
      16,    1,  177,    2, 0x0a /* Public */,
      17,    0,  180,    2, 0x0a /* Public */,
      18,    0,  181,    2, 0x0a /* Public */,
      19,    0,  182,    2, 0x0a /* Public */,
      20,    0,  183,    2, 0x0a /* Public */,
      21,    0,  184,    2, 0x0a /* Public */,
      22,    0,  185,    2, 0x0a /* Public */,
      23,    0,  186,    2, 0x0a /* Public */,
      24,    0,  187,    2, 0x0a /* Public */,
      25,    0,  188,    2, 0x0a /* Public */,
      26,    0,  189,    2, 0x0a /* Public */,
      27,    1,  190,    2, 0x0a /* Public */,
      29,    1,  193,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   28,
    QMetaType::Void, QMetaType::QString,   28,

       0        // eod
};

void ExportHTMLDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ExportHTMLDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->readSettings(); break;
        case 1: _t->setDirectory((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->dirChanged(); break;
        case 3: _t->browseDirectoryPressed(); break;
        case 4: _t->imageCheckBoxPressed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->includeImagesCheckBoxPressed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->TOCCheckBoxPressed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->numberingCheckBoxPressed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->taskFlagsCheckBoxPressed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->userFlagsCheckBoxPressed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->textcolorCheckBoxPressed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->saveSettingsInMapCheckBoxPressed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->warningsCheckBoxPressed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->outputCheckBoxPressed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->cssSrcChanged(); break;
        case 15: _t->cssDstChanged(); break;
        case 16: { QString _r = _t->getCssSrc();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 17: { QString _r = _t->getCssDst();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->copyCssPressed(); break;
        case 19: _t->browseCssSrcPressed(); break;
        case 20: _t->browseCssDstPressed(); break;
        case 21: _t->postscriptChanged(); break;
        case 22: _t->browsePostExportButtonPressed(); break;
        case 23: _t->saveSettings(); break;
        case 24: _t->setFilePath((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 25: _t->setMapName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ExportHTMLDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ExportHTMLDialog.data,
    qt_meta_data_ExportHTMLDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ExportHTMLDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExportHTMLDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ExportHTMLDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int ExportHTMLDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 26;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
