/********************************************************************************
** Form generated from reading UI file 'default-map-settings-dialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DEFAULT_2D_MAP_2D_SETTINGS_2D_DIALOG_H
#define UI_DEFAULT_2D_MAP_2D_SETTINGS_2D_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DefaultMapSettingsDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QCheckBox *autoCheckBox;
    QHBoxLayout *horizontalLayout;
    QLineEdit *pathLineEdit;
    QPushButton *setPathPushButton;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DefaultMapSettingsDialog)
    {
        if (DefaultMapSettingsDialog->objectName().isEmpty())
            DefaultMapSettingsDialog->setObjectName(QString::fromUtf8("DefaultMapSettingsDialog"));
        DefaultMapSettingsDialog->resize(412, 154);
        verticalLayout = new QVBoxLayout(DefaultMapSettingsDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        label = new QLabel(DefaultMapSettingsDialog);
        label->setObjectName(QString::fromUtf8("label"));

        verticalLayout->addWidget(label);

        autoCheckBox = new QCheckBox(DefaultMapSettingsDialog);
        autoCheckBox->setObjectName(QString::fromUtf8("autoCheckBox"));

        verticalLayout->addWidget(autoCheckBox);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        pathLineEdit = new QLineEdit(DefaultMapSettingsDialog);
        pathLineEdit->setObjectName(QString::fromUtf8("pathLineEdit"));

        horizontalLayout->addWidget(pathLineEdit);

        setPathPushButton = new QPushButton(DefaultMapSettingsDialog);
        setPathPushButton->setObjectName(QString::fromUtf8("setPathPushButton"));
        setPathPushButton->setText(QString::fromUtf8(""));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/fileopen.png"), QSize(), QIcon::Normal, QIcon::Off);
        setPathPushButton->setIcon(icon);

        horizontalLayout->addWidget(setPathPushButton);


        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 151, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(DefaultMapSettingsDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DefaultMapSettingsDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), DefaultMapSettingsDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), DefaultMapSettingsDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(DefaultMapSettingsDialog);
    } // setupUi

    void retranslateUi(QDialog *DefaultMapSettingsDialog)
    {
        DefaultMapSettingsDialog->setWindowTitle(QCoreApplication::translate("DefaultMapSettingsDialog", "VYM -  Set path for new maps", nullptr));
        label->setText(QString());
        autoCheckBox->setText(QCoreApplication::translate("DefaultMapSettingsDialog", "Automatic (based on theme)", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DefaultMapSettingsDialog: public Ui_DefaultMapSettingsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DEFAULT_2D_MAP_2D_SETTINGS_2D_DIALOG_H
