/****************************************************************************
** Meta object code from reading C++ file 'confluence-agent.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/confluence-agent.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'confluence-agent.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ConfluenceAgent_t {
    QByteArrayData data[21];
    char stringdata0[330];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ConfluenceAgent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ConfluenceAgent_t qt_meta_stringdata_ConfluenceAgent = {
    {
QT_MOC_LITERAL(0, 0, 15), // "ConfluenceAgent"
QT_MOC_LITERAL(1, 16, 10), // "foundUsers"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 21), // "QList<ConfluenceUser>"
QT_MOC_LITERAL(4, 50, 18), // "attachmentsSuccess"
QT_MOC_LITERAL(5, 69, 18), // "attachmentsFailure"
QT_MOC_LITERAL(6, 88, 18), // "pageSourceReceived"
QT_MOC_LITERAL(7, 107, 14), // "QNetworkReply*"
QT_MOC_LITERAL(8, 122, 5), // "reply"
QT_MOC_LITERAL(9, 128, 19), // "pageDetailsReceived"
QT_MOC_LITERAL(10, 148, 12), // "pageUploaded"
QT_MOC_LITERAL(11, 161, 16), // "userInfoReceived"
QT_MOC_LITERAL(12, 178, 23), // "attachmentsInfoReceived"
QT_MOC_LITERAL(13, 202, 17), // "attachmentCreated"
QT_MOC_LITERAL(14, 220, 17), // "attachmentUpdated"
QT_MOC_LITERAL(15, 238, 24), // "attachmentsUploadSuccess"
QT_MOC_LITERAL(16, 263, 24), // "attachmentsUploadFailure"
QT_MOC_LITERAL(17, 288, 7), // "timeout"
QT_MOC_LITERAL(18, 296, 9), // "sslErrors"
QT_MOC_LITERAL(19, 306, 16), // "QList<QSslError>"
QT_MOC_LITERAL(20, 323, 6) // "errors"

    },
    "ConfluenceAgent\0foundUsers\0\0"
    "QList<ConfluenceUser>\0attachmentsSuccess\0"
    "attachmentsFailure\0pageSourceReceived\0"
    "QNetworkReply*\0reply\0pageDetailsReceived\0"
    "pageUploaded\0userInfoReceived\0"
    "attachmentsInfoReceived\0attachmentCreated\0"
    "attachmentUpdated\0attachmentsUploadSuccess\0"
    "attachmentsUploadFailure\0timeout\0"
    "sslErrors\0QList<QSslError>\0errors"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ConfluenceAgent[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   84,    2, 0x06 /* Public */,
       4,    0,   87,    2, 0x06 /* Public */,
       5,    0,   88,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   89,    2, 0x08 /* Private */,
       9,    1,   92,    2, 0x08 /* Private */,
      10,    1,   95,    2, 0x08 /* Private */,
      11,    1,   98,    2, 0x08 /* Private */,
      12,    1,  101,    2, 0x08 /* Private */,
      13,    1,  104,    2, 0x08 /* Private */,
      14,    1,  107,    2, 0x08 /* Private */,
      15,    0,  110,    2, 0x0a /* Public */,
      16,    0,  111,    2, 0x0a /* Public */,
      17,    0,  112,    2, 0x08 /* Private */,
      18,    2,  113,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 19,    2,   20,

       0        // eod
};

void ConfluenceAgent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ConfluenceAgent *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->foundUsers((*reinterpret_cast< QList<ConfluenceUser>(*)>(_a[1]))); break;
        case 1: _t->attachmentsSuccess(); break;
        case 2: _t->attachmentsFailure(); break;
        case 3: _t->pageSourceReceived((*reinterpret_cast< QNetworkReply*(*)>(_a[1]))); break;
        case 4: _t->pageDetailsReceived((*reinterpret_cast< QNetworkReply*(*)>(_a[1]))); break;
        case 5: _t->pageUploaded((*reinterpret_cast< QNetworkReply*(*)>(_a[1]))); break;
        case 6: _t->userInfoReceived((*reinterpret_cast< QNetworkReply*(*)>(_a[1]))); break;
        case 7: _t->attachmentsInfoReceived((*reinterpret_cast< QNetworkReply*(*)>(_a[1]))); break;
        case 8: _t->attachmentCreated((*reinterpret_cast< QNetworkReply*(*)>(_a[1]))); break;
        case 9: _t->attachmentUpdated((*reinterpret_cast< QNetworkReply*(*)>(_a[1]))); break;
        case 10: _t->attachmentsUploadSuccess(); break;
        case 11: _t->attachmentsUploadFailure(); break;
        case 12: _t->timeout(); break;
        case 13: _t->sslErrors((*reinterpret_cast< QNetworkReply*(*)>(_a[1])),(*reinterpret_cast< const QList<QSslError>(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<ConfluenceUser> >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QSslError> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ConfluenceAgent::*)(QList<ConfluenceUser> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfluenceAgent::foundUsers)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ConfluenceAgent::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfluenceAgent::attachmentsSuccess)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ConfluenceAgent::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfluenceAgent::attachmentsFailure)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ConfluenceAgent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ConfluenceAgent.data,
    qt_meta_data_ConfluenceAgent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ConfluenceAgent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ConfluenceAgent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ConfluenceAgent.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ConfluenceAgent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void ConfluenceAgent::foundUsers(QList<ConfluenceUser> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ConfluenceAgent::attachmentsSuccess()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ConfluenceAgent::attachmentsFailure()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
