/********************************************************************************
** Form generated from reading UI file 'branchpropeditor.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BRANCHPROPEDITOR_H
#define UI_BRANCHPROPEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_BranchPropertyEditor
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *vboxLayout1;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QComboBox *frameTypeCombo;
    QSpacerItem *spacerItem;
    QLabel *framePaddingLabel;
    QLabel *frameBorderLabel;
    QSpinBox *framePaddingSpinBox;
    QSpinBox *frameWidthSpinBox;
    QCheckBox *includeChildrenCheckBox;
    QVBoxLayout *vboxLayout2;
    QGroupBox *colorGroupBox;
    QGridLayout *gridLayout1;
    QPushButton *framePenColorButton;
    QLabel *penColorLabelDesc;
    QPushButton *frameBrushColorButton;
    QLabel *brushColorLabelDesc;
    QSpacerItem *spacerItem1;
    QWidget *Layout;
    QVBoxLayout *vboxLayout3;
    QVBoxLayout *vboxLayout4;
    QCheckBox *incImgHor;
    QCheckBox *incImgVer;
    QCheckBox *childrenFreePositioning;
    QSpacerItem *spacerItem2;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QSpinBox *taskPrioDelta;
    QLabel *label;
    QSpacerItem *horizontalSpacer;
    QLabel *label_creation;
    QLineEdit *lineEditDateCreation;
    QLabel *label_modified;
    QLineEdit *lineEditDateModification;
    QLabel *label_sleep;
    QLineEdit *lineEditSleep;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_2;
    QVBoxLayout *vboxLayout5;
    QCheckBox *hideLinkIfUnselected;
    QSpacerItem *spacerItem3;
    QWidget *tab_3;
    QVBoxLayout *vboxLayout6;
    QTableView *attributeTableView;
    QHBoxLayout *hboxLayout;
    QPushButton *addAttributeButton;
    QPushButton *deleteAttributeButton;
    QSpacerItem *spacerItem4;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *BranchPropertyEditor)
    {
        if (BranchPropertyEditor->objectName().isEmpty())
            BranchPropertyEditor->setObjectName(QString::fromUtf8("BranchPropertyEditor"));
        BranchPropertyEditor->resize(325, 378);
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(BranchPropertyEditor->sizePolicy().hasHeightForWidth());
        BranchPropertyEditor->setSizePolicy(sizePolicy);
        vboxLayout = new QVBoxLayout(BranchPropertyEditor);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        tabWidget = new QTabWidget(BranchPropertyEditor);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        vboxLayout1 = new QVBoxLayout(tab);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        vboxLayout1->setContentsMargins(9, 9, 9, 9);
        groupBox = new QGroupBox(tab);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        frameTypeCombo = new QComboBox(groupBox);
        frameTypeCombo->addItem(QString());
        frameTypeCombo->addItem(QString());
        frameTypeCombo->addItem(QString());
        frameTypeCombo->addItem(QString());
        frameTypeCombo->addItem(QString());
        frameTypeCombo->setObjectName(QString::fromUtf8("frameTypeCombo"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frameTypeCombo->sizePolicy().hasHeightForWidth());
        frameTypeCombo->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(frameTypeCombo, 0, 0, 1, 4);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(spacerItem, 2, 3, 1, 1);

        framePaddingLabel = new QLabel(groupBox);
        framePaddingLabel->setObjectName(QString::fromUtf8("framePaddingLabel"));

        gridLayout->addWidget(framePaddingLabel, 2, 1, 1, 2);

        frameBorderLabel = new QLabel(groupBox);
        frameBorderLabel->setObjectName(QString::fromUtf8("frameBorderLabel"));

        gridLayout->addWidget(frameBorderLabel, 3, 1, 1, 2);

        framePaddingSpinBox = new QSpinBox(groupBox);
        framePaddingSpinBox->setObjectName(QString::fromUtf8("framePaddingSpinBox"));
        framePaddingSpinBox->setMaximum(50);

        gridLayout->addWidget(framePaddingSpinBox, 2, 0, 1, 1);

        frameWidthSpinBox = new QSpinBox(groupBox);
        frameWidthSpinBox->setObjectName(QString::fromUtf8("frameWidthSpinBox"));
        frameWidthSpinBox->setMinimum(1);
        frameWidthSpinBox->setMaximum(20);

        gridLayout->addWidget(frameWidthSpinBox, 3, 0, 1, 1);

        includeChildrenCheckBox = new QCheckBox(groupBox);
        includeChildrenCheckBox->setObjectName(QString::fromUtf8("includeChildrenCheckBox"));

        gridLayout->addWidget(includeChildrenCheckBox, 1, 0, 1, 3);


        vboxLayout1->addWidget(groupBox);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(6);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        colorGroupBox = new QGroupBox(tab);
        colorGroupBox->setObjectName(QString::fromUtf8("colorGroupBox"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(colorGroupBox->sizePolicy().hasHeightForWidth());
        colorGroupBox->setSizePolicy(sizePolicy2);
        gridLayout1 = new QGridLayout(colorGroupBox);
        gridLayout1->setSpacing(6);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        gridLayout1->setContentsMargins(7, 7, 7, 7);
        framePenColorButton = new QPushButton(colorGroupBox);
        framePenColorButton->setObjectName(QString::fromUtf8("framePenColorButton"));
        framePenColorButton->setMinimumSize(QSize(32, 32));
        framePenColorButton->setMaximumSize(QSize(20, 16777215));

        gridLayout1->addWidget(framePenColorButton, 0, 0, 1, 1);

        penColorLabelDesc = new QLabel(colorGroupBox);
        penColorLabelDesc->setObjectName(QString::fromUtf8("penColorLabelDesc"));
        sizePolicy2.setHeightForWidth(penColorLabelDesc->sizePolicy().hasHeightForWidth());
        penColorLabelDesc->setSizePolicy(sizePolicy2);

        gridLayout1->addWidget(penColorLabelDesc, 0, 1, 1, 1);

        frameBrushColorButton = new QPushButton(colorGroupBox);
        frameBrushColorButton->setObjectName(QString::fromUtf8("frameBrushColorButton"));
        frameBrushColorButton->setMinimumSize(QSize(32, 0));
        frameBrushColorButton->setMaximumSize(QSize(20, 16777215));

        gridLayout1->addWidget(frameBrushColorButton, 1, 0, 1, 1);

        brushColorLabelDesc = new QLabel(colorGroupBox);
        brushColorLabelDesc->setObjectName(QString::fromUtf8("brushColorLabelDesc"));

        gridLayout1->addWidget(brushColorLabelDesc, 1, 1, 1, 1);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout1->addItem(spacerItem1, 0, 2, 1, 1);


        vboxLayout2->addWidget(colorGroupBox);


        vboxLayout1->addLayout(vboxLayout2);

        tabWidget->addTab(tab, QString());
        Layout = new QWidget();
        Layout->setObjectName(QString::fromUtf8("Layout"));
        vboxLayout3 = new QVBoxLayout(Layout);
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        vboxLayout4 = new QVBoxLayout();
        vboxLayout4->setObjectName(QString::fromUtf8("vboxLayout4"));
        incImgHor = new QCheckBox(Layout);
        incImgHor->setObjectName(QString::fromUtf8("incImgHor"));

        vboxLayout4->addWidget(incImgHor);

        incImgVer = new QCheckBox(Layout);
        incImgVer->setObjectName(QString::fromUtf8("incImgVer"));

        vboxLayout4->addWidget(incImgVer);

        childrenFreePositioning = new QCheckBox(Layout);
        childrenFreePositioning->setObjectName(QString::fromUtf8("childrenFreePositioning"));

        vboxLayout4->addWidget(childrenFreePositioning);


        vboxLayout3->addLayout(vboxLayout4);

        spacerItem2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout3->addItem(spacerItem2);

        tabWidget->addTab(Layout, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName(QString::fromUtf8("tab_4"));
        verticalLayout = new QVBoxLayout(tab_4);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        taskPrioDelta = new QSpinBox(tab_4);
        taskPrioDelta->setObjectName(QString::fromUtf8("taskPrioDelta"));
        taskPrioDelta->setMinimum(-10000);
        taskPrioDelta->setMaximum(10000);

        horizontalLayout->addWidget(taskPrioDelta);

        label = new QLabel(tab_4);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout->addWidget(label);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        label_creation = new QLabel(tab_4);
        label_creation->setObjectName(QString::fromUtf8("label_creation"));

        verticalLayout->addWidget(label_creation);

        lineEditDateCreation = new QLineEdit(tab_4);
        lineEditDateCreation->setObjectName(QString::fromUtf8("lineEditDateCreation"));
        lineEditDateCreation->setEnabled(false);
        lineEditDateCreation->setReadOnly(true);

        verticalLayout->addWidget(lineEditDateCreation);

        label_modified = new QLabel(tab_4);
        label_modified->setObjectName(QString::fromUtf8("label_modified"));

        verticalLayout->addWidget(label_modified);

        lineEditDateModification = new QLineEdit(tab_4);
        lineEditDateModification->setObjectName(QString::fromUtf8("lineEditDateModification"));
        lineEditDateModification->setEnabled(false);
        lineEditDateModification->setReadOnly(true);

        verticalLayout->addWidget(lineEditDateModification);

        label_sleep = new QLabel(tab_4);
        label_sleep->setObjectName(QString::fromUtf8("label_sleep"));

        verticalLayout->addWidget(label_sleep);

        lineEditSleep = new QLineEdit(tab_4);
        lineEditSleep->setObjectName(QString::fromUtf8("lineEditSleep"));
        lineEditSleep->setEnabled(false);

        verticalLayout->addWidget(lineEditSleep);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        tabWidget->addTab(tab_4, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName(QString::fromUtf8("tab_2"));
        vboxLayout5 = new QVBoxLayout(tab_2);
        vboxLayout5->setObjectName(QString::fromUtf8("vboxLayout5"));
        hideLinkIfUnselected = new QCheckBox(tab_2);
        hideLinkIfUnselected->setObjectName(QString::fromUtf8("hideLinkIfUnselected"));

        vboxLayout5->addWidget(hideLinkIfUnselected);

        spacerItem3 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout5->addItem(spacerItem3);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName(QString::fromUtf8("tab_3"));
        vboxLayout6 = new QVBoxLayout(tab_3);
        vboxLayout6->setObjectName(QString::fromUtf8("vboxLayout6"));
        attributeTableView = new QTableView(tab_3);
        attributeTableView->setObjectName(QString::fromUtf8("attributeTableView"));

        vboxLayout6->addWidget(attributeTableView);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        addAttributeButton = new QPushButton(tab_3);
        addAttributeButton->setObjectName(QString::fromUtf8("addAttributeButton"));

        hboxLayout->addWidget(addAttributeButton);

        deleteAttributeButton = new QPushButton(tab_3);
        deleteAttributeButton->setObjectName(QString::fromUtf8("deleteAttributeButton"));

        hboxLayout->addWidget(deleteAttributeButton);

        spacerItem4 = new QSpacerItem(41, 31, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem4);


        vboxLayout6->addLayout(hboxLayout);

        tabWidget->addTab(tab_3, QString());

        vboxLayout->addWidget(tabWidget);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        penColorLabelDesc->setBuddy(framePenColorButton);
        brushColorLabelDesc->setBuddy(frameBrushColorButton);
#endif // QT_CONFIG(shortcut)

        retranslateUi(BranchPropertyEditor);

        tabWidget->setCurrentIndex(2);


        QMetaObject::connectSlotsByName(BranchPropertyEditor);
    } // setupUi

    void retranslateUi(QWidget *BranchPropertyEditor)
    {
        BranchPropertyEditor->setWindowTitle(QCoreApplication::translate("BranchPropertyEditor", "Branch Property Editor", nullptr));
        groupBox->setTitle(QCoreApplication::translate("BranchPropertyEditor", "Geometry", nullptr));
        frameTypeCombo->setItemText(0, QCoreApplication::translate("BranchPropertyEditor", "No Frame", nullptr));
        frameTypeCombo->setItemText(1, QCoreApplication::translate("BranchPropertyEditor", "Rectangle", nullptr));
        frameTypeCombo->setItemText(2, QCoreApplication::translate("BranchPropertyEditor", "Rounded Rectangle", nullptr));
        frameTypeCombo->setItemText(3, QCoreApplication::translate("BranchPropertyEditor", "Ellipse", nullptr));
        frameTypeCombo->setItemText(4, QCoreApplication::translate("BranchPropertyEditor", "Cloud", nullptr));

        framePaddingLabel->setText(QCoreApplication::translate("BranchPropertyEditor", "Padding", nullptr));
        frameBorderLabel->setText(QCoreApplication::translate("BranchPropertyEditor", "Borderline width", nullptr));
        includeChildrenCheckBox->setText(QCoreApplication::translate("BranchPropertyEditor", "Include children", nullptr));
        colorGroupBox->setTitle(QCoreApplication::translate("BranchPropertyEditor", "Colors", nullptr));
        framePenColorButton->setText(QString());
        penColorLabelDesc->setText(QCoreApplication::translate("BranchPropertyEditor", "Borderline color", nullptr));
        frameBrushColorButton->setText(QString());
        brushColorLabelDesc->setText(QCoreApplication::translate("BranchPropertyEditor", "Background color", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("BranchPropertyEditor", "Frame", nullptr));
        incImgHor->setText(QCoreApplication::translate("BranchPropertyEditor", "Include images horizontally", nullptr));
        incImgVer->setText(QCoreApplication::translate("BranchPropertyEditor", "Include images vertically", nullptr));
        childrenFreePositioning->setText(QCoreApplication::translate("BranchPropertyEditor", "Free positioning of children (experimental!)", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(Layout), QCoreApplication::translate("BranchPropertyEditor", "Layout", nullptr));
        label->setText(QCoreApplication::translate("BranchPropertyEditor", "Delta priority", nullptr));
        label_creation->setText(QCoreApplication::translate("BranchPropertyEditor", "Created:", nullptr));
        label_modified->setText(QCoreApplication::translate("BranchPropertyEditor", "Modified:", nullptr));
        label_sleep->setText(QCoreApplication::translate("BranchPropertyEditor", "Sleep:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("BranchPropertyEditor", "Task", nullptr));
        hideLinkIfUnselected->setText(QCoreApplication::translate("BranchPropertyEditor", "Hide link if unselected", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("BranchPropertyEditor", "Link", nullptr));
        addAttributeButton->setText(QCoreApplication::translate("BranchPropertyEditor", "+", nullptr));
        deleteAttributeButton->setText(QCoreApplication::translate("BranchPropertyEditor", "-", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("BranchPropertyEditor", "Attributes", nullptr));
    } // retranslateUi

};

namespace Ui {
    class BranchPropertyEditor: public Ui_BranchPropertyEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_BRANCHPROPEDITOR_H
