/********************************************************************************
** Form generated from reading UI file 'lineeditdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LINEEDITDIALOG_H
#define UI_LINEEDITDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_LineEditDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QLineEdit *lineEdit;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *cancelButton;
    QPushButton *okButton;

    void setupUi(QDialog *LineEditDialog)
    {
        if (LineEditDialog->objectName().isEmpty())
            LineEditDialog->setObjectName(QString::fromUtf8("LineEditDialog"));
        LineEditDialog->resize(545, 116);
#if QT_CONFIG(accessibility)
        LineEditDialog->setAccessibleName(QString::fromUtf8(""));
#endif // QT_CONFIG(accessibility)
        verticalLayout = new QVBoxLayout(LineEditDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        label = new QLabel(LineEditDialog);
        label->setObjectName(QString::fromUtf8("label"));
#if QT_CONFIG(accessibility)
        label->setAccessibleName(QString::fromUtf8(""));
#endif // QT_CONFIG(accessibility)

        verticalLayout->addWidget(label);

        lineEdit = new QLineEdit(LineEditDialog);
        lineEdit->setObjectName(QString::fromUtf8("lineEdit"));
#if QT_CONFIG(accessibility)
        lineEdit->setAccessibleName(QString::fromUtf8(""));
#endif // QT_CONFIG(accessibility)

        verticalLayout->addWidget(lineEdit);

        verticalSpacer = new QSpacerItem(20, 26, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(73, 31, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        cancelButton = new QPushButton(LineEditDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));
        cancelButton->setAutoDefault(false);

        hboxLayout->addWidget(cancelButton);

        okButton = new QPushButton(LineEditDialog);
        okButton->setObjectName(QString::fromUtf8("okButton"));

        hboxLayout->addWidget(okButton);


        verticalLayout->addLayout(hboxLayout);


        retranslateUi(LineEditDialog);
        QObject::connect(okButton, SIGNAL(clicked()), LineEditDialog, SLOT(accept()));
        QObject::connect(cancelButton, SIGNAL(clicked()), LineEditDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(LineEditDialog);
    } // setupUi

    void retranslateUi(QDialog *LineEditDialog)
    {
        LineEditDialog->setWindowTitle(QCoreApplication::translate("LineEditDialog", "Dialog", nullptr));
        label->setText(QCoreApplication::translate("LineEditDialog", "TextLabel", nullptr));
#if QT_CONFIG(accessibility)
        cancelButton->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        cancelButton->setText(QCoreApplication::translate("LineEditDialog", "Cancel", nullptr));
        okButton->setText(QCoreApplication::translate("LineEditDialog", "Ok", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LineEditDialog: public Ui_LineEditDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LINEEDITDIALOG_H
