/****************************************************************************
** Meta object code from reading C++ file 'branchpropeditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/branchpropeditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'branchpropeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_BranchPropertyEditor_t {
    QByteArrayData data[16];
    char stringdata0[327];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BranchPropertyEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BranchPropertyEditor_t qt_meta_stringdata_BranchPropertyEditor = {
    {
QT_MOC_LITERAL(0, 0, 20), // "BranchPropertyEditor"
QT_MOC_LITERAL(1, 21, 12), // "windowClosed"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 16), // "frameTypeChanged"
QT_MOC_LITERAL(4, 52, 20), // "framePenColorClicked"
QT_MOC_LITERAL(5, 73, 22), // "frameBrushColorClicked"
QT_MOC_LITERAL(6, 96, 19), // "framePaddingChanged"
QT_MOC_LITERAL(7, 116, 23), // "frameBorderWidthChanged"
QT_MOC_LITERAL(8, 140, 27), // "frameIncludeChildrenChanged"
QT_MOC_LITERAL(9, 168, 25), // "linkHideUnselectedChanged"
QT_MOC_LITERAL(10, 194, 16), // "incImgVerChanged"
QT_MOC_LITERAL(11, 211, 16), // "incImgHorChanged"
QT_MOC_LITERAL(12, 228, 30), // "childrenFreePositioningChanged"
QT_MOC_LITERAL(13, 259, 24), // "taskPriorityDeltaChanged"
QT_MOC_LITERAL(14, 284, 19), // "addAttributeClicked"
QT_MOC_LITERAL(15, 304, 22) // "deleteAttributeClicked"

    },
    "BranchPropertyEditor\0windowClosed\0\0"
    "frameTypeChanged\0framePenColorClicked\0"
    "frameBrushColorClicked\0framePaddingChanged\0"
    "frameBorderWidthChanged\0"
    "frameIncludeChildrenChanged\0"
    "linkHideUnselectedChanged\0incImgVerChanged\0"
    "incImgHorChanged\0childrenFreePositioningChanged\0"
    "taskPriorityDeltaChanged\0addAttributeClicked\0"
    "deleteAttributeClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BranchPropertyEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   84,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   85,    2, 0x08 /* Private */,
       4,    0,   88,    2, 0x08 /* Private */,
       5,    0,   89,    2, 0x08 /* Private */,
       6,    1,   90,    2, 0x08 /* Private */,
       7,    1,   93,    2, 0x08 /* Private */,
       8,    1,   96,    2, 0x08 /* Private */,
       9,    1,   99,    2, 0x08 /* Private */,
      10,    1,  102,    2, 0x08 /* Private */,
      11,    1,  105,    2, 0x08 /* Private */,
      12,    1,  108,    2, 0x08 /* Private */,
      13,    1,  111,    2, 0x08 /* Private */,
      14,    0,  114,    2, 0x08 /* Private */,
      15,    0,  115,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void BranchPropertyEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BranchPropertyEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->windowClosed(); break;
        case 1: _t->frameTypeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->framePenColorClicked(); break;
        case 3: _t->frameBrushColorClicked(); break;
        case 4: _t->framePaddingChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->frameBorderWidthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->frameIncludeChildrenChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->linkHideUnselectedChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->incImgVerChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->incImgHorChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->childrenFreePositioningChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->taskPriorityDeltaChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->addAttributeClicked(); break;
        case 13: _t->deleteAttributeClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (BranchPropertyEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BranchPropertyEditor::windowClosed)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject BranchPropertyEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_BranchPropertyEditor.data,
    qt_meta_data_BranchPropertyEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BranchPropertyEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BranchPropertyEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BranchPropertyEditor.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int BranchPropertyEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void BranchPropertyEditor::windowClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
