/****************************************************************************
** Meta object code from reading C++ file 'vymmodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/vymmodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vymmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_VymModel_t {
    QByteArrayData data[37];
    char stringdata0[466];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_VymModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_VymModel_t qt_meta_stringdata_VymModel = {
    {
QT_MOC_LITERAL(0, 0, 8), // "VymModel"
QT_MOC_LITERAL(1, 9, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 25, 29), // "org.insilmaril.vym.VymModel-h"
QT_MOC_LITERAL(3, 55, 17), // "sortFilterChanged"
QT_MOC_LITERAL(4, 73, 0), // ""
QT_MOC_LITERAL(5, 74, 9), // "expandAll"
QT_MOC_LITERAL(6, 84, 14), // "expandOneLevel"
QT_MOC_LITERAL(7, 99, 16), // "collapseOneLevel"
QT_MOC_LITERAL(8, 116, 18), // "collapseUnselected"
QT_MOC_LITERAL(9, 135, 13), // "updateQueries"
QT_MOC_LITERAL(10, 149, 9), // "VymModel*"
QT_MOC_LITERAL(11, 159, 1), // "m"
QT_MOC_LITERAL(12, 161, 12), // "updateLayout"
QT_MOC_LITERAL(13, 174, 11), // "noteChanged"
QT_MOC_LITERAL(14, 186, 11), // "QModelIndex"
QT_MOC_LITERAL(15, 198, 2), // "ix"
QT_MOC_LITERAL(16, 201, 14), // "newChildObject"
QT_MOC_LITERAL(17, 216, 13), // "showSelection"
QT_MOC_LITERAL(18, 230, 6), // "scaled"
QT_MOC_LITERAL(19, 237, 16), // "selectionChanged"
QT_MOC_LITERAL(20, 254, 14), // "QItemSelection"
QT_MOC_LITERAL(21, 269, 6), // "newsel"
QT_MOC_LITERAL(22, 276, 6), // "oldsel"
QT_MOC_LITERAL(23, 283, 8), // "autosave"
QT_MOC_LITERAL(24, 292, 11), // "fileChanged"
QT_MOC_LITERAL(25, 304, 16), // "updateTasksAlarm"
QT_MOC_LITERAL(26, 321, 5), // "force"
QT_MOC_LITERAL(27, 327, 14), // "updateJiraData"
QT_MOC_LITERAL(28, 342, 7), // "animate"
QT_MOC_LITERAL(29, 350, 9), // "newClient"
QT_MOC_LITERAL(30, 360, 8), // "sendData"
QT_MOC_LITERAL(31, 369, 1), // "s"
QT_MOC_LITERAL(32, 371, 8), // "readData"
QT_MOC_LITERAL(33, 380, 19), // "displayNetworkError"
QT_MOC_LITERAL(34, 400, 28), // "QAbstractSocket::SocketError"
QT_MOC_LITERAL(35, 429, 15), // "updateSelection"
QT_MOC_LITERAL(36, 445, 20) // "updateSlideSelection"

    },
    "VymModel\0D-Bus Interface\0"
    "org.insilmaril.vym.VymModel-h\0"
    "sortFilterChanged\0\0expandAll\0"
    "expandOneLevel\0collapseOneLevel\0"
    "collapseUnselected\0updateQueries\0"
    "VymModel*\0m\0updateLayout\0noteChanged\0"
    "QModelIndex\0ix\0newChildObject\0"
    "showSelection\0scaled\0selectionChanged\0"
    "QItemSelection\0newsel\0oldsel\0autosave\0"
    "fileChanged\0updateTasksAlarm\0force\0"
    "updateJiraData\0animate\0newClient\0"
    "sendData\0s\0readData\0displayNetworkError\0"
    "QAbstractSocket::SocketError\0"
    "updateSelection\0updateSlideSelection"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_VymModel[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      23,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,  131,    4, 0x06 /* Public */,
       5,    0,  134,    4, 0x06 /* Public */,
       6,    0,  135,    4, 0x06 /* Public */,
       7,    0,  136,    4, 0x06 /* Public */,
       8,    0,  137,    4, 0x06 /* Public */,
       9,    1,  138,    4, 0x06 /* Public */,
      12,    0,  141,    4, 0x06 /* Public */,
      13,    1,  142,    4, 0x06 /* Public */,
      16,    1,  145,    4, 0x06 /* Public */,
      17,    1,  148,    4, 0x06 /* Public */,
      19,    2,  151,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      23,    0,  156,    4, 0x08 /* Private */,
      24,    0,  157,    4, 0x08 /* Private */,
      25,    1,  158,    4, 0x08 /* Private */,
      25,    0,  161,    4, 0x28 /* Private | MethodCloned */,
      27,    1,  162,    4, 0x0a /* Public */,
      28,    0,  165,    4, 0x08 /* Private */,
      29,    0,  166,    4, 0x08 /* Private */,
      30,    1,  167,    4, 0x08 /* Private */,
      32,    0,  170,    4, 0x08 /* Private */,
      33,    1,  171,    4, 0x08 /* Private */,
      35,    2,  174,    4, 0x0a /* Public */,
      36,    2,  179,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, 0x80000000 | 20, 0x80000000 | 20,   21,   22,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QJsonObject,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   31,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 34,    4,
    QMetaType::Void, 0x80000000 | 20, 0x80000000 | 20,    4,    4,
    QMetaType::Void, 0x80000000 | 20, 0x80000000 | 20,    4,    4,

       0        // eod
};

void VymModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<VymModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sortFilterChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->expandAll(); break;
        case 2: _t->expandOneLevel(); break;
        case 3: _t->collapseOneLevel(); break;
        case 4: _t->collapseUnselected(); break;
        case 5: _t->updateQueries((*reinterpret_cast< VymModel*(*)>(_a[1]))); break;
        case 6: _t->updateLayout(); break;
        case 7: _t->noteChanged((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 8: _t->newChildObject((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 9: _t->showSelection((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->selectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 11: _t->autosave(); break;
        case 12: _t->fileChanged(); break;
        case 13: _t->updateTasksAlarm((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->updateTasksAlarm(); break;
        case 15: _t->updateJiraData((*reinterpret_cast< QJsonObject(*)>(_a[1]))); break;
        case 16: _t->animate(); break;
        case 17: _t->newClient(); break;
        case 18: _t->sendData((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 19: _t->readData(); break;
        case 20: _t->displayNetworkError((*reinterpret_cast< QAbstractSocket::SocketError(*)>(_a[1]))); break;
        case 21: _t->updateSelection((*reinterpret_cast< QItemSelection(*)>(_a[1])),(*reinterpret_cast< QItemSelection(*)>(_a[2]))); break;
        case 22: _t->updateSlideSelection((*reinterpret_cast< QItemSelection(*)>(_a[1])),(*reinterpret_cast< QItemSelection(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VymModel* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractSocket::SocketError >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (VymModel::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VymModel::sortFilterChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (VymModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VymModel::expandAll)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (VymModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VymModel::expandOneLevel)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (VymModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VymModel::collapseOneLevel)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (VymModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VymModel::collapseUnselected)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (VymModel::*)(VymModel * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VymModel::updateQueries)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (VymModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VymModel::updateLayout)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (VymModel::*)(QModelIndex );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VymModel::noteChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (VymModel::*)(QModelIndex );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VymModel::newChildObject)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (VymModel::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VymModel::showSelection)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (VymModel::*)(const QItemSelection & , const QItemSelection & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VymModel::selectionChanged)) {
                *result = 10;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject VymModel::staticMetaObject = { {
    QMetaObject::SuperData::link<TreeModel::staticMetaObject>(),
    qt_meta_stringdata_VymModel.data,
    qt_meta_data_VymModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *VymModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VymModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_VymModel.stringdata0))
        return static_cast<void*>(this);
    return TreeModel::qt_metacast(_clname);
}

int VymModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TreeModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void VymModel::sortFilterChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void VymModel::expandAll()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void VymModel::expandOneLevel()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void VymModel::collapseOneLevel()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void VymModel::collapseUnselected()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void VymModel::updateQueries(VymModel * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void VymModel::updateLayout()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void VymModel::noteChanged(QModelIndex _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void VymModel::newChildObject(QModelIndex _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void VymModel::showSelection(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void VymModel::selectionChanged(const QItemSelection & _t1, const QItemSelection & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
