/****************************************************************************
** Meta object code from reading C++ file 'mapeditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/mapeditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mapeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MapEditor_t {
    QByteArrayData data[26];
    char stringdata0[341];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MapEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MapEditor_t qt_meta_stringdata_MapEditor = {
    {
QT_MOC_LITERAL(0, 0, 9), // "MapEditor"
QT_MOC_LITERAL(1, 10, 16), // "selectionChanged"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 14), // "QItemSelection"
QT_MOC_LITERAL(4, 43, 7), // "panView"
QT_MOC_LITERAL(5, 51, 10), // "autoLayout"
QT_MOC_LITERAL(6, 62, 13), // "testFunction1"
QT_MOC_LITERAL(7, 76, 13), // "testFunction2"
QT_MOC_LITERAL(8, 90, 8), // "cursorUp"
QT_MOC_LITERAL(9, 99, 23), // "cursorUpToggleSelection"
QT_MOC_LITERAL(10, 123, 10), // "cursorDown"
QT_MOC_LITERAL(11, 134, 25), // "cursorDownToggleSelection"
QT_MOC_LITERAL(12, 160, 10), // "cursorLeft"
QT_MOC_LITERAL(13, 171, 11), // "cursorRight"
QT_MOC_LITERAL(14, 183, 11), // "cursorFirst"
QT_MOC_LITERAL(15, 195, 10), // "cursorLast"
QT_MOC_LITERAL(16, 206, 11), // "editHeading"
QT_MOC_LITERAL(17, 218, 19), // "editHeadingFinished"
QT_MOC_LITERAL(18, 238, 15), // "updateSelection"
QT_MOC_LITERAL(19, 254, 10), // "updateData"
QT_MOC_LITERAL(20, 265, 11), // "QModelIndex"
QT_MOC_LITERAL(21, 277, 22), // "togglePresentationMode"
QT_MOC_LITERAL(22, 300, 12), // "scrollBarPos"
QT_MOC_LITERAL(23, 313, 10), // "zoomFactor"
QT_MOC_LITERAL(24, 324, 5), // "angle"
QT_MOC_LITERAL(25, 330, 10) // "viewCenter"

    },
    "MapEditor\0selectionChanged\0\0QItemSelection\0"
    "panView\0autoLayout\0testFunction1\0"
    "testFunction2\0cursorUp\0cursorUpToggleSelection\0"
    "cursorDown\0cursorDownToggleSelection\0"
    "cursorLeft\0cursorRight\0cursorFirst\0"
    "cursorLast\0editHeading\0editHeadingFinished\0"
    "updateSelection\0updateData\0QModelIndex\0"
    "togglePresentationMode\0scrollBarPos\0"
    "zoomFactor\0angle\0viewCenter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MapEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       4,  132, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,  104,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  109,    2, 0x08 /* Private */,
       5,    0,  110,    2, 0x0a /* Public */,
       6,    0,  111,    2, 0x0a /* Public */,
       7,    0,  112,    2, 0x0a /* Public */,
       8,    0,  113,    2, 0x0a /* Public */,
       9,    0,  114,    2, 0x0a /* Public */,
      10,    0,  115,    2, 0x0a /* Public */,
      11,    0,  116,    2, 0x0a /* Public */,
      12,    0,  117,    2, 0x0a /* Public */,
      13,    0,  118,    2, 0x0a /* Public */,
      14,    0,  119,    2, 0x0a /* Public */,
      15,    0,  120,    2, 0x0a /* Public */,
      16,    0,  121,    2, 0x0a /* Public */,
      17,    0,  122,    2, 0x0a /* Public */,
      18,    2,  123,    2, 0x0a /* Public */,
      19,    1,  128,    2, 0x0a /* Public */,
      21,    0,  131,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    2,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    2,    2,
    QMetaType::Void, 0x80000000 | 20,    2,
    QMetaType::Void,

 // properties: name, type, flags
      22, QMetaType::QPointF, 0x00095103,
      23, QMetaType::QReal, 0x00095103,
      24, QMetaType::QReal, 0x00095103,
      25, QMetaType::QPointF, 0x00095103,

       0        // eod
};

void MapEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MapEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->selectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 1: _t->panView(); break;
        case 2: _t->autoLayout(); break;
        case 3: _t->testFunction1(); break;
        case 4: _t->testFunction2(); break;
        case 5: _t->cursorUp(); break;
        case 6: _t->cursorUpToggleSelection(); break;
        case 7: _t->cursorDown(); break;
        case 8: _t->cursorDownToggleSelection(); break;
        case 9: _t->cursorLeft(); break;
        case 10: _t->cursorRight(); break;
        case 11: _t->cursorFirst(); break;
        case 12: _t->cursorLast(); break;
        case 13: _t->editHeading(); break;
        case 14: _t->editHeadingFinished(); break;
        case 15: _t->updateSelection((*reinterpret_cast< QItemSelection(*)>(_a[1])),(*reinterpret_cast< QItemSelection(*)>(_a[2]))); break;
        case 16: _t->updateData((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 17: _t->togglePresentationMode(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MapEditor::*)(const QItemSelection & , const QItemSelection & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MapEditor::selectionChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<MapEditor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QPointF*>(_v) = _t->getScrollBarPos(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->getZoomFactor(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->getAngle(); break;
        case 3: *reinterpret_cast< QPointF*>(_v) = _t->getViewCenter(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<MapEditor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setScrollBarPos(*reinterpret_cast< QPointF*>(_v)); break;
        case 1: _t->setZoomFactor(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setAngle(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setViewCenter(*reinterpret_cast< QPointF*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject MapEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_meta_stringdata_MapEditor.data,
    qt_meta_data_MapEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MapEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MapEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MapEditor.stringdata0))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int MapEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void MapEditor::selectionChanged(const QItemSelection & _t1, const QItemSelection & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
