/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/mainwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Main_t {
    QByteArrayData data[254];
    char stringdata0[4128];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Main_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Main_t qt_meta_stringdata_Main = {
    {
QT_MOC_LITERAL(0, 0, 4), // "Main"
QT_MOC_LITERAL(1, 5, 7), // "fileNew"
QT_MOC_LITERAL(2, 13, 0), // ""
QT_MOC_LITERAL(3, 14, 11), // "fileNewCopy"
QT_MOC_LITERAL(4, 26, 11), // "addUserFlag"
QT_MOC_LITERAL(5, 38, 13), // "editorChanged"
QT_MOC_LITERAL(6, 52, 8), // "fileLoad"
QT_MOC_LITERAL(7, 61, 15), // "File::ErrorCode"
QT_MOC_LITERAL(8, 77, 8), // "LoadMode"
QT_MOC_LITERAL(9, 86, 8), // "FileType"
QT_MOC_LITERAL(10, 95, 5), // "ftype"
QT_MOC_LITERAL(11, 101, 15), // "fileSaveSession"
QT_MOC_LITERAL(12, 117, 18), // "fileRestoreSession"
QT_MOC_LITERAL(13, 136, 14), // "fileLoadRecent"
QT_MOC_LITERAL(14, 151, 12), // "addRecentMap"
QT_MOC_LITERAL(15, 164, 8), // "fileSave"
QT_MOC_LITERAL(16, 173, 9), // "VymModel*"
QT_MOC_LITERAL(17, 183, 8), // "SaveMode"
QT_MOC_LITERAL(18, 192, 10), // "fileSaveAs"
QT_MOC_LITERAL(19, 203, 17), // "fileSaveAsDefault"
QT_MOC_LITERAL(20, 221, 26), // "fileImportFirefoxBookmarks"
QT_MOC_LITERAL(21, 248, 18), // "fileImportFreemind"
QT_MOC_LITERAL(22, 267, 12), // "fileImportMM"
QT_MOC_LITERAL(23, 280, 13), // "fileImportDir"
QT_MOC_LITERAL(24, 294, 12), // "fileExportAO"
QT_MOC_LITERAL(25, 307, 15), // "fileExportASCII"
QT_MOC_LITERAL(26, 323, 20), // "fileExportASCIITasks"
QT_MOC_LITERAL(27, 344, 13), // "fileExportCSV"
QT_MOC_LITERAL(28, 358, 20), // "fileExportConfluence"
QT_MOC_LITERAL(29, 379, 26), // "fileExportFirefoxBookmarks"
QT_MOC_LITERAL(30, 406, 14), // "fileExportHTML"
QT_MOC_LITERAL(31, 421, 15), // "fileExportImage"
QT_MOC_LITERAL(32, 437, 17), // "fileExportImpress"
QT_MOC_LITERAL(33, 455, 15), // "fileExportLaTeX"
QT_MOC_LITERAL(34, 471, 18), // "fileExportMarkdown"
QT_MOC_LITERAL(35, 490, 17), // "fileExportOrgMode"
QT_MOC_LITERAL(36, 508, 13), // "fileExportPDF"
QT_MOC_LITERAL(37, 522, 13), // "fileExportSVG"
QT_MOC_LITERAL(38, 536, 21), // "fileExportTaskjuggler"
QT_MOC_LITERAL(39, 558, 13), // "fileExportXML"
QT_MOC_LITERAL(40, 572, 14), // "fileExportLast"
QT_MOC_LITERAL(41, 587, 12), // "fileCloseMap"
QT_MOC_LITERAL(42, 600, 1), // "i"
QT_MOC_LITERAL(43, 602, 9), // "filePrint"
QT_MOC_LITERAL(44, 612, 11), // "fileExitVYM"
QT_MOC_LITERAL(45, 624, 8), // "editUndo"
QT_MOC_LITERAL(46, 633, 8), // "editRedo"
QT_MOC_LITERAL(47, 642, 15), // "gotoHistoryStep"
QT_MOC_LITERAL(48, 658, 8), // "editCopy"
QT_MOC_LITERAL(49, 667, 9), // "editPaste"
QT_MOC_LITERAL(50, 677, 7), // "editCut"
QT_MOC_LITERAL(51, 685, 13), // "updateQueries"
QT_MOC_LITERAL(52, 699, 7), // "openURL"
QT_MOC_LITERAL(53, 707, 3), // "url"
QT_MOC_LITERAL(54, 711, 8), // "openTabs"
QT_MOC_LITERAL(55, 720, 11), // "editOpenURL"
QT_MOC_LITERAL(56, 732, 14), // "editOpenURLTab"
QT_MOC_LITERAL(57, 747, 26), // "editOpenMultipleVisURLTabs"
QT_MOC_LITERAL(58, 774, 14), // "ignoreScrolled"
QT_MOC_LITERAL(59, 789, 23), // "editOpenMultipleURLTabs"
QT_MOC_LITERAL(60, 813, 13), // "editNote2URLs"
QT_MOC_LITERAL(61, 827, 7), // "editURL"
QT_MOC_LITERAL(62, 835, 12), // "editLocalURL"
QT_MOC_LITERAL(63, 848, 15), // "editHeading2URL"
QT_MOC_LITERAL(64, 864, 18), // "getJiraDataSubtree"
QT_MOC_LITERAL(65, 883, 28), // "setHeadingConfluencePageName"
QT_MOC_LITERAL(66, 912, 17), // "getConfluenceUser"
QT_MOC_LITERAL(67, 930, 12), // "openVymLinks"
QT_MOC_LITERAL(68, 943, 10), // "background"
QT_MOC_LITERAL(69, 954, 11), // "editVymLink"
QT_MOC_LITERAL(70, 966, 24), // "editOpenMultipleVymLinks"
QT_MOC_LITERAL(71, 991, 11), // "editHeading"
QT_MOC_LITERAL(72, 1003, 19), // "editHeadingFinished"
QT_MOC_LITERAL(73, 1023, 1), // "m"
QT_MOC_LITERAL(74, 1025, 15), // "editOpenVymLink"
QT_MOC_LITERAL(75, 1041, 25), // "editOpenVymLinkBackground"
QT_MOC_LITERAL(76, 1067, 17), // "editDeleteVymLink"
QT_MOC_LITERAL(77, 1085, 20), // "editToggleHideExport"
QT_MOC_LITERAL(78, 1106, 14), // "editToggleTask"
QT_MOC_LITERAL(79, 1121, 19), // "editCycleTaskStatus"
QT_MOC_LITERAL(80, 1141, 22), // "editTaskResetDeltaPrio"
QT_MOC_LITERAL(81, 1164, 14), // "editTaskSleepN"
QT_MOC_LITERAL(82, 1179, 16), // "editAddTimestamp"
QT_MOC_LITERAL(83, 1196, 17), // "editMapProperties"
QT_MOC_LITERAL(84, 1214, 10), // "editMoveUp"
QT_MOC_LITERAL(85, 1225, 12), // "editMoveDown"
QT_MOC_LITERAL(86, 1238, 20), // "editMoveUpDiagonally"
QT_MOC_LITERAL(87, 1259, 22), // "editMoveDownDiagonally"
QT_MOC_LITERAL(88, 1282, 10), // "editDetach"
QT_MOC_LITERAL(89, 1293, 16), // "editSortChildren"
QT_MOC_LITERAL(90, 1310, 20), // "editSortBackChildren"
QT_MOC_LITERAL(91, 1331, 16), // "editToggleScroll"
QT_MOC_LITERAL(92, 1348, 13), // "editExpandAll"
QT_MOC_LITERAL(93, 1362, 18), // "editExpandOneLevel"
QT_MOC_LITERAL(94, 1381, 20), // "editCollapseOneLevel"
QT_MOC_LITERAL(95, 1402, 22), // "editCollapseUnselected"
QT_MOC_LITERAL(96, 1425, 20), // "editUnscrollChildren"
QT_MOC_LITERAL(97, 1446, 21), // "editGrowSelectionSize"
QT_MOC_LITERAL(98, 1468, 23), // "editShrinkSelectionSize"
QT_MOC_LITERAL(99, 1492, 22), // "editResetSelectionSize"
QT_MOC_LITERAL(100, 1515, 16), // "editAddAttribute"
QT_MOC_LITERAL(101, 1532, 16), // "editAddMapCenter"
QT_MOC_LITERAL(102, 1549, 13), // "editNewBranch"
QT_MOC_LITERAL(103, 1563, 19), // "editNewBranchBefore"
QT_MOC_LITERAL(104, 1583, 18), // "editNewBranchAbove"
QT_MOC_LITERAL(105, 1602, 18), // "editNewBranchBelow"
QT_MOC_LITERAL(106, 1621, 13), // "editImportAdd"
QT_MOC_LITERAL(107, 1635, 17), // "editImportReplace"
QT_MOC_LITERAL(108, 1653, 14), // "editSaveBranch"
QT_MOC_LITERAL(109, 1668, 22), // "editDeleteKeepChildren"
QT_MOC_LITERAL(110, 1691, 18), // "editDeleteChildren"
QT_MOC_LITERAL(111, 1710, 19), // "editDeleteSelection"
QT_MOC_LITERAL(112, 1730, 13), // "editLoadImage"
QT_MOC_LITERAL(113, 1744, 13), // "editSaveImage"
QT_MOC_LITERAL(114, 1758, 16), // "popupFollowXLink"
QT_MOC_LITERAL(115, 1775, 15), // "editFollowXLink"
QT_MOC_LITERAL(116, 1791, 8), // "QAction*"
QT_MOC_LITERAL(117, 1800, 13), // "editEditXLink"
QT_MOC_LITERAL(118, 1814, 18), // "initLinkedMapsMenu"
QT_MOC_LITERAL(119, 1833, 5), // "model"
QT_MOC_LITERAL(120, 1839, 6), // "QMenu*"
QT_MOC_LITERAL(121, 1846, 4), // "menu"
QT_MOC_LITERAL(122, 1851, 17), // "editGoToLinkedMap"
QT_MOC_LITERAL(123, 1869, 16), // "editToggleTarget"
QT_MOC_LITERAL(124, 1886, 15), // "initTargetsMenu"
QT_MOC_LITERAL(125, 1902, 14), // "editGoToTarget"
QT_MOC_LITERAL(126, 1917, 16), // "editMoveToTarget"
QT_MOC_LITERAL(127, 1934, 18), // "editSelectPrevious"
QT_MOC_LITERAL(128, 1953, 14), // "editSelectNext"
QT_MOC_LITERAL(129, 1968, 17), // "editSelectNothing"
QT_MOC_LITERAL(130, 1986, 24), // "editOpenFindResultWidget"
QT_MOC_LITERAL(131, 2011, 12), // "editFindNext"
QT_MOC_LITERAL(132, 2024, 1), // "s"
QT_MOC_LITERAL(133, 2026, 15), // "searchNotesFlag"
QT_MOC_LITERAL(134, 2042, 21), // "editFindDuplicateURLs"
QT_MOC_LITERAL(135, 2064, 16), // "selectQuickColor"
QT_MOC_LITERAL(136, 2081, 1), // "n"
QT_MOC_LITERAL(137, 2083, 13), // "setQuickColor"
QT_MOC_LITERAL(138, 2097, 3), // "col"
QT_MOC_LITERAL(139, 2101, 17), // "quickColorPressed"
QT_MOC_LITERAL(140, 2119, 15), // "formatPickColor"
QT_MOC_LITERAL(141, 2135, 15), // "getCurrentColor"
QT_MOC_LITERAL(142, 2151, 20), // "getCurrentColorIndex"
QT_MOC_LITERAL(143, 2172, 15), // "setCurrentColor"
QT_MOC_LITERAL(144, 2188, 17), // "formatColorBranch"
QT_MOC_LITERAL(145, 2206, 18), // "formatColorSubtree"
QT_MOC_LITERAL(146, 2225, 19), // "formatLinkStyleLine"
QT_MOC_LITERAL(147, 2245, 22), // "formatLinkStyleParabel"
QT_MOC_LITERAL(148, 2268, 23), // "formatLinkStylePolyLine"
QT_MOC_LITERAL(149, 2292, 26), // "formatLinkStylePolyParabel"
QT_MOC_LITERAL(150, 2319, 21), // "formatSelectBackColor"
QT_MOC_LITERAL(151, 2341, 21), // "formatSelectBackImage"
QT_MOC_LITERAL(152, 2363, 21), // "formatSelectLinkColor"
QT_MOC_LITERAL(153, 2385, 26), // "formatSelectSelectionColor"
QT_MOC_LITERAL(154, 2412, 16), // "formatSelectFont"
QT_MOC_LITERAL(155, 2429, 25), // "formatToggleLinkColorHint"
QT_MOC_LITERAL(156, 2455, 24), // "formatHideLinkUnselected"
QT_MOC_LITERAL(157, 2480, 13), // "viewZoomReset"
QT_MOC_LITERAL(158, 2494, 10), // "viewZoomIn"
QT_MOC_LITERAL(159, 2505, 11), // "viewZoomOut"
QT_MOC_LITERAL(160, 2517, 26), // "viewRotateCounterClockwise"
QT_MOC_LITERAL(161, 2544, 19), // "viewRotateClockwise"
QT_MOC_LITERAL(162, 2564, 10), // "viewCenter"
QT_MOC_LITERAL(163, 2575, 16), // "viewCenterScaled"
QT_MOC_LITERAL(164, 2592, 18), // "networkStartServer"
QT_MOC_LITERAL(165, 2611, 14), // "networkConnect"
QT_MOC_LITERAL(166, 2626, 16), // "downloadFinished"
QT_MOC_LITERAL(167, 2643, 11), // "settingsPDF"
QT_MOC_LITERAL(168, 2655, 11), // "settingsURL"
QT_MOC_LITERAL(169, 2667, 15), // "settingsZipTool"
QT_MOC_LITERAL(170, 2683, 17), // "settingsMacroPath"
QT_MOC_LITERAL(171, 2701, 18), // "settingsUndoLevels"
QT_MOC_LITERAL(172, 2720, 22), // "settingsDefaultMapPath"
QT_MOC_LITERAL(173, 2743, 20), // "settingsAutosaveTime"
QT_MOC_LITERAL(174, 2764, 24), // "settingsDefaultMapAuthor"
QT_MOC_LITERAL(175, 2789, 17), // "settingsDarkTheme"
QT_MOC_LITERAL(176, 2807, 29), // "settingsShowParentsLevelTasks"
QT_MOC_LITERAL(177, 2837, 35), // "settingsShowParentsLevelFindR..."
QT_MOC_LITERAL(178, 2873, 24), // "settingsToggleAutoLayout"
QT_MOC_LITERAL(179, 2898, 29), // "settingsToggleWriteBackupFile"
QT_MOC_LITERAL(180, 2928, 23), // "settingsToggleAnimation"
QT_MOC_LITERAL(181, 2952, 23), // "settingsToggleDownloads"
QT_MOC_LITERAL(182, 2976, 18), // "settingsConfluence"
QT_MOC_LITERAL(183, 2995, 12), // "settingsJIRA"
QT_MOC_LITERAL(184, 3008, 22), // "windowToggleNoteEditor"
QT_MOC_LITERAL(185, 3031, 22), // "windowToggleTreeEditor"
QT_MOC_LITERAL(186, 3054, 22), // "windowToggleTaskEditor"
QT_MOC_LITERAL(187, 3077, 23), // "windowToggleSlideEditor"
QT_MOC_LITERAL(188, 3101, 24), // "windowToggleScriptEditor"
QT_MOC_LITERAL(189, 3126, 24), // "windowToggleScriptOutput"
QT_MOC_LITERAL(190, 3151, 19), // "windowToggleHistory"
QT_MOC_LITERAL(191, 3171, 20), // "windowToggleProperty"
QT_MOC_LITERAL(192, 3192, 23), // "windowShowHeadingEditor"
QT_MOC_LITERAL(193, 3216, 25), // "windowToggleHeadingEditor"
QT_MOC_LITERAL(194, 3242, 13), // "updateHistory"
QT_MOC_LITERAL(195, 3256, 15), // "SimpleSettings&"
QT_MOC_LITERAL(196, 3272, 21), // "windowToggleAntiAlias"
QT_MOC_LITERAL(197, 3294, 9), // "isAliased"
QT_MOC_LITERAL(198, 3304, 24), // "hasSmoothPixmapTransform"
QT_MOC_LITERAL(199, 3329, 24), // "windowToggleSmoothPixmap"
QT_MOC_LITERAL(200, 3354, 17), // "clearScriptOutput"
QT_MOC_LITERAL(201, 3372, 13), // "updateHeading"
QT_MOC_LITERAL(202, 3386, 7), // "VymText"
QT_MOC_LITERAL(203, 3394, 2), // "vt"
QT_MOC_LITERAL(204, 3397, 14), // "updateNoteText"
QT_MOC_LITERAL(205, 3412, 16), // "updateNoteEditor"
QT_MOC_LITERAL(206, 3429, 9), // "TreeItem*"
QT_MOC_LITERAL(207, 3439, 2), // "ti"
QT_MOC_LITERAL(208, 3442, 19), // "updateHeadingEditor"
QT_MOC_LITERAL(209, 3462, 11), // "BranchItem*"
QT_MOC_LITERAL(210, 3474, 2), // "bi"
QT_MOC_LITERAL(211, 3477, 18), // "selectInNoteEditor"
QT_MOC_LITERAL(212, 3496, 17), // "setFocusMapEditor"
QT_MOC_LITERAL(213, 3514, 15), // "changeSelection"
QT_MOC_LITERAL(214, 3530, 14), // "QItemSelection"
QT_MOC_LITERAL(215, 3545, 6), // "newSel"
QT_MOC_LITERAL(216, 3552, 6), // "delSel"
QT_MOC_LITERAL(217, 3559, 22), // "updateDockWidgetTitles"
QT_MOC_LITERAL(218, 3582, 13), // "updateActions"
QT_MOC_LITERAL(219, 3596, 10), // "getModMode"
QT_MOC_LITERAL(220, 3607, 7), // "ModMode"
QT_MOC_LITERAL(221, 3615, 19), // "autoSelectNewBranch"
QT_MOC_LITERAL(222, 3635, 9), // "runScript"
QT_MOC_LITERAL(223, 3645, 22), // "getCurrentModelWrapper"
QT_MOC_LITERAL(224, 3668, 10), // "gotoWindow"
QT_MOC_LITERAL(225, 3679, 16), // "windowNextEditor"
QT_MOC_LITERAL(226, 3696, 20), // "windowPreviousEditor"
QT_MOC_LITERAL(227, 3717, 9), // "nextSlide"
QT_MOC_LITERAL(228, 3727, 13), // "previousSlide"
QT_MOC_LITERAL(229, 3741, 11), // "flagChanged"
QT_MOC_LITERAL(230, 3753, 13), // "testFunction1"
QT_MOC_LITERAL(231, 3767, 13), // "testFunction2"
QT_MOC_LITERAL(232, 3781, 12), // "toggleWinter"
QT_MOC_LITERAL(233, 3794, 16), // "toggleHideExport"
QT_MOC_LITERAL(234, 3811, 11), // "testCommand"
QT_MOC_LITERAL(235, 3823, 7), // "helpDoc"
QT_MOC_LITERAL(236, 3831, 8), // "helpDemo"
QT_MOC_LITERAL(237, 3840, 13), // "helpShortcuts"
QT_MOC_LITERAL(238, 3854, 10), // "helpMacros"
QT_MOC_LITERAL(239, 3865, 21), // "helpScriptingCommands"
QT_MOC_LITERAL(240, 3887, 13), // "helpDebugInfo"
QT_MOC_LITERAL(241, 3901, 9), // "helpAbout"
QT_MOC_LITERAL(242, 3911, 11), // "helpAboutQT"
QT_MOC_LITERAL(243, 3923, 9), // "callMacro"
QT_MOC_LITERAL(244, 3933, 28), // "downloadReleaseNotesFinished"
QT_MOC_LITERAL(245, 3962, 17), // "checkReleaseNotes"
QT_MOC_LITERAL(246, 3980, 16), // "downloadsEnabled"
QT_MOC_LITERAL(247, 3997, 13), // "userTriggered"
QT_MOC_LITERAL(248, 4011, 23), // "downloadUpdatesFinished"
QT_MOC_LITERAL(249, 4035, 26), // "downloadUpdatesFinishedInt"
QT_MOC_LITERAL(250, 4062, 15), // "downloadUpdates"
QT_MOC_LITERAL(251, 4078, 12), // "checkUpdates"
QT_MOC_LITERAL(252, 4091, 13), // "escapePressed"
QT_MOC_LITERAL(253, 4105, 22) // "togglePresentationMode"

    },
    "Main\0fileNew\0\0fileNewCopy\0addUserFlag\0"
    "editorChanged\0fileLoad\0File::ErrorCode\0"
    "LoadMode\0FileType\0ftype\0fileSaveSession\0"
    "fileRestoreSession\0fileLoadRecent\0"
    "addRecentMap\0fileSave\0VymModel*\0"
    "SaveMode\0fileSaveAs\0fileSaveAsDefault\0"
    "fileImportFirefoxBookmarks\0"
    "fileImportFreemind\0fileImportMM\0"
    "fileImportDir\0fileExportAO\0fileExportASCII\0"
    "fileExportASCIITasks\0fileExportCSV\0"
    "fileExportConfluence\0fileExportFirefoxBookmarks\0"
    "fileExportHTML\0fileExportImage\0"
    "fileExportImpress\0fileExportLaTeX\0"
    "fileExportMarkdown\0fileExportOrgMode\0"
    "fileExportPDF\0fileExportSVG\0"
    "fileExportTaskjuggler\0fileExportXML\0"
    "fileExportLast\0fileCloseMap\0i\0filePrint\0"
    "fileExitVYM\0editUndo\0editRedo\0"
    "gotoHistoryStep\0editCopy\0editPaste\0"
    "editCut\0updateQueries\0openURL\0url\0"
    "openTabs\0editOpenURL\0editOpenURLTab\0"
    "editOpenMultipleVisURLTabs\0ignoreScrolled\0"
    "editOpenMultipleURLTabs\0editNote2URLs\0"
    "editURL\0editLocalURL\0editHeading2URL\0"
    "getJiraDataSubtree\0setHeadingConfluencePageName\0"
    "getConfluenceUser\0openVymLinks\0"
    "background\0editVymLink\0editOpenMultipleVymLinks\0"
    "editHeading\0editHeadingFinished\0m\0"
    "editOpenVymLink\0editOpenVymLinkBackground\0"
    "editDeleteVymLink\0editToggleHideExport\0"
    "editToggleTask\0editCycleTaskStatus\0"
    "editTaskResetDeltaPrio\0editTaskSleepN\0"
    "editAddTimestamp\0editMapProperties\0"
    "editMoveUp\0editMoveDown\0editMoveUpDiagonally\0"
    "editMoveDownDiagonally\0editDetach\0"
    "editSortChildren\0editSortBackChildren\0"
    "editToggleScroll\0editExpandAll\0"
    "editExpandOneLevel\0editCollapseOneLevel\0"
    "editCollapseUnselected\0editUnscrollChildren\0"
    "editGrowSelectionSize\0editShrinkSelectionSize\0"
    "editResetSelectionSize\0editAddAttribute\0"
    "editAddMapCenter\0editNewBranch\0"
    "editNewBranchBefore\0editNewBranchAbove\0"
    "editNewBranchBelow\0editImportAdd\0"
    "editImportReplace\0editSaveBranch\0"
    "editDeleteKeepChildren\0editDeleteChildren\0"
    "editDeleteSelection\0editLoadImage\0"
    "editSaveImage\0popupFollowXLink\0"
    "editFollowXLink\0QAction*\0editEditXLink\0"
    "initLinkedMapsMenu\0model\0QMenu*\0menu\0"
    "editGoToLinkedMap\0editToggleTarget\0"
    "initTargetsMenu\0editGoToTarget\0"
    "editMoveToTarget\0editSelectPrevious\0"
    "editSelectNext\0editSelectNothing\0"
    "editOpenFindResultWidget\0editFindNext\0"
    "s\0searchNotesFlag\0editFindDuplicateURLs\0"
    "selectQuickColor\0n\0setQuickColor\0col\0"
    "quickColorPressed\0formatPickColor\0"
    "getCurrentColor\0getCurrentColorIndex\0"
    "setCurrentColor\0formatColorBranch\0"
    "formatColorSubtree\0formatLinkStyleLine\0"
    "formatLinkStyleParabel\0formatLinkStylePolyLine\0"
    "formatLinkStylePolyParabel\0"
    "formatSelectBackColor\0formatSelectBackImage\0"
    "formatSelectLinkColor\0formatSelectSelectionColor\0"
    "formatSelectFont\0formatToggleLinkColorHint\0"
    "formatHideLinkUnselected\0viewZoomReset\0"
    "viewZoomIn\0viewZoomOut\0"
    "viewRotateCounterClockwise\0"
    "viewRotateClockwise\0viewCenter\0"
    "viewCenterScaled\0networkStartServer\0"
    "networkConnect\0downloadFinished\0"
    "settingsPDF\0settingsURL\0settingsZipTool\0"
    "settingsMacroPath\0settingsUndoLevels\0"
    "settingsDefaultMapPath\0settingsAutosaveTime\0"
    "settingsDefaultMapAuthor\0settingsDarkTheme\0"
    "settingsShowParentsLevelTasks\0"
    "settingsShowParentsLevelFindResults\0"
    "settingsToggleAutoLayout\0"
    "settingsToggleWriteBackupFile\0"
    "settingsToggleAnimation\0settingsToggleDownloads\0"
    "settingsConfluence\0settingsJIRA\0"
    "windowToggleNoteEditor\0windowToggleTreeEditor\0"
    "windowToggleTaskEditor\0windowToggleSlideEditor\0"
    "windowToggleScriptEditor\0"
    "windowToggleScriptOutput\0windowToggleHistory\0"
    "windowToggleProperty\0windowShowHeadingEditor\0"
    "windowToggleHeadingEditor\0updateHistory\0"
    "SimpleSettings&\0windowToggleAntiAlias\0"
    "isAliased\0hasSmoothPixmapTransform\0"
    "windowToggleSmoothPixmap\0clearScriptOutput\0"
    "updateHeading\0VymText\0vt\0updateNoteText\0"
    "updateNoteEditor\0TreeItem*\0ti\0"
    "updateHeadingEditor\0BranchItem*\0bi\0"
    "selectInNoteEditor\0setFocusMapEditor\0"
    "changeSelection\0QItemSelection\0newSel\0"
    "delSel\0updateDockWidgetTitles\0"
    "updateActions\0getModMode\0ModMode\0"
    "autoSelectNewBranch\0runScript\0"
    "getCurrentModelWrapper\0gotoWindow\0"
    "windowNextEditor\0windowPreviousEditor\0"
    "nextSlide\0previousSlide\0flagChanged\0"
    "testFunction1\0testFunction2\0toggleWinter\0"
    "toggleHideExport\0testCommand\0helpDoc\0"
    "helpDemo\0helpShortcuts\0helpMacros\0"
    "helpScriptingCommands\0helpDebugInfo\0"
    "helpAbout\0helpAboutQT\0callMacro\0"
    "downloadReleaseNotesFinished\0"
    "checkReleaseNotes\0downloadsEnabled\0"
    "userTriggered\0downloadUpdatesFinished\0"
    "downloadUpdatesFinishedInt\0downloadUpdates\0"
    "checkUpdates\0escapePressed\0"
    "togglePresentationMode"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Main[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     233,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0, 1179,    2, 0x0a /* Public */,
       3,    0, 1180,    2, 0x0a /* Public */,
       4,    0, 1181,    2, 0x0a /* Public */,
       5,    0, 1182,    2, 0x08 /* Private */,
       6,    3, 1183,    2, 0x0a /* Public */,
       6,    1, 1190,    2, 0x0a /* Public */,
       6,    0, 1193,    2, 0x08 /* Private */,
      11,    0, 1194,    2, 0x08 /* Private */,
      12,    0, 1195,    2, 0x0a /* Public */,
      13,    0, 1196,    2, 0x08 /* Private */,
      14,    1, 1197,    2, 0x08 /* Private */,
      15,    2, 1200,    2, 0x08 /* Private */,
      15,    0, 1205,    2, 0x08 /* Private */,
      15,    1, 1206,    2, 0x0a /* Public */,
      18,    0, 1209,    2, 0x08 /* Private */,
      18,    1, 1210,    2, 0x08 /* Private */,
      19,    0, 1213,    2, 0x08 /* Private */,
      20,    0, 1214,    2, 0x08 /* Private */,
      21,    0, 1215,    2, 0x08 /* Private */,
      22,    0, 1216,    2, 0x08 /* Private */,
      23,    0, 1217,    2, 0x08 /* Private */,
      24,    0, 1218,    2, 0x08 /* Private */,
      25,    0, 1219,    2, 0x08 /* Private */,
      26,    0, 1220,    2, 0x08 /* Private */,
      27,    0, 1221,    2, 0x08 /* Private */,
      28,    0, 1222,    2, 0x08 /* Private */,
      29,    0, 1223,    2, 0x08 /* Private */,
      30,    0, 1224,    2, 0x08 /* Private */,
      31,    0, 1225,    2, 0x08 /* Private */,
      32,    0, 1226,    2, 0x08 /* Private */,
      33,    0, 1227,    2, 0x08 /* Private */,
      34,    0, 1228,    2, 0x08 /* Private */,
      35,    0, 1229,    2, 0x08 /* Private */,
      36,    0, 1230,    2, 0x08 /* Private */,
      37,    0, 1231,    2, 0x08 /* Private */,
      38,    0, 1232,    2, 0x08 /* Private */,
      39,    0, 1233,    2, 0x08 /* Private */,
      40,    0, 1234,    2, 0x08 /* Private */,
      41,    1, 1235,    2, 0x08 /* Private */,
      41,    0, 1238,    2, 0x28 /* Private | MethodCloned */,
      43,    0, 1239,    2, 0x08 /* Private */,
      44,    0, 1240,    2, 0x08 /* Private */,
      45,    0, 1241,    2, 0x0a /* Public */,
      46,    0, 1242,    2, 0x0a /* Public */,
      47,    1, 1243,    2, 0x0a /* Public */,
      48,    0, 1246,    2, 0x08 /* Private */,
      49,    0, 1247,    2, 0x08 /* Private */,
      50,    0, 1248,    2, 0x08 /* Private */,
      51,    1, 1249,    2, 0x0a /* Public */,
      52,    1, 1252,    2, 0x0a /* Public */,
      54,    1, 1255,    2, 0x0a /* Public */,
      55,    0, 1258,    2, 0x0a /* Public */,
      56,    0, 1259,    2, 0x0a /* Public */,
      57,    1, 1260,    2, 0x08 /* Private */,
      57,    0, 1263,    2, 0x28 /* Private | MethodCloned */,
      59,    0, 1264,    2, 0x08 /* Private */,
      60,    0, 1265,    2, 0x08 /* Private */,
      61,    0, 1266,    2, 0x08 /* Private */,
      62,    0, 1267,    2, 0x08 /* Private */,
      63,    0, 1268,    2, 0x08 /* Private */,
      64,    0, 1269,    2, 0x08 /* Private */,
      65,    0, 1270,    2, 0x08 /* Private */,
      66,    0, 1271,    2, 0x08 /* Private */,
      67,    2, 1272,    2, 0x08 /* Private */,
      67,    1, 1277,    2, 0x28 /* Private | MethodCloned */,
      69,    0, 1280,    2, 0x08 /* Private */,
      70,    0, 1281,    2, 0x08 /* Private */,
      71,    0, 1282,    2, 0x0a /* Public */,
      72,    1, 1283,    2, 0x0a /* Public */,
      74,    1, 1286,    2, 0x0a /* Public */,
      74,    0, 1289,    2, 0x2a /* Public | MethodCloned */,
      75,    0, 1290,    2, 0x0a /* Public */,
      76,    0, 1291,    2, 0x08 /* Private */,
      77,    0, 1292,    2, 0x08 /* Private */,
      78,    0, 1293,    2, 0x08 /* Private */,
      79,    0, 1294,    2, 0x08 /* Private */,
      80,    0, 1295,    2, 0x08 /* Private */,
      81,    0, 1296,    2, 0x08 /* Private */,
      82,    0, 1297,    2, 0x08 /* Private */,
      83,    0, 1298,    2, 0x08 /* Private */,
      84,    0, 1299,    2, 0x08 /* Private */,
      85,    0, 1300,    2, 0x08 /* Private */,
      86,    0, 1301,    2, 0x08 /* Private */,
      87,    0, 1302,    2, 0x08 /* Private */,
      88,    0, 1303,    2, 0x08 /* Private */,
      89,    0, 1304,    2, 0x08 /* Private */,
      90,    0, 1305,    2, 0x08 /* Private */,
      91,    0, 1306,    2, 0x08 /* Private */,
      92,    0, 1307,    2, 0x08 /* Private */,
      93,    0, 1308,    2, 0x08 /* Private */,
      94,    0, 1309,    2, 0x08 /* Private */,
      95,    0, 1310,    2, 0x08 /* Private */,
      96,    0, 1311,    2, 0x08 /* Private */,
      97,    0, 1312,    2, 0x08 /* Private */,
      98,    0, 1313,    2, 0x08 /* Private */,
      99,    0, 1314,    2, 0x08 /* Private */,
     100,    0, 1315,    2, 0x08 /* Private */,
     101,    0, 1316,    2, 0x08 /* Private */,
     102,    0, 1317,    2, 0x08 /* Private */,
     103,    0, 1318,    2, 0x08 /* Private */,
     104,    0, 1319,    2, 0x08 /* Private */,
     105,    0, 1320,    2, 0x08 /* Private */,
     106,    0, 1321,    2, 0x08 /* Private */,
     107,    0, 1322,    2, 0x08 /* Private */,
     108,    0, 1323,    2, 0x08 /* Private */,
     109,    0, 1324,    2, 0x08 /* Private */,
     110,    0, 1325,    2, 0x08 /* Private */,
     111,    0, 1326,    2, 0x08 /* Private */,
     112,    0, 1327,    2, 0x08 /* Private */,
     113,    0, 1328,    2, 0x08 /* Private */,
     114,    0, 1329,    2, 0x08 /* Private */,
     115,    1, 1330,    2, 0x08 /* Private */,
     117,    1, 1333,    2, 0x08 /* Private */,
     118,    2, 1336,    2, 0x08 /* Private */,
     122,    0, 1341,    2, 0x0a /* Public */,
     123,    0, 1342,    2, 0x08 /* Private */,
     124,    2, 1343,    2, 0x08 /* Private */,
     125,    0, 1348,    2, 0x08 /* Private */,
     126,    0, 1349,    2, 0x08 /* Private */,
     127,    0, 1350,    2, 0x08 /* Private */,
     128,    0, 1351,    2, 0x08 /* Private */,
     129,    0, 1352,    2, 0x08 /* Private */,
     130,    0, 1353,    2, 0x08 /* Private */,
     131,    2, 1354,    2, 0x08 /* Private */,
     134,    0, 1359,    2, 0x08 /* Private */,
     135,    1, 1360,    2, 0x0a /* Public */,
     137,    1, 1363,    2, 0x0a /* Public */,
     139,    0, 1366,    2, 0x0a /* Public */,
     140,    0, 1367,    2, 0x0a /* Public */,
     141,    0, 1368,    2, 0x0a /* Public */,
     142,    0, 1369,    2, 0x0a /* Public */,
     143,    1, 1370,    2, 0x0a /* Public */,
     144,    0, 1373,    2, 0x08 /* Private */,
     145,    0, 1374,    2, 0x08 /* Private */,
     146,    0, 1375,    2, 0x08 /* Private */,
     147,    0, 1376,    2, 0x08 /* Private */,
     148,    0, 1377,    2, 0x08 /* Private */,
     149,    0, 1378,    2, 0x08 /* Private */,
     150,    0, 1379,    2, 0x08 /* Private */,
     151,    0, 1380,    2, 0x08 /* Private */,
     152,    0, 1381,    2, 0x08 /* Private */,
     153,    0, 1382,    2, 0x08 /* Private */,
     154,    0, 1383,    2, 0x08 /* Private */,
     155,    0, 1384,    2, 0x08 /* Private */,
     156,    0, 1385,    2, 0x08 /* Private */,
     157,    0, 1386,    2, 0x0a /* Public */,
     158,    0, 1387,    2, 0x0a /* Public */,
     159,    0, 1388,    2, 0x0a /* Public */,
     160,    0, 1389,    2, 0x0a /* Public */,
     161,    0, 1390,    2, 0x0a /* Public */,
     162,    0, 1391,    2, 0x0a /* Public */,
     163,    0, 1392,    2, 0x0a /* Public */,
     164,    0, 1393,    2, 0x0a /* Public */,
     165,    0, 1394,    2, 0x0a /* Public */,
     166,    0, 1395,    2, 0x0a /* Public */,
     167,    0, 1396,    2, 0x0a /* Public */,
     168,    0, 1397,    2, 0x0a /* Public */,
     169,    0, 1398,    2, 0x0a /* Public */,
     170,    0, 1399,    2, 0x0a /* Public */,
     171,    0, 1400,    2, 0x0a /* Public */,
     172,    0, 1401,    2, 0x0a /* Public */,
     173,    0, 1402,    2, 0x0a /* Public */,
     174,    0, 1403,    2, 0x0a /* Public */,
     175,    0, 1404,    2, 0x0a /* Public */,
     176,    0, 1405,    2, 0x0a /* Public */,
     177,    0, 1406,    2, 0x0a /* Public */,
     178,    0, 1407,    2, 0x0a /* Public */,
     179,    0, 1408,    2, 0x0a /* Public */,
     180,    0, 1409,    2, 0x0a /* Public */,
     181,    0, 1410,    2, 0x0a /* Public */,
     182,    0, 1411,    2, 0x0a /* Public */,
     183,    0, 1412,    2, 0x0a /* Public */,
     184,    0, 1413,    2, 0x0a /* Public */,
     185,    0, 1414,    2, 0x0a /* Public */,
     186,    0, 1415,    2, 0x0a /* Public */,
     187,    0, 1416,    2, 0x0a /* Public */,
     188,    0, 1417,    2, 0x0a /* Public */,
     189,    0, 1418,    2, 0x0a /* Public */,
     190,    0, 1419,    2, 0x0a /* Public */,
     191,    0, 1420,    2, 0x0a /* Public */,
     192,    0, 1421,    2, 0x0a /* Public */,
     193,    0, 1422,    2, 0x0a /* Public */,
     194,    1, 1423,    2, 0x0a /* Public */,
     196,    0, 1426,    2, 0x0a /* Public */,
     197,    0, 1427,    2, 0x0a /* Public */,
     198,    0, 1428,    2, 0x0a /* Public */,
     199,    0, 1429,    2, 0x0a /* Public */,
     200,    0, 1430,    2, 0x0a /* Public */,
     201,    1, 1431,    2, 0x0a /* Public */,
     204,    1, 1434,    2, 0x0a /* Public */,
     205,    1, 1437,    2, 0x0a /* Public */,
     208,    1, 1440,    2, 0x0a /* Public */,
     208,    0, 1443,    2, 0x2a /* Public | MethodCloned */,
     211,    2, 1444,    2, 0x0a /* Public */,
     212,    0, 1449,    2, 0x0a /* Public */,
     213,    3, 1450,    2, 0x0a /* Public */,
     217,    1, 1457,    2, 0x0a /* Public */,
     218,    0, 1460,    2, 0x0a /* Public */,
     219,    0, 1461,    2, 0x0a /* Public */,
     221,    0, 1462,    2, 0x0a /* Public */,
     222,    1, 1463,    2, 0x0a /* Public */,
     223,    0, 1466,    2, 0x0a /* Public */,
     224,    1, 1467,    2, 0x0a /* Public */,
     225,    0, 1470,    2, 0x08 /* Private */,
     226,    0, 1471,    2, 0x08 /* Private */,
     227,    0, 1472,    2, 0x08 /* Private */,
     228,    0, 1473,    2, 0x08 /* Private */,
     229,    0, 1474,    2, 0x08 /* Private */,
     230,    0, 1475,    2, 0x08 /* Private */,
     231,    0, 1476,    2, 0x08 /* Private */,
     232,    0, 1477,    2, 0x08 /* Private */,
     233,    0, 1478,    2, 0x08 /* Private */,
     234,    0, 1479,    2, 0x08 /* Private */,
     235,    0, 1480,    2, 0x08 /* Private */,
     236,    0, 1481,    2, 0x08 /* Private */,
     237,    0, 1482,    2, 0x08 /* Private */,
     238,    0, 1483,    2, 0x08 /* Private */,
     239,    0, 1484,    2, 0x08 /* Private */,
     240,    0, 1485,    2, 0x08 /* Private */,
     241,    0, 1486,    2, 0x08 /* Private */,
     242,    0, 1487,    2, 0x08 /* Private */,
     243,    0, 1488,    2, 0x08 /* Private */,
     244,    0, 1489,    2, 0x08 /* Private */,
     245,    0, 1490,    2, 0x0a /* Public */,
     246,    1, 1491,    2, 0x0a /* Public */,
     246,    0, 1494,    2, 0x2a /* Public | MethodCloned */,
     248,    1, 1495,    2, 0x0a /* Public */,
     248,    0, 1498,    2, 0x2a /* Public | MethodCloned */,
     249,    0, 1499,    2, 0x0a /* Public */,
     250,    1, 1500,    2, 0x0a /* Public */,
     251,    0, 1503,    2, 0x0a /* Public */,
     252,    0, 1504,    2, 0x0a /* Public */,
     253,    0, 1505,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    0x80000000 | 7, QMetaType::QString, 0x80000000 | 8, 0x80000000 | 9,    2,    2,   10,
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 16, 0x80000000 | 17,    2,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::Int,   42,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,    2,
    QMetaType::Bool, QMetaType::QString,   53,
    QMetaType::Void, QMetaType::QStringList,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   58,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QStringList, QMetaType::Bool,    2,   68,
    QMetaType::Void, QMetaType::QStringList,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,   73,
    QMetaType::Void, QMetaType::Bool,   68,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 116,    2,
    QMetaType::Void, 0x80000000 | 116,    2,
    QMetaType::Bool, 0x80000000 | 16, 0x80000000 | 120,  119,  121,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, 0x80000000 | 16, 0x80000000 | 120,  119,  121,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,  132,  133,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,  136,
    QMetaType::Void, QMetaType::QColor,  138,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::QColor,
    QMetaType::Int,
    QMetaType::Void, QMetaType::QColor,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 195,    2,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 202,  203,
    QMetaType::Void, 0x80000000 | 202,  203,
    QMetaType::Void, 0x80000000 | 206,  207,
    QMetaType::Void, 0x80000000 | 209,  210,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,  132,   42,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16, 0x80000000 | 214, 0x80000000 | 214,  119,  215,  216,
    QMetaType::Void, 0x80000000 | 16,  119,
    QMetaType::Void,
    0x80000000 | 220,
    QMetaType::Bool,
    QMetaType::QVariant, QMetaType::QString,    2,
    QMetaType::QObjectStar,
    QMetaType::Bool, QMetaType::Int,  136,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::Bool,  247,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,  247,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  247,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Main::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Main *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fileNew(); break;
        case 1: _t->fileNewCopy(); break;
        case 2: _t->addUserFlag(); break;
        case 3: _t->editorChanged(); break;
        case 4: { File::ErrorCode _r = _t->fileLoad((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< const LoadMode(*)>(_a[2])),(*reinterpret_cast< const FileType(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< File::ErrorCode*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->fileLoad((*reinterpret_cast< const LoadMode(*)>(_a[1]))); break;
        case 6: _t->fileLoad(); break;
        case 7: _t->fileSaveSession(); break;
        case 8: _t->fileRestoreSession(); break;
        case 9: _t->fileLoadRecent(); break;
        case 10: _t->addRecentMap((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->fileSave((*reinterpret_cast< VymModel*(*)>(_a[1])),(*reinterpret_cast< const SaveMode(*)>(_a[2]))); break;
        case 12: _t->fileSave(); break;
        case 13: _t->fileSave((*reinterpret_cast< VymModel*(*)>(_a[1]))); break;
        case 14: _t->fileSaveAs(); break;
        case 15: _t->fileSaveAs((*reinterpret_cast< const SaveMode(*)>(_a[1]))); break;
        case 16: _t->fileSaveAsDefault(); break;
        case 17: _t->fileImportFirefoxBookmarks(); break;
        case 18: _t->fileImportFreemind(); break;
        case 19: _t->fileImportMM(); break;
        case 20: _t->fileImportDir(); break;
        case 21: _t->fileExportAO(); break;
        case 22: _t->fileExportASCII(); break;
        case 23: _t->fileExportASCIITasks(); break;
        case 24: _t->fileExportCSV(); break;
        case 25: _t->fileExportConfluence(); break;
        case 26: _t->fileExportFirefoxBookmarks(); break;
        case 27: _t->fileExportHTML(); break;
        case 28: _t->fileExportImage(); break;
        case 29: _t->fileExportImpress(); break;
        case 30: _t->fileExportLaTeX(); break;
        case 31: _t->fileExportMarkdown(); break;
        case 32: _t->fileExportOrgMode(); break;
        case 33: _t->fileExportPDF(); break;
        case 34: _t->fileExportSVG(); break;
        case 35: _t->fileExportTaskjuggler(); break;
        case 36: _t->fileExportXML(); break;
        case 37: _t->fileExportLast(); break;
        case 38: { bool _r = _t->fileCloseMap((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 39: { bool _r = _t->fileCloseMap();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 40: _t->filePrint(); break;
        case 41: { bool _r = _t->fileExitVYM();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 42: _t->editUndo(); break;
        case 43: _t->editRedo(); break;
        case 44: _t->gotoHistoryStep((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 45: _t->editCopy(); break;
        case 46: _t->editPaste(); break;
        case 47: _t->editCut(); break;
        case 48: _t->updateQueries((*reinterpret_cast< VymModel*(*)>(_a[1]))); break;
        case 49: { bool _r = _t->openURL((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 50: _t->openTabs((*reinterpret_cast< QStringList(*)>(_a[1]))); break;
        case 51: _t->editOpenURL(); break;
        case 52: _t->editOpenURLTab(); break;
        case 53: _t->editOpenMultipleVisURLTabs((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 54: _t->editOpenMultipleVisURLTabs(); break;
        case 55: _t->editOpenMultipleURLTabs(); break;
        case 56: _t->editNote2URLs(); break;
        case 57: _t->editURL(); break;
        case 58: _t->editLocalURL(); break;
        case 59: _t->editHeading2URL(); break;
        case 60: _t->getJiraDataSubtree(); break;
        case 61: _t->setHeadingConfluencePageName(); break;
        case 62: _t->getConfluenceUser(); break;
        case 63: _t->openVymLinks((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 64: _t->openVymLinks((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 65: _t->editVymLink(); break;
        case 66: _t->editOpenMultipleVymLinks(); break;
        case 67: _t->editHeading(); break;
        case 68: _t->editHeadingFinished((*reinterpret_cast< VymModel*(*)>(_a[1]))); break;
        case 69: _t->editOpenVymLink((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 70: _t->editOpenVymLink(); break;
        case 71: _t->editOpenVymLinkBackground(); break;
        case 72: _t->editDeleteVymLink(); break;
        case 73: _t->editToggleHideExport(); break;
        case 74: _t->editToggleTask(); break;
        case 75: _t->editCycleTaskStatus(); break;
        case 76: _t->editTaskResetDeltaPrio(); break;
        case 77: _t->editTaskSleepN(); break;
        case 78: _t->editAddTimestamp(); break;
        case 79: _t->editMapProperties(); break;
        case 80: _t->editMoveUp(); break;
        case 81: _t->editMoveDown(); break;
        case 82: _t->editMoveUpDiagonally(); break;
        case 83: _t->editMoveDownDiagonally(); break;
        case 84: _t->editDetach(); break;
        case 85: _t->editSortChildren(); break;
        case 86: _t->editSortBackChildren(); break;
        case 87: _t->editToggleScroll(); break;
        case 88: _t->editExpandAll(); break;
        case 89: _t->editExpandOneLevel(); break;
        case 90: _t->editCollapseOneLevel(); break;
        case 91: _t->editCollapseUnselected(); break;
        case 92: _t->editUnscrollChildren(); break;
        case 93: _t->editGrowSelectionSize(); break;
        case 94: _t->editShrinkSelectionSize(); break;
        case 95: _t->editResetSelectionSize(); break;
        case 96: _t->editAddAttribute(); break;
        case 97: _t->editAddMapCenter(); break;
        case 98: _t->editNewBranch(); break;
        case 99: _t->editNewBranchBefore(); break;
        case 100: _t->editNewBranchAbove(); break;
        case 101: _t->editNewBranchBelow(); break;
        case 102: _t->editImportAdd(); break;
        case 103: _t->editImportReplace(); break;
        case 104: _t->editSaveBranch(); break;
        case 105: _t->editDeleteKeepChildren(); break;
        case 106: _t->editDeleteChildren(); break;
        case 107: _t->editDeleteSelection(); break;
        case 108: _t->editLoadImage(); break;
        case 109: _t->editSaveImage(); break;
        case 110: _t->popupFollowXLink(); break;
        case 111: _t->editFollowXLink((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 112: _t->editEditXLink((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 113: { bool _r = _t->initLinkedMapsMenu((*reinterpret_cast< VymModel*(*)>(_a[1])),(*reinterpret_cast< QMenu*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 114: _t->editGoToLinkedMap(); break;
        case 115: _t->editToggleTarget(); break;
        case 116: { bool _r = _t->initTargetsMenu((*reinterpret_cast< VymModel*(*)>(_a[1])),(*reinterpret_cast< QMenu*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 117: _t->editGoToTarget(); break;
        case 118: _t->editMoveToTarget(); break;
        case 119: _t->editSelectPrevious(); break;
        case 120: _t->editSelectNext(); break;
        case 121: _t->editSelectNothing(); break;
        case 122: _t->editOpenFindResultWidget(); break;
        case 123: _t->editFindNext((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 124: _t->editFindDuplicateURLs(); break;
        case 125: _t->selectQuickColor((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 126: _t->setQuickColor((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 127: _t->quickColorPressed(); break;
        case 128: _t->formatPickColor(); break;
        case 129: { QColor _r = _t->getCurrentColor();
            if (_a[0]) *reinterpret_cast< QColor*>(_a[0]) = std::move(_r); }  break;
        case 130: { int _r = _t->getCurrentColorIndex();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 131: _t->setCurrentColor((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 132: _t->formatColorBranch(); break;
        case 133: _t->formatColorSubtree(); break;
        case 134: _t->formatLinkStyleLine(); break;
        case 135: _t->formatLinkStyleParabel(); break;
        case 136: _t->formatLinkStylePolyLine(); break;
        case 137: _t->formatLinkStylePolyParabel(); break;
        case 138: _t->formatSelectBackColor(); break;
        case 139: _t->formatSelectBackImage(); break;
        case 140: _t->formatSelectLinkColor(); break;
        case 141: _t->formatSelectSelectionColor(); break;
        case 142: _t->formatSelectFont(); break;
        case 143: _t->formatToggleLinkColorHint(); break;
        case 144: _t->formatHideLinkUnselected(); break;
        case 145: _t->viewZoomReset(); break;
        case 146: _t->viewZoomIn(); break;
        case 147: _t->viewZoomOut(); break;
        case 148: _t->viewRotateCounterClockwise(); break;
        case 149: _t->viewRotateClockwise(); break;
        case 150: _t->viewCenter(); break;
        case 151: _t->viewCenterScaled(); break;
        case 152: _t->networkStartServer(); break;
        case 153: _t->networkConnect(); break;
        case 154: _t->downloadFinished(); break;
        case 155: { bool _r = _t->settingsPDF();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 156: { bool _r = _t->settingsURL();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 157: _t->settingsZipTool(); break;
        case 158: _t->settingsMacroPath(); break;
        case 159: _t->settingsUndoLevels(); break;
        case 160: _t->settingsDefaultMapPath(); break;
        case 161: _t->settingsAutosaveTime(); break;
        case 162: _t->settingsDefaultMapAuthor(); break;
        case 163: _t->settingsDarkTheme(); break;
        case 164: _t->settingsShowParentsLevelTasks(); break;
        case 165: _t->settingsShowParentsLevelFindResults(); break;
        case 166: _t->settingsToggleAutoLayout(); break;
        case 167: _t->settingsToggleWriteBackupFile(); break;
        case 168: _t->settingsToggleAnimation(); break;
        case 169: _t->settingsToggleDownloads(); break;
        case 170: { bool _r = _t->settingsConfluence();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 171: { bool _r = _t->settingsJIRA();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 172: _t->windowToggleNoteEditor(); break;
        case 173: _t->windowToggleTreeEditor(); break;
        case 174: _t->windowToggleTaskEditor(); break;
        case 175: _t->windowToggleSlideEditor(); break;
        case 176: _t->windowToggleScriptEditor(); break;
        case 177: _t->windowToggleScriptOutput(); break;
        case 178: _t->windowToggleHistory(); break;
        case 179: _t->windowToggleProperty(); break;
        case 180: _t->windowShowHeadingEditor(); break;
        case 181: _t->windowToggleHeadingEditor(); break;
        case 182: _t->updateHistory((*reinterpret_cast< SimpleSettings(*)>(_a[1]))); break;
        case 183: _t->windowToggleAntiAlias(); break;
        case 184: { bool _r = _t->isAliased();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 185: { bool _r = _t->hasSmoothPixmapTransform();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 186: _t->windowToggleSmoothPixmap(); break;
        case 187: _t->clearScriptOutput(); break;
        case 188: _t->updateHeading((*reinterpret_cast< const VymText(*)>(_a[1]))); break;
        case 189: _t->updateNoteText((*reinterpret_cast< const VymText(*)>(_a[1]))); break;
        case 190: _t->updateNoteEditor((*reinterpret_cast< TreeItem*(*)>(_a[1]))); break;
        case 191: _t->updateHeadingEditor((*reinterpret_cast< BranchItem*(*)>(_a[1]))); break;
        case 192: _t->updateHeadingEditor(); break;
        case 193: _t->selectInNoteEditor((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 194: _t->setFocusMapEditor(); break;
        case 195: _t->changeSelection((*reinterpret_cast< VymModel*(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2])),(*reinterpret_cast< const QItemSelection(*)>(_a[3]))); break;
        case 196: _t->updateDockWidgetTitles((*reinterpret_cast< VymModel*(*)>(_a[1]))); break;
        case 197: _t->updateActions(); break;
        case 198: { ModMode _r = _t->getModMode();
            if (_a[0]) *reinterpret_cast< ModMode*>(_a[0]) = std::move(_r); }  break;
        case 199: { bool _r = _t->autoSelectNewBranch();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 200: { QVariant _r = _t->runScript((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 201: { QObject* _r = _t->getCurrentModelWrapper();
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        case 202: { bool _r = _t->gotoWindow((*reinterpret_cast< const int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 203: _t->windowNextEditor(); break;
        case 204: _t->windowPreviousEditor(); break;
        case 205: _t->nextSlide(); break;
        case 206: _t->previousSlide(); break;
        case 207: _t->flagChanged(); break;
        case 208: _t->testFunction1(); break;
        case 209: _t->testFunction2(); break;
        case 210: _t->toggleWinter(); break;
        case 211: _t->toggleHideExport(); break;
        case 212: _t->testCommand(); break;
        case 213: _t->helpDoc(); break;
        case 214: _t->helpDemo(); break;
        case 215: _t->helpShortcuts(); break;
        case 216: _t->helpMacros(); break;
        case 217: _t->helpScriptingCommands(); break;
        case 218: _t->helpDebugInfo(); break;
        case 219: _t->helpAbout(); break;
        case 220: _t->helpAboutQT(); break;
        case 221: _t->callMacro(); break;
        case 222: _t->downloadReleaseNotesFinished(); break;
        case 223: _t->checkReleaseNotes(); break;
        case 224: { bool _r = _t->downloadsEnabled((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 225: { bool _r = _t->downloadsEnabled();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 226: _t->downloadUpdatesFinished((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 227: _t->downloadUpdatesFinished(); break;
        case 228: _t->downloadUpdatesFinishedInt(); break;
        case 229: _t->downloadUpdates((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 230: _t->checkUpdates(); break;
        case 231: _t->escapePressed(); break;
        case 232: _t->togglePresentationMode(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VymModel* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VymModel* >(); break;
            }
            break;
        case 48:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VymModel* >(); break;
            }
            break;
        case 68:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VymModel* >(); break;
            }
            break;
        case 111:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        case 112:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        case 113:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VymModel* >(); break;
            }
            break;
        case 116:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VymModel* >(); break;
            }
            break;
        case 195:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VymModel* >(); break;
            }
            break;
        case 196:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VymModel* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Main::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Main.data,
    qt_meta_data_Main,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Main::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Main::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Main.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Main::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 233)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 233;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 233)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 233;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
