/********************************************************************************
** Form generated from reading UI file 'extrainfodialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXTRAINFODIALOG_H
#define UI_EXTRAINFODIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ExtraInfoDialog
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *textLabel1_2;
    QLineEdit *mapNameLE;
    QLabel *label_2;
    QLineEdit *fileLocationLE;
    QLabel *label;
    QLineEdit *mapTitleLE;
    QLabel *textLabel1;
    QLineEdit *authorLE;
    QSpacerItem *verticalSpacer;
    QLabel *textLabel2;
    QTextEdit *commentTE;
    QLabel *textLabel2_2;
    QTextEdit *statsTE;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *closeButton;

    void setupUi(QDialog *ExtraInfoDialog)
    {
        if (ExtraInfoDialog->objectName().isEmpty())
            ExtraInfoDialog->setObjectName(QString::fromUtf8("ExtraInfoDialog"));
        ExtraInfoDialog->resize(595, 686);
        vboxLayout = new QVBoxLayout(ExtraInfoDialog);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        textLabel1_2 = new QLabel(ExtraInfoDialog);
        textLabel1_2->setObjectName(QString::fromUtf8("textLabel1_2"));

        vboxLayout->addWidget(textLabel1_2);

        mapNameLE = new QLineEdit(ExtraInfoDialog);
        mapNameLE->setObjectName(QString::fromUtf8("mapNameLE"));
        mapNameLE->setReadOnly(true);

        vboxLayout->addWidget(mapNameLE);

        label_2 = new QLabel(ExtraInfoDialog);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        vboxLayout->addWidget(label_2);

        fileLocationLE = new QLineEdit(ExtraInfoDialog);
        fileLocationLE->setObjectName(QString::fromUtf8("fileLocationLE"));
        fileLocationLE->setEnabled(false);
        fileLocationLE->setAutoFillBackground(false);
        fileLocationLE->setReadOnly(true);

        vboxLayout->addWidget(fileLocationLE);

        label = new QLabel(ExtraInfoDialog);
        label->setObjectName(QString::fromUtf8("label"));

        vboxLayout->addWidget(label);

        mapTitleLE = new QLineEdit(ExtraInfoDialog);
        mapTitleLE->setObjectName(QString::fromUtf8("mapTitleLE"));

        vboxLayout->addWidget(mapTitleLE);

        textLabel1 = new QLabel(ExtraInfoDialog);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));

        vboxLayout->addWidget(textLabel1);

        authorLE = new QLineEdit(ExtraInfoDialog);
        authorLE->setObjectName(QString::fromUtf8("authorLE"));

        vboxLayout->addWidget(authorLE);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout->addItem(verticalSpacer);

        textLabel2 = new QLabel(ExtraInfoDialog);
        textLabel2->setObjectName(QString::fromUtf8("textLabel2"));

        vboxLayout->addWidget(textLabel2);

        commentTE = new QTextEdit(ExtraInfoDialog);
        commentTE->setObjectName(QString::fromUtf8("commentTE"));

        vboxLayout->addWidget(commentTE);

        textLabel2_2 = new QLabel(ExtraInfoDialog);
        textLabel2_2->setObjectName(QString::fromUtf8("textLabel2_2"));

        vboxLayout->addWidget(textLabel2_2);

        statsTE = new QTextEdit(ExtraInfoDialog);
        statsTE->setObjectName(QString::fromUtf8("statsTE"));
        statsTE->setEnabled(false);
        QFont font;
        font.setFamily(QString::fromUtf8("DejaVu Sans Mono"));
        statsTE->setFont(font);

        vboxLayout->addWidget(statsTE);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(160, 21, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        closeButton = new QPushButton(ExtraInfoDialog);
        closeButton->setObjectName(QString::fromUtf8("closeButton"));

        hboxLayout->addWidget(closeButton);


        vboxLayout->addLayout(hboxLayout);

        QWidget::setTabOrder(authorLE, closeButton);
        QWidget::setTabOrder(closeButton, mapNameLE);

        retranslateUi(ExtraInfoDialog);
        QObject::connect(closeButton, SIGNAL(clicked(bool)), ExtraInfoDialog, SLOT(accept()));

        QMetaObject::connectSlotsByName(ExtraInfoDialog);
    } // setupUi

    void retranslateUi(QDialog *ExtraInfoDialog)
    {
        ExtraInfoDialog->setWindowTitle(QCoreApplication::translate("ExtraInfoDialog", "VYM - Info", nullptr));
        textLabel1_2->setText(QCoreApplication::translate("ExtraInfoDialog", "Map:", nullptr));
        label_2->setText(QCoreApplication::translate("ExtraInfoDialog", "File location:", nullptr));
#if QT_CONFIG(accessibility)
        label->setAccessibleDescription(QString());
#endif // QT_CONFIG(accessibility)
        label->setText(QCoreApplication::translate("ExtraInfoDialog", "Title:", nullptr));
        textLabel1->setText(QCoreApplication::translate("ExtraInfoDialog", "Author:", nullptr));
        textLabel2->setText(QCoreApplication::translate("ExtraInfoDialog", "Comment:", nullptr));
        textLabel2_2->setText(QCoreApplication::translate("ExtraInfoDialog", "Statistics:", nullptr));
        closeButton->setText(QCoreApplication::translate("ExtraInfoDialog", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExtraInfoDialog: public Ui_ExtraInfoDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXTRAINFODIALOG_H
