/********************************************************************************
** Form generated from reading UI file 'scripteditor.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCRIPTEDITOR_H
#define UI_SCRIPTEDITOR_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ScriptEditor
{
public:
    QVBoxLayout *verticalLayout_2;
    QTabWidget *modeTabWidget;
    QWidget *slideTab;
    QVBoxLayout *verticalLayout_4;
    QVBoxLayout *slideVerticalLayout;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *slideRunButton;
    QSpacerItem *horizontalSpacer;
    QPushButton *slideSaveButton;
    QWidget *macroTab;
    QVBoxLayout *verticalLayout_3;
    QVBoxLayout *macroVerticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *macroRunButton;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *macroLoadButton;
    QPushButton *macroSaveButton;
    QLineEdit *macroPathLineEdit;
    QWidget *fileTab;
    QVBoxLayout *verticalLayout;
    QVBoxLayout *fileVerticalLayout;
    QHBoxLayout *fileButtonLayout;
    QPushButton *fileRunButton;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *fileLoadButton;
    QPushButton *fileSaveButton;
    QPushButton *fileSaveAsButton;
    QLineEdit *scriptPathLineEdit;

    void setupUi(QWidget *ScriptEditor)
    {
        if (ScriptEditor->objectName().isEmpty())
            ScriptEditor->setObjectName(QString::fromUtf8("ScriptEditor"));
        ScriptEditor->setEnabled(true);
        ScriptEditor->resize(404, 268);
        ScriptEditor->setMinimumSize(QSize(200, 0));
        verticalLayout_2 = new QVBoxLayout(ScriptEditor);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        modeTabWidget = new QTabWidget(ScriptEditor);
        modeTabWidget->setObjectName(QString::fromUtf8("modeTabWidget"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(modeTabWidget->sizePolicy().hasHeightForWidth());
        modeTabWidget->setSizePolicy(sizePolicy);
        modeTabWidget->setMaximumSize(QSize(16777215, 1000000));
#if QT_CONFIG(accessibility)
        modeTabWidget->setAccessibleName(QString::fromUtf8(""));
#endif // QT_CONFIG(accessibility)
        slideTab = new QWidget();
        slideTab->setObjectName(QString::fromUtf8("slideTab"));
        verticalLayout_4 = new QVBoxLayout(slideTab);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        slideVerticalLayout = new QVBoxLayout();
        slideVerticalLayout->setObjectName(QString::fromUtf8("slideVerticalLayout"));
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        slideRunButton = new QPushButton(slideTab);
        slideRunButton->setObjectName(QString::fromUtf8("slideRunButton"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/script-run.svg"), QSize(), QIcon::Normal, QIcon::Off);
        slideRunButton->setIcon(icon);

        horizontalLayout_4->addWidget(slideRunButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);

        slideSaveButton = new QPushButton(slideTab);
        slideSaveButton->setObjectName(QString::fromUtf8("slideSaveButton"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/filesave.png"), QSize(), QIcon::Normal, QIcon::Off);
        slideSaveButton->setIcon(icon1);

        horizontalLayout_4->addWidget(slideSaveButton);


        slideVerticalLayout->addLayout(horizontalLayout_4);


        verticalLayout_4->addLayout(slideVerticalLayout);

        modeTabWidget->addTab(slideTab, QString());
        modeTabWidget->setTabText(modeTabWidget->indexOf(slideTab), QString::fromUtf8("Slide tab"));
        macroTab = new QWidget();
        macroTab->setObjectName(QString::fromUtf8("macroTab"));
        macroTab->setEnabled(true);
        verticalLayout_3 = new QVBoxLayout(macroTab);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        macroVerticalLayout = new QVBoxLayout();
        macroVerticalLayout->setObjectName(QString::fromUtf8("macroVerticalLayout"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        macroRunButton = new QPushButton(macroTab);
        macroRunButton->setObjectName(QString::fromUtf8("macroRunButton"));
        macroRunButton->setIcon(icon);

        horizontalLayout_2->addWidget(macroRunButton);

        horizontalSpacer_2 = new QSpacerItem(410, 13, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        macroLoadButton = new QPushButton(macroTab);
        macroLoadButton->setObjectName(QString::fromUtf8("macroLoadButton"));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/script-reload.svg"), QSize(), QIcon::Normal, QIcon::Off);
        macroLoadButton->setIcon(icon2);

        horizontalLayout_2->addWidget(macroLoadButton);

        macroSaveButton = new QPushButton(macroTab);
        macroSaveButton->setObjectName(QString::fromUtf8("macroSaveButton"));
        macroSaveButton->setIcon(icon1);

        horizontalLayout_2->addWidget(macroSaveButton);


        macroVerticalLayout->addLayout(horizontalLayout_2);


        verticalLayout_3->addLayout(macroVerticalLayout);

        macroPathLineEdit = new QLineEdit(macroTab);
        macroPathLineEdit->setObjectName(QString::fromUtf8("macroPathLineEdit"));
        macroPathLineEdit->setMinimumSize(QSize(0, 0));
        macroPathLineEdit->setText(QString::fromUtf8(""));
        macroPathLineEdit->setFrame(false);
        macroPathLineEdit->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        macroPathLineEdit->setReadOnly(true);

        verticalLayout_3->addWidget(macroPathLineEdit);

        modeTabWidget->addTab(macroTab, QString());
        modeTabWidget->setTabText(modeTabWidget->indexOf(macroTab), QString::fromUtf8("Macro tab"));
        fileTab = new QWidget();
        fileTab->setObjectName(QString::fromUtf8("fileTab"));
        verticalLayout = new QVBoxLayout(fileTab);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        fileVerticalLayout = new QVBoxLayout();
        fileVerticalLayout->setObjectName(QString::fromUtf8("fileVerticalLayout"));
        fileButtonLayout = new QHBoxLayout();
        fileButtonLayout->setObjectName(QString::fromUtf8("fileButtonLayout"));
        fileRunButton = new QPushButton(fileTab);
        fileRunButton->setObjectName(QString::fromUtf8("fileRunButton"));
        fileRunButton->setIcon(icon);

        fileButtonLayout->addWidget(fileRunButton);

        horizontalSpacer_4 = new QSpacerItem(369, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        fileButtonLayout->addItem(horizontalSpacer_4);

        fileLoadButton = new QPushButton(fileTab);
        fileLoadButton->setObjectName(QString::fromUtf8("fileLoadButton"));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/fileopen.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileLoadButton->setIcon(icon3);

        fileButtonLayout->addWidget(fileLoadButton);

        fileSaveButton = new QPushButton(fileTab);
        fileSaveButton->setObjectName(QString::fromUtf8("fileSaveButton"));
        fileSaveButton->setIcon(icon1);

        fileButtonLayout->addWidget(fileSaveButton);

        fileSaveAsButton = new QPushButton(fileTab);
        fileSaveAsButton->setObjectName(QString::fromUtf8("fileSaveAsButton"));
        QSizePolicy sizePolicy1(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(fileSaveAsButton->sizePolicy().hasHeightForWidth());
        fileSaveAsButton->setSizePolicy(sizePolicy1);
        fileSaveAsButton->setMinimumSize(QSize(84, 0));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/filesaveas.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileSaveAsButton->setIcon(icon4);

        fileButtonLayout->addWidget(fileSaveAsButton);


        fileVerticalLayout->addLayout(fileButtonLayout);


        verticalLayout->addLayout(fileVerticalLayout);

        scriptPathLineEdit = new QLineEdit(fileTab);
        scriptPathLineEdit->setObjectName(QString::fromUtf8("scriptPathLineEdit"));
        scriptPathLineEdit->setEnabled(true);
        scriptPathLineEdit->setReadOnly(true);

        verticalLayout->addWidget(scriptPathLineEdit);

        modeTabWidget->addTab(fileTab, QString());
        modeTabWidget->setTabText(modeTabWidget->indexOf(fileTab), QString::fromUtf8("File tab"));

        verticalLayout_2->addWidget(modeTabWidget);


        retranslateUi(ScriptEditor);

        modeTabWidget->setCurrentIndex(2);


        QMetaObject::connectSlotsByName(ScriptEditor);
    } // setupUi

    void retranslateUi(QWidget *ScriptEditor)
    {
        ScriptEditor->setWindowTitle(QCoreApplication::translate("ScriptEditor", "Form", nullptr));
#if QT_CONFIG(accessibility)
        slideTab->setAccessibleDescription(QString());
#endif // QT_CONFIG(accessibility)
        slideRunButton->setText(QCoreApplication::translate("ScriptEditor", "Run", nullptr));
#if QT_CONFIG(shortcut)
        slideRunButton->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        slideSaveButton->setText(QCoreApplication::translate("ScriptEditor", "Save to selected slide", nullptr));
#if QT_CONFIG(shortcut)
        slideSaveButton->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        macroRunButton->setText(QCoreApplication::translate("ScriptEditor", "Run", nullptr));
        macroLoadButton->setText(QCoreApplication::translate("ScriptEditor", "Reload", nullptr));
        macroSaveButton->setText(QCoreApplication::translate("ScriptEditor", "Save", nullptr));
        macroPathLineEdit->setInputMask(QString());
        fileRunButton->setText(QCoreApplication::translate("ScriptEditor", "Run", nullptr));
        fileLoadButton->setText(QCoreApplication::translate("ScriptEditor", "Load", nullptr));
        fileSaveButton->setText(QCoreApplication::translate("ScriptEditor", "Save", nullptr));
        fileSaveAsButton->setText(QCoreApplication::translate("ScriptEditor", "Save as", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ScriptEditor: public Ui_ScriptEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCRIPTEDITOR_H
