/********************************************************************************
** Form generated from reading UI file 'confluence-userdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFLUENCE_2D_USERDIALOG_H
#define UI_CONFLUENCE_2D_USERDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ConfluenceUserDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLineEdit *lineEdit;
    QListWidget *userListWidget;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ConfluenceUserDialog)
    {
        if (ConfluenceUserDialog->objectName().isEmpty())
            ConfluenceUserDialog->setObjectName(QString::fromUtf8("ConfluenceUserDialog"));
        ConfluenceUserDialog->resize(392, 308);
        verticalLayout = new QVBoxLayout(ConfluenceUserDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        lineEdit = new QLineEdit(ConfluenceUserDialog);
        lineEdit->setObjectName(QString::fromUtf8("lineEdit"));

        verticalLayout->addWidget(lineEdit);

        userListWidget = new QListWidget(ConfluenceUserDialog);
        userListWidget->setObjectName(QString::fromUtf8("userListWidget"));

        verticalLayout->addWidget(userListWidget);

        buttonBox = new QDialogButtonBox(ConfluenceUserDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setLayoutDirection(Qt::LeftToRight);
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(ConfluenceUserDialog);

        QMetaObject::connectSlotsByName(ConfluenceUserDialog);
    } // setupUi

    void retranslateUi(QDialog *ConfluenceUserDialog)
    {
        ConfluenceUserDialog->setWindowTitle(QCoreApplication::translate("ConfluenceUserDialog", "Dialog", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ConfluenceUserDialog: public Ui_ConfluenceUserDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONFLUENCE_2D_USERDIALOG_H
