/********************************************************************************
** Form generated from reading UI file 'zip-settings-dialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ZIP_2D_SETTINGS_2D_DIALOG_H
#define UI_ZIP_2D_SETTINGS_2D_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ZipSettingsDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *zipToolLabel;
    QHBoxLayout *horizontalLayout;
    QLineEdit *zipToolPathLE;
    QPushButton *zipToolButton;
    QLabel *zipToolStatusLabel;
    QLabel *unzipToolLabel;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *unzipToolPathLE;
    QPushButton *unzipToolButton;
    QLabel *unzipToolStatusLabel;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer;
    QPushButton *closeButton;

    void setupUi(QDialog *ZipSettingsDialog)
    {
        if (ZipSettingsDialog->objectName().isEmpty())
            ZipSettingsDialog->setObjectName(QString::fromUtf8("ZipSettingsDialog"));
        ZipSettingsDialog->resize(766, 222);
        verticalLayout = new QVBoxLayout(ZipSettingsDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        zipToolLabel = new QLabel(ZipSettingsDialog);
        zipToolLabel->setObjectName(QString::fromUtf8("zipToolLabel"));

        verticalLayout->addWidget(zipToolLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        zipToolPathLE = new QLineEdit(ZipSettingsDialog);
        zipToolPathLE->setObjectName(QString::fromUtf8("zipToolPathLE"));

        horizontalLayout->addWidget(zipToolPathLE);

        zipToolButton = new QPushButton(ZipSettingsDialog);
        zipToolButton->setObjectName(QString::fromUtf8("zipToolButton"));

        horizontalLayout->addWidget(zipToolButton);

        zipToolStatusLabel = new QLabel(ZipSettingsDialog);
        zipToolStatusLabel->setObjectName(QString::fromUtf8("zipToolStatusLabel"));

        horizontalLayout->addWidget(zipToolStatusLabel);


        verticalLayout->addLayout(horizontalLayout);

        unzipToolLabel = new QLabel(ZipSettingsDialog);
        unzipToolLabel->setObjectName(QString::fromUtf8("unzipToolLabel"));

        verticalLayout->addWidget(unzipToolLabel);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        unzipToolPathLE = new QLineEdit(ZipSettingsDialog);
        unzipToolPathLE->setObjectName(QString::fromUtf8("unzipToolPathLE"));

        horizontalLayout_2->addWidget(unzipToolPathLE);

        unzipToolButton = new QPushButton(ZipSettingsDialog);
        unzipToolButton->setObjectName(QString::fromUtf8("unzipToolButton"));

        horizontalLayout_2->addWidget(unzipToolButton);

        unzipToolStatusLabel = new QLabel(ZipSettingsDialog);
        unzipToolStatusLabel->setObjectName(QString::fromUtf8("unzipToolStatusLabel"));

        horizontalLayout_2->addWidget(unzipToolStatusLabel);


        verticalLayout->addLayout(horizontalLayout_2);

        verticalSpacer = new QSpacerItem(17, 28, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        horizontalSpacer = new QSpacerItem(595, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);

        closeButton = new QPushButton(ZipSettingsDialog);
        closeButton->setObjectName(QString::fromUtf8("closeButton"));

        horizontalLayout_3->addWidget(closeButton);


        verticalLayout->addLayout(horizontalLayout_3);


        retranslateUi(ZipSettingsDialog);

        QMetaObject::connectSlotsByName(ZipSettingsDialog);
    } // setupUi

    void retranslateUi(QDialog *ZipSettingsDialog)
    {
        ZipSettingsDialog->setWindowTitle(QCoreApplication::translate("ZipSettingsDialog", "zip settings", nullptr));
        zipToolLabel->setText(QCoreApplication::translate("ZipSettingsDialog", "zip tool:", nullptr));
        zipToolButton->setText(QCoreApplication::translate("ZipSettingsDialog", "Choose", nullptr));
        zipToolStatusLabel->setText(QCoreApplication::translate("ZipSettingsDialog", "Status", nullptr));
        unzipToolLabel->setText(QCoreApplication::translate("ZipSettingsDialog", "unzip tool:", nullptr));
        unzipToolButton->setText(QCoreApplication::translate("ZipSettingsDialog", "Choose", nullptr));
        unzipToolStatusLabel->setText(QCoreApplication::translate("ZipSettingsDialog", "Status", nullptr));
        closeButton->setText(QCoreApplication::translate("ZipSettingsDialog", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ZipSettingsDialog: public Ui_ZipSettingsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ZIP_2D_SETTINGS_2D_DIALOG_H
