/********************************************************************************
** Form generated from reading UI file 'export-confluence-dialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORT_2D_CONFLUENCE_2D_DIALOG_H
#define UI_EXPORT_2D_CONFLUENCE_2D_DIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ExportConfluenceDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *exportModeLabel;
    QRadioButton *updatePageButton;
    QRadioButton *createPageButton;
    QLabel *URLLabel;
    QGridLayout *gridLayout;
    QLineEdit *lineEditURL;
    QPushButton *openPageButton;
    QLabel *pageNameLabel;
    QLineEdit *lineEditPageName;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QCheckBox *mapCenterToPageNameCheckBox;
    QCheckBox *includeMapImageCheckBox;
    QCheckBox *includeImagesCheckBox;
    QCheckBox *textColorCheckBox;
    QCheckBox *useNumberingCheckBox;
    QCheckBox *saveSettingsInMapCheckBox;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *spacer;
    QPushButton *cancelButton;
    QPushButton *exportButton;
    QButtonGroup *buttonGroup;

    void setupUi(QDialog *ExportConfluenceDialog)
    {
        if (ExportConfluenceDialog->objectName().isEmpty())
            ExportConfluenceDialog->setObjectName(QString::fromUtf8("ExportConfluenceDialog"));
        ExportConfluenceDialog->resize(591, 444);
        verticalLayout_2 = new QVBoxLayout(ExportConfluenceDialog);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        exportModeLabel = new QLabel(ExportConfluenceDialog);
        exportModeLabel->setObjectName(QString::fromUtf8("exportModeLabel"));

        verticalLayout_2->addWidget(exportModeLabel);

        updatePageButton = new QRadioButton(ExportConfluenceDialog);
        buttonGroup = new QButtonGroup(ExportConfluenceDialog);
        buttonGroup->setObjectName(QString::fromUtf8("buttonGroup"));
        buttonGroup->addButton(updatePageButton);
        updatePageButton->setObjectName(QString::fromUtf8("updatePageButton"));

        verticalLayout_2->addWidget(updatePageButton);

        createPageButton = new QRadioButton(ExportConfluenceDialog);
        buttonGroup->addButton(createPageButton);
        createPageButton->setObjectName(QString::fromUtf8("createPageButton"));

        verticalLayout_2->addWidget(createPageButton);

        URLLabel = new QLabel(ExportConfluenceDialog);
        URLLabel->setObjectName(QString::fromUtf8("URLLabel"));

        verticalLayout_2->addWidget(URLLabel);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        lineEditURL = new QLineEdit(ExportConfluenceDialog);
        lineEditURL->setObjectName(QString::fromUtf8("lineEditURL"));
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(lineEditURL->sizePolicy().hasHeightForWidth());
        lineEditURL->setSizePolicy(sizePolicy);

        gridLayout->addWidget(lineEditURL, 0, 0, 1, 1);

        openPageButton = new QPushButton(ExportConfluenceDialog);
        openPageButton->setObjectName(QString::fromUtf8("openPageButton"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/flag-url.svg"), QSize(), QIcon::Normal, QIcon::Off);
        openPageButton->setIcon(icon);

        gridLayout->addWidget(openPageButton, 0, 1, 1, 1);

        pageNameLabel = new QLabel(ExportConfluenceDialog);
        pageNameLabel->setObjectName(QString::fromUtf8("pageNameLabel"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pageNameLabel->sizePolicy().hasHeightForWidth());
        pageNameLabel->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(pageNameLabel, 1, 0, 1, 1);

        lineEditPageName = new QLineEdit(ExportConfluenceDialog);
        lineEditPageName->setObjectName(QString::fromUtf8("lineEditPageName"));
        sizePolicy.setHeightForWidth(lineEditPageName->sizePolicy().hasHeightForWidth());
        lineEditPageName->setSizePolicy(sizePolicy);

        gridLayout->addWidget(lineEditPageName, 2, 0, 1, 1);


        verticalLayout_2->addLayout(gridLayout);

        groupBox = new QGroupBox(ExportConfluenceDialog);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        mapCenterToPageNameCheckBox = new QCheckBox(groupBox);
        mapCenterToPageNameCheckBox->setObjectName(QString::fromUtf8("mapCenterToPageNameCheckBox"));

        verticalLayout->addWidget(mapCenterToPageNameCheckBox);

        includeMapImageCheckBox = new QCheckBox(groupBox);
        includeMapImageCheckBox->setObjectName(QString::fromUtf8("includeMapImageCheckBox"));

        verticalLayout->addWidget(includeMapImageCheckBox);

        includeImagesCheckBox = new QCheckBox(groupBox);
        includeImagesCheckBox->setObjectName(QString::fromUtf8("includeImagesCheckBox"));

        verticalLayout->addWidget(includeImagesCheckBox);

        textColorCheckBox = new QCheckBox(groupBox);
        textColorCheckBox->setObjectName(QString::fromUtf8("textColorCheckBox"));

        verticalLayout->addWidget(textColorCheckBox);

        useNumberingCheckBox = new QCheckBox(groupBox);
        useNumberingCheckBox->setObjectName(QString::fromUtf8("useNumberingCheckBox"));

        verticalLayout->addWidget(useNumberingCheckBox);

        saveSettingsInMapCheckBox = new QCheckBox(groupBox);
        saveSettingsInMapCheckBox->setObjectName(QString::fromUtf8("saveSettingsInMapCheckBox"));

        verticalLayout->addWidget(saveSettingsInMapCheckBox);


        verticalLayout_2->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        spacer = new QSpacerItem(258, 21, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(spacer);

        cancelButton = new QPushButton(ExportConfluenceDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        horizontalLayout->addWidget(cancelButton);

        exportButton = new QPushButton(ExportConfluenceDialog);
        exportButton->setObjectName(QString::fromUtf8("exportButton"));

        horizontalLayout->addWidget(exportButton);


        verticalLayout_2->addLayout(horizontalLayout);


        retranslateUi(ExportConfluenceDialog);
        QObject::connect(cancelButton, SIGNAL(clicked()), ExportConfluenceDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(ExportConfluenceDialog);
    } // setupUi

    void retranslateUi(QDialog *ExportConfluenceDialog)
    {
        ExportConfluenceDialog->setWindowTitle(QCoreApplication::translate("ExportConfluenceDialog", "Export Confluence", nullptr));
        exportModeLabel->setText(QCoreApplication::translate("ExportConfluenceDialog", "When exporting to confluence, do", nullptr));
        updatePageButton->setText(QCoreApplication::translate("ExportConfluenceDialog", "update existing page", nullptr));
        createPageButton->setText(QCoreApplication::translate("ExportConfluenceDialog", "create new page", nullptr));
        URLLabel->setText(QCoreApplication::translate("ExportConfluenceDialog", "Page URL:", nullptr));
        openPageButton->setText(QCoreApplication::translate("ExportConfluenceDialog", "Open", nullptr));
        pageNameLabel->setText(QCoreApplication::translate("ExportConfluenceDialog", "Page name:", nullptr));
        groupBox->setTitle(QCoreApplication::translate("ExportConfluenceDialog", "Options", nullptr));
        mapCenterToPageNameCheckBox->setText(QCoreApplication::translate("ExportConfluenceDialog", "Use heading of first map center as page name", nullptr));
        includeMapImageCheckBox->setText(QCoreApplication::translate("ExportConfluenceDialog", "Image of map on top of page", nullptr));
        includeImagesCheckBox->setText(QCoreApplication::translate("ExportConfluenceDialog", "Images used in map", nullptr));
        textColorCheckBox->setText(QCoreApplication::translate("ExportConfluenceDialog", "Colored headings in text", nullptr));
        useNumberingCheckBox->setText(QCoreApplication::translate("ExportConfluenceDialog", "Use numbering", nullptr));
        saveSettingsInMapCheckBox->setText(QCoreApplication::translate("ExportConfluenceDialog", "Save settings in map", nullptr));
        cancelButton->setText(QCoreApplication::translate("ExportConfluenceDialog", "Cancel", nullptr));
        exportButton->setText(QCoreApplication::translate("ExportConfluenceDialog", "Export", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExportConfluenceDialog: public Ui_ExportConfluenceDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORT_2D_CONFLUENCE_2D_DIALOG_H
