/********************************************************************************
** Form generated from reading UI file 'jira-settings-dialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_JIRA_2D_SETTINGS_2D_DIALOG_H
#define UI_JIRA_2D_SETTINGS_2D_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_JiraSettingsDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *UrlLabel;
    QTableWidget *tableWidget;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *addServerButton;
    QPushButton *deleteServerButton;
    QLabel *selectedServerLabel;
    QLineEdit *selectedServerLineEdit;
    QCheckBox *usePATCheckBox;
    QLabel *PATLabel;
    QLineEdit *PATLineEdit;
    QLabel *userLabel;
    QLineEdit *userLineEdit;
    QLabel *passwordLabel;
    QLineEdit *passwordLineEdit;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *JiraSettingsDialog)
    {
        if (JiraSettingsDialog->objectName().isEmpty())
            JiraSettingsDialog->setObjectName(QString::fromUtf8("JiraSettingsDialog"));
        JiraSettingsDialog->resize(904, 509);
        verticalLayout = new QVBoxLayout(JiraSettingsDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        UrlLabel = new QLabel(JiraSettingsDialog);
        UrlLabel->setObjectName(QString::fromUtf8("UrlLabel"));

        verticalLayout->addWidget(UrlLabel);

        tableWidget = new QTableWidget(JiraSettingsDialog);
        tableWidget->setObjectName(QString::fromUtf8("tableWidget"));

        verticalLayout->addWidget(tableWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        addServerButton = new QPushButton(JiraSettingsDialog);
        addServerButton->setObjectName(QString::fromUtf8("addServerButton"));

        horizontalLayout->addWidget(addServerButton);

        deleteServerButton = new QPushButton(JiraSettingsDialog);
        deleteServerButton->setObjectName(QString::fromUtf8("deleteServerButton"));

        horizontalLayout->addWidget(deleteServerButton);


        verticalLayout->addLayout(horizontalLayout);

        selectedServerLabel = new QLabel(JiraSettingsDialog);
        selectedServerLabel->setObjectName(QString::fromUtf8("selectedServerLabel"));

        verticalLayout->addWidget(selectedServerLabel);

        selectedServerLineEdit = new QLineEdit(JiraSettingsDialog);
        selectedServerLineEdit->setObjectName(QString::fromUtf8("selectedServerLineEdit"));
        selectedServerLineEdit->setEnabled(false);
        selectedServerLineEdit->setInputMask(QString::fromUtf8(""));
        selectedServerLineEdit->setText(QString::fromUtf8(""));
        selectedServerLineEdit->setReadOnly(true);

        verticalLayout->addWidget(selectedServerLineEdit);

        usePATCheckBox = new QCheckBox(JiraSettingsDialog);
        usePATCheckBox->setObjectName(QString::fromUtf8("usePATCheckBox"));

        verticalLayout->addWidget(usePATCheckBox);

        PATLabel = new QLabel(JiraSettingsDialog);
        PATLabel->setObjectName(QString::fromUtf8("PATLabel"));

        verticalLayout->addWidget(PATLabel);

        PATLineEdit = new QLineEdit(JiraSettingsDialog);
        PATLineEdit->setObjectName(QString::fromUtf8("PATLineEdit"));
        PATLineEdit->setInputMethodHints(Qt::ImhNone);
        PATLineEdit->setEchoMode(QLineEdit::Password);

        verticalLayout->addWidget(PATLineEdit);

        userLabel = new QLabel(JiraSettingsDialog);
        userLabel->setObjectName(QString::fromUtf8("userLabel"));

        verticalLayout->addWidget(userLabel);

        userLineEdit = new QLineEdit(JiraSettingsDialog);
        userLineEdit->setObjectName(QString::fromUtf8("userLineEdit"));

        verticalLayout->addWidget(userLineEdit);

        passwordLabel = new QLabel(JiraSettingsDialog);
        passwordLabel->setObjectName(QString::fromUtf8("passwordLabel"));

        verticalLayout->addWidget(passwordLabel);

        passwordLineEdit = new QLineEdit(JiraSettingsDialog);
        passwordLineEdit->setObjectName(QString::fromUtf8("passwordLineEdit"));
        passwordLineEdit->setInputMethodHints(Qt::ImhNone);
        passwordLineEdit->setEchoMode(QLineEdit::Password);

        verticalLayout->addWidget(passwordLineEdit);

        buttonBox = new QDialogButtonBox(JiraSettingsDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setLayoutDirection(Qt::LeftToRight);
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);
        buttonBox->setCenterButtons(false);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(JiraSettingsDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), JiraSettingsDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), JiraSettingsDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(JiraSettingsDialog);
    } // setupUi

    void retranslateUi(QDialog *JiraSettingsDialog)
    {
        JiraSettingsDialog->setWindowTitle(QCoreApplication::translate("JiraSettingsDialog", "Dialog", nullptr));
        UrlLabel->setText(QCoreApplication::translate("JiraSettingsDialog", "JIRA servers:", nullptr));
        addServerButton->setText(QCoreApplication::translate("JiraSettingsDialog", "Add", nullptr));
        deleteServerButton->setText(QCoreApplication::translate("JiraSettingsDialog", "Remove", nullptr));
        selectedServerLabel->setText(QCoreApplication::translate("JiraSettingsDialog", "Selected Jira server (click above):", nullptr));
        usePATCheckBox->setText(QCoreApplication::translate("JiraSettingsDialog", "Use Personal Access Token instead of username and password for authentication", nullptr));
        PATLabel->setText(QCoreApplication::translate("JiraSettingsDialog", "Personal Access Token:", nullptr));
        userLabel->setText(QCoreApplication::translate("JiraSettingsDialog", "Username:", nullptr));
        passwordLabel->setText(QCoreApplication::translate("JiraSettingsDialog", "Password:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class JiraSettingsDialog: public Ui_JiraSettingsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_JIRA_2D_SETTINGS_2D_DIALOG_H
