/********************************************************************************
** Form generated from reading UI file 'historywindow.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HISTORYWINDOW_H
#define UI_HISTORYWINDOW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>

QT_BEGIN_NAMESPACE

class Ui_HistoryWindow
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *spacerItem;
    QPushButton *redoButton;
    QSpacerItem *spacerItem1;
    QTableWidget *historyTable;
    QPushButton *undoButton;
    QHBoxLayout *hboxLayout;

    void setupUi(QDialog *HistoryWindow)
    {
        if (HistoryWindow->objectName().isEmpty())
            HistoryWindow->setObjectName(QString::fromUtf8("HistoryWindow"));
        HistoryWindow->resize(1160, 340);
        gridLayout = new QGridLayout(HistoryWindow);
        gridLayout->setSpacing(6);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(spacerItem, 0, 0, 1, 1);

        redoButton = new QPushButton(HistoryWindow);
        redoButton->setObjectName(QString::fromUtf8("redoButton"));

        gridLayout->addWidget(redoButton, 2, 0, 1, 1);

        spacerItem1 = new QSpacerItem(20, 61, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(spacerItem1, 3, 0, 1, 1);

        historyTable = new QTableWidget(HistoryWindow);
        if (historyTable->columnCount() < 2)
            historyTable->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        historyTable->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        historyTable->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        if (historyTable->rowCount() < 1)
            historyTable->setRowCount(1);
        historyTable->setObjectName(QString::fromUtf8("historyTable"));

        gridLayout->addWidget(historyTable, 0, 1, 4, 1);

        undoButton = new QPushButton(HistoryWindow);
        undoButton->setObjectName(QString::fromUtf8("undoButton"));

        gridLayout->addWidget(undoButton, 1, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);

        gridLayout->addLayout(hboxLayout, 4, 0, 1, 2);


        retranslateUi(HistoryWindow);

        QMetaObject::connectSlotsByName(HistoryWindow);
    } // setupUi

    void retranslateUi(QDialog *HistoryWindow)
    {
        HistoryWindow->setWindowTitle(QCoreApplication::translate("HistoryWindow", "Dialog", nullptr));
        redoButton->setText(QCoreApplication::translate("HistoryWindow", "Redo", nullptr));
        QTableWidgetItem *___qtablewidgetitem = historyTable->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("HistoryWindow", "Time", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = historyTable->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("HistoryWindow", "Comment", nullptr));
        undoButton->setText(QCoreApplication::translate("HistoryWindow", "Undo", nullptr));
    } // retranslateUi

};

namespace Ui {
    class HistoryWindow: public Ui_HistoryWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HISTORYWINDOW_H
