/****************************************************************************
** Meta object code from reading C++ file 'taskeditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/taskeditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'taskeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TaskEditor_t {
    QByteArrayData data[16];
    char stringdata0[242];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TaskEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TaskEditor_t qt_meta_stringdata_TaskEditor = {
    {
QT_MOC_LITERAL(0, 0, 10), // "TaskEditor"
QT_MOC_LITERAL(1, 11, 11), // "cellClicked"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 11), // "QModelIndex"
QT_MOC_LITERAL(4, 36, 17), // "headerContextMenu"
QT_MOC_LITERAL(5, 54, 18), // "updateColumnLayout"
QT_MOC_LITERAL(6, 73, 16), // "selectionChanged"
QT_MOC_LITERAL(7, 90, 14), // "QItemSelection"
QT_MOC_LITERAL(8, 105, 8), // "selected"
QT_MOC_LITERAL(9, 114, 15), // "toggleFilterMap"
QT_MOC_LITERAL(10, 130, 18), // "toggleFilterActive"
QT_MOC_LITERAL(11, 149, 15), // "toggleFilterNew"
QT_MOC_LITERAL(12, 165, 19), // "toggleFilterBlocker"
QT_MOC_LITERAL(13, 185, 18), // "toggleFilterFlags1"
QT_MOC_LITERAL(14, 204, 18), // "toggleFilterFlags2"
QT_MOC_LITERAL(15, 223, 18) // "toggleFilterFlags3"

    },
    "TaskEditor\0cellClicked\0\0QModelIndex\0"
    "headerContextMenu\0updateColumnLayout\0"
    "selectionChanged\0QItemSelection\0"
    "selected\0toggleFilterMap\0toggleFilterActive\0"
    "toggleFilterNew\0toggleFilterBlocker\0"
    "toggleFilterFlags1\0toggleFilterFlags2\0"
    "toggleFilterFlags3"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TaskEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x08 /* Private */,
       4,    0,   72,    2, 0x08 /* Private */,
       5,    0,   73,    2, 0x08 /* Private */,
       6,    2,   74,    2, 0x08 /* Private */,
       9,    0,   79,    2, 0x08 /* Private */,
      10,    0,   80,    2, 0x08 /* Private */,
      11,    0,   81,    2, 0x08 /* Private */,
      12,    0,   82,    2, 0x08 /* Private */,
      13,    0,   83,    2, 0x08 /* Private */,
      14,    0,   84,    2, 0x08 /* Private */,
      15,    0,   85,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 7,    8,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void TaskEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TaskEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cellClicked((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 1: _t->headerContextMenu(); break;
        case 2: _t->updateColumnLayout(); break;
        case 3: _t->selectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 4: _t->toggleFilterMap(); break;
        case 5: _t->toggleFilterActive(); break;
        case 6: _t->toggleFilterNew(); break;
        case 7: _t->toggleFilterBlocker(); break;
        case 8: _t->toggleFilterFlags1(); break;
        case 9: _t->toggleFilterFlags2(); break;
        case 10: _t->toggleFilterFlags3(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TaskEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_TaskEditor.data,
    qt_meta_data_TaskEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TaskEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TaskEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TaskEditor.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TaskEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
