/********************************************************************************
** Form generated from reading UI file 'editxlinkdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITXLINKDIALOG_H
#define UI_EDITXLINKDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include "src/linkablemapobj.h"
#include "src/xlinkitem.h"

QT_BEGIN_NAMESPACE

class Ui_EditXLinkDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QSpinBox *widthBox;
    QLabel *textLabel1;
    QPushButton *setColorHeadingButton;
    QPushButton *colorButton;
    QCheckBox *useSettings;
    QLabel *textLabel2;
    QSpacerItem *spacerItem;
    QSpacerItem *spacerItem1;
    QLabel *textLabel1_2;
    QSpacerItem *spacerItem2;
    QLabel *label;
    QComboBox *lineStyleCombo;
    QLabel *label_3;
    QCheckBox *checkBoxArrowBegin;
    QCheckBox *checkBoxArrowEnd;
    QSpacerItem *spacerItem3;
    QHBoxLayout *boxLayout;
    QSpacerItem *spacerItem4;
    QPushButton *okButton;

    void setupUi(QDialog *EditXLinkDialog)
    {
        if (EditXLinkDialog->objectName().isEmpty())
            EditXLinkDialog->setObjectName(QString::fromUtf8("EditXLinkDialog"));
        EditXLinkDialog->resize(337, 219);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(EditXLinkDialog->sizePolicy().hasHeightForWidth());
        EditXLinkDialog->setSizePolicy(sizePolicy);
        vboxLayout = new QVBoxLayout(EditXLinkDialog);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_2 = new QLabel(EditXLinkDialog);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 3, 0, 1, 1);

        widthBox = new QSpinBox(EditXLinkDialog);
        widthBox->setObjectName(QString::fromUtf8("widthBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(widthBox->sizePolicy().hasHeightForWidth());
        widthBox->setSizePolicy(sizePolicy1);
        widthBox->setMinimum(1);
        widthBox->setMaximum(15);

        gridLayout->addWidget(widthBox, 0, 1, 1, 1);

        textLabel1 = new QLabel(EditXLinkDialog);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        textLabel1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        setColorHeadingButton = new QPushButton(EditXLinkDialog);
        setColorHeadingButton->setObjectName(QString::fromUtf8("setColorHeadingButton"));
        setColorHeadingButton->setMaximumSize(QSize(32767, 20));

        gridLayout->addWidget(setColorHeadingButton, 1, 4, 1, 1);

        colorButton = new QPushButton(EditXLinkDialog);
        colorButton->setObjectName(QString::fromUtf8("colorButton"));
        colorButton->setEnabled(true);
        colorButton->setMaximumSize(QSize(50, 20));

        gridLayout->addWidget(colorButton, 1, 1, 1, 1);

        useSettings = new QCheckBox(EditXLinkDialog);
        useSettings->setObjectName(QString::fromUtf8("useSettings"));

        gridLayout->addWidget(useSettings, 5, 1, 1, 2);

        textLabel2 = new QLabel(EditXLinkDialog);
        textLabel2->setObjectName(QString::fromUtf8("textLabel2"));
        textLabel2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabel2, 1, 0, 1, 1);

        spacerItem = new QSpacerItem(285, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(spacerItem, 1, 2, 1, 2);

        spacerItem1 = new QSpacerItem(280, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(spacerItem1, 0, 2, 1, 2);

        textLabel1_2 = new QLabel(EditXLinkDialog);
        textLabel1_2->setObjectName(QString::fromUtf8("textLabel1_2"));
        textLabel1_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabel1_2, 5, 0, 1, 1);

        spacerItem2 = new QSpacerItem(241, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(spacerItem2, 5, 3, 1, 2);

        label = new QLabel(EditXLinkDialog);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 2, 0, 1, 1);

        lineStyleCombo = new QComboBox(EditXLinkDialog);
        lineStyleCombo->setObjectName(QString::fromUtf8("lineStyleCombo"));

        gridLayout->addWidget(lineStyleCombo, 2, 1, 1, 3);

        label_3 = new QLabel(EditXLinkDialog);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 4, 0, 1, 1);

        checkBoxArrowBegin = new QCheckBox(EditXLinkDialog);
        checkBoxArrowBegin->setObjectName(QString::fromUtf8("checkBoxArrowBegin"));

        gridLayout->addWidget(checkBoxArrowBegin, 3, 1, 1, 1);

        checkBoxArrowEnd = new QCheckBox(EditXLinkDialog);
        checkBoxArrowEnd->setObjectName(QString::fromUtf8("checkBoxArrowEnd"));

        gridLayout->addWidget(checkBoxArrowEnd, 4, 1, 1, 1);


        vboxLayout->addLayout(gridLayout);

        spacerItem3 = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout->addItem(spacerItem3);

        boxLayout = new QHBoxLayout();
        boxLayout->setSpacing(6);
        boxLayout->setContentsMargins(0, 0, 0, 0);
        boxLayout->setObjectName(QString::fromUtf8("boxLayout"));
        spacerItem4 = new QSpacerItem(100, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        boxLayout->addItem(spacerItem4);

        okButton = new QPushButton(EditXLinkDialog);
        okButton->setObjectName(QString::fromUtf8("okButton"));

        boxLayout->addWidget(okButton);


        vboxLayout->addLayout(boxLayout);


        retranslateUi(EditXLinkDialog);
        QObject::connect(okButton, SIGNAL(clicked()), EditXLinkDialog, SLOT(accept()));

        QMetaObject::connectSlotsByName(EditXLinkDialog);
    } // setupUi

    void retranslateUi(QDialog *EditXLinkDialog)
    {
        EditXLinkDialog->setWindowTitle(QCoreApplication::translate("EditXLinkDialog", "Edit XLink", nullptr));
        label_2->setText(QCoreApplication::translate("EditXLinkDialog", "Arrow begin:", nullptr));
        textLabel1->setText(QCoreApplication::translate("EditXLinkDialog", "XLink width:", nullptr));
        setColorHeadingButton->setText(QCoreApplication::translate("EditXLinkDialog", "Use color of heading", nullptr));
        colorButton->setText(QString());
        useSettings->setText(QString());
        textLabel2->setText(QCoreApplication::translate("EditXLinkDialog", "XLink color:", nullptr));
        textLabel1_2->setText(QCoreApplication::translate("EditXLinkDialog", "Use as default:", nullptr));
        label->setText(QCoreApplication::translate("EditXLinkDialog", "Line style:", nullptr));
        label_3->setText(QCoreApplication::translate("EditXLinkDialog", "Arrow end:", nullptr));
        checkBoxArrowBegin->setText(QString());
        checkBoxArrowEnd->setText(QString());
        okButton->setText(QCoreApplication::translate("EditXLinkDialog", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditXLinkDialog: public Ui_EditXLinkDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITXLINKDIALOG_H
