/****************************************************************************
** Meta object code from reading C++ file 'slidecontrolwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/slidecontrolwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'slidecontrolwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SlideControlWidget_t {
    QByteArrayData data[17];
    char stringdata0[252];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SlideControlWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SlideControlWidget_t qt_meta_stringdata_SlideControlWidget = {
    {
QT_MOC_LITERAL(0, 0, 18), // "SlideControlWidget"
QT_MOC_LITERAL(1, 19, 14), // "hideFindWidget"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 12), // "takeSnapshot"
QT_MOC_LITERAL(4, 48, 19), // "deleteButtonPressed"
QT_MOC_LITERAL(5, 68, 17), // "editButtonPressed"
QT_MOC_LITERAL(6, 86, 21), // "previousButtonPressed"
QT_MOC_LITERAL(7, 108, 17), // "nextButtonPressed"
QT_MOC_LITERAL(8, 126, 15), // "upButtonPressed"
QT_MOC_LITERAL(9, 142, 17), // "downButtonPressed"
QT_MOC_LITERAL(10, 160, 15), // "previousPressed"
QT_MOC_LITERAL(11, 176, 15), // "snapshotPressed"
QT_MOC_LITERAL(12, 192, 11), // "nextPressed"
QT_MOC_LITERAL(13, 204, 9), // "upPressed"
QT_MOC_LITERAL(14, 214, 11), // "downPressed"
QT_MOC_LITERAL(15, 226, 13), // "deletePressed"
QT_MOC_LITERAL(16, 240, 11) // "editPressed"

    },
    "SlideControlWidget\0hideFindWidget\0\0"
    "takeSnapshot\0deleteButtonPressed\0"
    "editButtonPressed\0previousButtonPressed\0"
    "nextButtonPressed\0upButtonPressed\0"
    "downButtonPressed\0previousPressed\0"
    "snapshotPressed\0nextPressed\0upPressed\0"
    "downPressed\0deletePressed\0editPressed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SlideControlWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   89,    2, 0x06 /* Public */,
       3,    0,   90,    2, 0x06 /* Public */,
       4,    0,   91,    2, 0x06 /* Public */,
       5,    0,   92,    2, 0x06 /* Public */,
       6,    0,   93,    2, 0x06 /* Public */,
       7,    0,   94,    2, 0x06 /* Public */,
       8,    0,   95,    2, 0x06 /* Public */,
       9,    0,   96,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,   97,    2, 0x0a /* Public */,
      11,    0,   98,    2, 0x0a /* Public */,
      12,    0,   99,    2, 0x0a /* Public */,
      13,    0,  100,    2, 0x0a /* Public */,
      14,    0,  101,    2, 0x0a /* Public */,
      15,    0,  102,    2, 0x0a /* Public */,
      16,    0,  103,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SlideControlWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SlideControlWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->hideFindWidget(); break;
        case 1: _t->takeSnapshot(); break;
        case 2: _t->deleteButtonPressed(); break;
        case 3: _t->editButtonPressed(); break;
        case 4: _t->previousButtonPressed(); break;
        case 5: _t->nextButtonPressed(); break;
        case 6: _t->upButtonPressed(); break;
        case 7: _t->downButtonPressed(); break;
        case 8: _t->previousPressed(); break;
        case 9: _t->snapshotPressed(); break;
        case 10: _t->nextPressed(); break;
        case 11: _t->upPressed(); break;
        case 12: _t->downPressed(); break;
        case 13: _t->deletePressed(); break;
        case 14: _t->editPressed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SlideControlWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SlideControlWidget::hideFindWidget)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SlideControlWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SlideControlWidget::takeSnapshot)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SlideControlWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SlideControlWidget::deleteButtonPressed)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (SlideControlWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SlideControlWidget::editButtonPressed)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (SlideControlWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SlideControlWidget::previousButtonPressed)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (SlideControlWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SlideControlWidget::nextButtonPressed)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (SlideControlWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SlideControlWidget::upButtonPressed)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (SlideControlWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SlideControlWidget::downButtonPressed)) {
                *result = 7;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SlideControlWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_SlideControlWidget.data,
    qt_meta_data_SlideControlWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SlideControlWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SlideControlWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SlideControlWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int SlideControlWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void SlideControlWidget::hideFindWidget()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SlideControlWidget::takeSnapshot()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void SlideControlWidget::deleteButtonPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void SlideControlWidget::editButtonPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void SlideControlWidget::previousButtonPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void SlideControlWidget::nextButtonPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void SlideControlWidget::upButtonPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void SlideControlWidget::downButtonPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
