/********************************************************************************
** Form generated from reading UI file 'warningdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WARNINGDIALOG_H
#define UI_WARNINGDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_WarningDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTextEdit *warningTE;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *hboxLayout;
    QCheckBox *showAgainBox;
    QSpacerItem *spacerItem;
    QPushButton *cancelButton;
    QPushButton *okButton;

    void setupUi(QDialog *WarningDialog)
    {
        if (WarningDialog->objectName().isEmpty())
            WarningDialog->setObjectName(QString::fromUtf8("WarningDialog"));
        WarningDialog->resize(545, 256);
        verticalLayout = new QVBoxLayout(WarningDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        warningTE = new QTextEdit(WarningDialog);
        warningTE->setObjectName(QString::fromUtf8("warningTE"));
        warningTE->setFrameShape(QFrame::NoFrame);
        warningTE->setFrameShadow(QFrame::Plain);
        warningTE->setReadOnly(true);

        verticalLayout->addWidget(warningTE);

        verticalSpacer = new QSpacerItem(20, 26, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        showAgainBox = new QCheckBox(WarningDialog);
        showAgainBox->setObjectName(QString::fromUtf8("showAgainBox"));
        showAgainBox->setChecked(true);

        hboxLayout->addWidget(showAgainBox);

        spacerItem = new QSpacerItem(73, 31, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        cancelButton = new QPushButton(WarningDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        hboxLayout->addWidget(cancelButton);

        okButton = new QPushButton(WarningDialog);
        okButton->setObjectName(QString::fromUtf8("okButton"));

        hboxLayout->addWidget(okButton);


        verticalLayout->addLayout(hboxLayout);


        retranslateUi(WarningDialog);
        QObject::connect(okButton, SIGNAL(pressed()), WarningDialog, SLOT(accept()));
        QObject::connect(cancelButton, SIGNAL(pressed()), WarningDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(WarningDialog);
    } // setupUi

    void retranslateUi(QDialog *WarningDialog)
    {
        WarningDialog->setWindowTitle(QCoreApplication::translate("WarningDialog", "Dialog", nullptr));
        showAgainBox->setText(QCoreApplication::translate("WarningDialog", "showAgainBox", nullptr));
        cancelButton->setText(QCoreApplication::translate("WarningDialog", "Cancel", nullptr));
        okButton->setText(QCoreApplication::translate("WarningDialog", "Ok", nullptr));
    } // retranslateUi

};

namespace Ui {
    class WarningDialog: public Ui_WarningDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WARNINGDIALOG_H
