/********************************************************************************
** Form generated from reading UI file 'confluence-settings-dialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFLUENCE_2D_SETTINGS_2D_DIALOG_H
#define UI_CONFLUENCE_2D_SETTINGS_2D_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ConfluenceSettingsDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *UrlLabel;
    QLineEdit *urlLineEdit;
    QCheckBox *usePATCheckBox;
    QLabel *PATLabel;
    QLineEdit *PATLineEdit;
    QLabel *userLabel;
    QLineEdit *userLineEdit;
    QLabel *passwordLabel;
    QLineEdit *passwordLineEdit;
    QCheckBox *savePasswordCheckBox;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ConfluenceSettingsDialog)
    {
        if (ConfluenceSettingsDialog->objectName().isEmpty())
            ConfluenceSettingsDialog->setObjectName(QString::fromUtf8("ConfluenceSettingsDialog"));
        ConfluenceSettingsDialog->resize(631, 356);
        verticalLayout = new QVBoxLayout(ConfluenceSettingsDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        UrlLabel = new QLabel(ConfluenceSettingsDialog);
        UrlLabel->setObjectName(QString::fromUtf8("UrlLabel"));

        verticalLayout->addWidget(UrlLabel);

        urlLineEdit = new QLineEdit(ConfluenceSettingsDialog);
        urlLineEdit->setObjectName(QString::fromUtf8("urlLineEdit"));

        verticalLayout->addWidget(urlLineEdit);

        usePATCheckBox = new QCheckBox(ConfluenceSettingsDialog);
        usePATCheckBox->setObjectName(QString::fromUtf8("usePATCheckBox"));

        verticalLayout->addWidget(usePATCheckBox);

        PATLabel = new QLabel(ConfluenceSettingsDialog);
        PATLabel->setObjectName(QString::fromUtf8("PATLabel"));

        verticalLayout->addWidget(PATLabel);

        PATLineEdit = new QLineEdit(ConfluenceSettingsDialog);
        PATLineEdit->setObjectName(QString::fromUtf8("PATLineEdit"));
        PATLineEdit->setEchoMode(QLineEdit::Password);

        verticalLayout->addWidget(PATLineEdit);

        userLabel = new QLabel(ConfluenceSettingsDialog);
        userLabel->setObjectName(QString::fromUtf8("userLabel"));

        verticalLayout->addWidget(userLabel);

        userLineEdit = new QLineEdit(ConfluenceSettingsDialog);
        userLineEdit->setObjectName(QString::fromUtf8("userLineEdit"));

        verticalLayout->addWidget(userLineEdit);

        passwordLabel = new QLabel(ConfluenceSettingsDialog);
        passwordLabel->setObjectName(QString::fromUtf8("passwordLabel"));

        verticalLayout->addWidget(passwordLabel);

        passwordLineEdit = new QLineEdit(ConfluenceSettingsDialog);
        passwordLineEdit->setObjectName(QString::fromUtf8("passwordLineEdit"));
        passwordLineEdit->setEchoMode(QLineEdit::Password);

        verticalLayout->addWidget(passwordLineEdit);

        savePasswordCheckBox = new QCheckBox(ConfluenceSettingsDialog);
        savePasswordCheckBox->setObjectName(QString::fromUtf8("savePasswordCheckBox"));

        verticalLayout->addWidget(savePasswordCheckBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(ConfluenceSettingsDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setLayoutDirection(Qt::LeftToRight);
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        buttonBox->setCenterButtons(false);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(ConfluenceSettingsDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), ConfluenceSettingsDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), ConfluenceSettingsDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(ConfluenceSettingsDialog);
    } // setupUi

    void retranslateUi(QDialog *ConfluenceSettingsDialog)
    {
        ConfluenceSettingsDialog->setWindowTitle(QCoreApplication::translate("ConfluenceSettingsDialog", "Dialog", nullptr));
        UrlLabel->setText(QCoreApplication::translate("ConfluenceSettingsDialog", "Confluence URL:", nullptr));
        usePATCheckBox->setText(QCoreApplication::translate("ConfluenceSettingsDialog", "Use Personal Access Token instead of username and password for authentication", nullptr));
        PATLabel->setText(QCoreApplication::translate("ConfluenceSettingsDialog", "Personal Access Token:", nullptr));
        userLabel->setText(QCoreApplication::translate("ConfluenceSettingsDialog", "Username:", nullptr));
        passwordLabel->setText(QCoreApplication::translate("ConfluenceSettingsDialog", "Password:", nullptr));
        savePasswordCheckBox->setText(QCoreApplication::translate("ConfluenceSettingsDialog", "Save password", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ConfluenceSettingsDialog: public Ui_ConfluenceSettingsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONFLUENCE_2D_SETTINGS_2D_DIALOG_H
