/********************************************************************************
** Form generated from reading UI file 'lockedfiledialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOCKEDFILEDIALOG_H
#define UI_LOCKEDFILEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_LockedFileDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTextEdit *warningTE;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *deleteLockfileButton;
    QPushButton *openReadonlyButton;

    void setupUi(QDialog *LockedFileDialog)
    {
        if (LockedFileDialog->objectName().isEmpty())
            LockedFileDialog->setObjectName(QString::fromUtf8("LockedFileDialog"));
        LockedFileDialog->resize(557, 177);
        verticalLayout = new QVBoxLayout(LockedFileDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        warningTE = new QTextEdit(LockedFileDialog);
        warningTE->setObjectName(QString::fromUtf8("warningTE"));
        warningTE->setEnabled(true);
        warningTE->setAutoFillBackground(false);
        warningTE->setFrameShape(QFrame::NoFrame);
        warningTE->setFrameShadow(QFrame::Plain);
        warningTE->setReadOnly(true);

        verticalLayout->addWidget(warningTE);

        verticalSpacer = new QSpacerItem(20, 26, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(73, 31, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        deleteLockfileButton = new QPushButton(LockedFileDialog);
        deleteLockfileButton->setObjectName(QString::fromUtf8("deleteLockfileButton"));

        hboxLayout->addWidget(deleteLockfileButton);

        openReadonlyButton = new QPushButton(LockedFileDialog);
        openReadonlyButton->setObjectName(QString::fromUtf8("openReadonlyButton"));

        hboxLayout->addWidget(openReadonlyButton);


        verticalLayout->addLayout(hboxLayout);


        retranslateUi(LockedFileDialog);
        QObject::connect(openReadonlyButton, SIGNAL(pressed()), LockedFileDialog, SLOT(accept()));
        QObject::connect(deleteLockfileButton, SIGNAL(pressed()), LockedFileDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(LockedFileDialog);
    } // setupUi

    void retranslateUi(QDialog *LockedFileDialog)
    {
        LockedFileDialog->setWindowTitle(QCoreApplication::translate("LockedFileDialog", "Dialog", nullptr));
        deleteLockfileButton->setText(QCoreApplication::translate("LockedFileDialog", "Delete lockfile", nullptr));
        openReadonlyButton->setText(QCoreApplication::translate("LockedFileDialog", "Open readonly", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LockedFileDialog: public Ui_LockedFileDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LOCKEDFILEDIALOG_H
