/********************************************************************************
** Form generated from reading UI file 'export-html-dialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORT_2D_HTML_2D_DIALOG_H
#define UI_EXPORT_2D_HTML_2D_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "src/vymprocess.h"
#include "src/xsltproc.h"

QT_BEGIN_NAMESPACE

class Ui_ExportHTMLDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel1;
    QLineEdit *lineEditDir;
    QPushButton *browseExportDirButton;
    QGroupBox *groupBox;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QCheckBox *imageCheckBox;
    QCheckBox *includeImagesCheckBox;
    QCheckBox *TOCCheckBox;
    QCheckBox *taskFlagsCheckBox;
    QCheckBox *userFlagsCheckBox;
    QCheckBox *numberingCheckBox;
    QCheckBox *textColorCheckBox;
    QCheckBox *saveSettingsInMapCheckBox;
    QGroupBox *groupBox_3;
    QWidget *layoutWidget1_;
    QGridLayout *gridLayout;
    QCheckBox *copyCssCheckBox;
    QLineEdit *lineEditCssSrc;
    QLineEdit *lineEditCssDst;
    QLabel *label;
    QPushButton *browseCssSrcButton;
    QPushButton *browseCssDstButton;
    QGroupBox *groupBox_2;
    QWidget *layoutWidget2;
    QHBoxLayout *horizontalLayout;
    QLabel *textLabel2_2;
    QLineEdit *lineEditPostScript;
    QPushButton *browsePostExportButton;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem;
    QPushButton *cancelButton;
    QPushButton *exportButton;

    void setupUi(QDialog *ExportHTMLDialog)
    {
        if (ExportHTMLDialog->objectName().isEmpty())
            ExportHTMLDialog->setObjectName(QString::fromUtf8("ExportHTMLDialog"));
        ExportHTMLDialog->resize(629, 547);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ExportHTMLDialog->sizePolicy().hasHeightForWidth());
        ExportHTMLDialog->setSizePolicy(sizePolicy);
        ExportHTMLDialog->setMinimumSize(QSize(0, 130));
        verticalLayout_2 = new QVBoxLayout(ExportHTMLDialog);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        textLabel1 = new QLabel(ExportHTMLDialog);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textLabel1->sizePolicy().hasHeightForWidth());
        textLabel1->setSizePolicy(sizePolicy1);
        textLabel1->setMinimumSize(QSize(125, 0));

        hboxLayout->addWidget(textLabel1);

        lineEditDir = new QLineEdit(ExportHTMLDialog);
        lineEditDir->setObjectName(QString::fromUtf8("lineEditDir"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(lineEditDir->sizePolicy().hasHeightForWidth());
        lineEditDir->setSizePolicy(sizePolicy2);
        lineEditDir->setMinimumSize(QSize(300, 0));

        hboxLayout->addWidget(lineEditDir);

        browseExportDirButton = new QPushButton(ExportHTMLDialog);
        browseExportDirButton->setObjectName(QString::fromUtf8("browseExportDirButton"));
        QSizePolicy sizePolicy3(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(browseExportDirButton->sizePolicy().hasHeightForWidth());
        browseExportDirButton->setSizePolicy(sizePolicy3);

        hboxLayout->addWidget(browseExportDirButton);


        verticalLayout_2->addLayout(hboxLayout);

        groupBox = new QGroupBox(ExportHTMLDialog);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        QSizePolicy sizePolicy4(QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy4);
        groupBox->setMinimumSize(QSize(0, 600));
        groupBox->setMaximumSize(QSize(16777215, 16777214));
        layoutWidget = new QWidget(groupBox);
        layoutWidget->setObjectName(QString::fromUtf8("layoutWidget"));
        layoutWidget->setGeometry(QRect(20, 20, 581, 259));
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        imageCheckBox = new QCheckBox(layoutWidget);
        imageCheckBox->setObjectName(QString::fromUtf8("imageCheckBox"));
        QSizePolicy sizePolicy5(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(imageCheckBox->sizePolicy().hasHeightForWidth());
        imageCheckBox->setSizePolicy(sizePolicy5);

        verticalLayout->addWidget(imageCheckBox);

        includeImagesCheckBox = new QCheckBox(layoutWidget);
        includeImagesCheckBox->setObjectName(QString::fromUtf8("includeImagesCheckBox"));

        verticalLayout->addWidget(includeImagesCheckBox);

        TOCCheckBox = new QCheckBox(layoutWidget);
        TOCCheckBox->setObjectName(QString::fromUtf8("TOCCheckBox"));

        verticalLayout->addWidget(TOCCheckBox);

        taskFlagsCheckBox = new QCheckBox(layoutWidget);
        taskFlagsCheckBox->setObjectName(QString::fromUtf8("taskFlagsCheckBox"));

        verticalLayout->addWidget(taskFlagsCheckBox);

        userFlagsCheckBox = new QCheckBox(layoutWidget);
        userFlagsCheckBox->setObjectName(QString::fromUtf8("userFlagsCheckBox"));

        verticalLayout->addWidget(userFlagsCheckBox);

        numberingCheckBox = new QCheckBox(layoutWidget);
        numberingCheckBox->setObjectName(QString::fromUtf8("numberingCheckBox"));

        verticalLayout->addWidget(numberingCheckBox);

        textColorCheckBox = new QCheckBox(layoutWidget);
        textColorCheckBox->setObjectName(QString::fromUtf8("textColorCheckBox"));
        sizePolicy5.setHeightForWidth(textColorCheckBox->sizePolicy().hasHeightForWidth());
        textColorCheckBox->setSizePolicy(sizePolicy5);

        verticalLayout->addWidget(textColorCheckBox);

        saveSettingsInMapCheckBox = new QCheckBox(layoutWidget);
        saveSettingsInMapCheckBox->setObjectName(QString::fromUtf8("saveSettingsInMapCheckBox"));
        sizePolicy5.setHeightForWidth(saveSettingsInMapCheckBox->sizePolicy().hasHeightForWidth());
        saveSettingsInMapCheckBox->setSizePolicy(sizePolicy5);

        verticalLayout->addWidget(saveSettingsInMapCheckBox);


        verticalLayout_2->addWidget(groupBox);

        groupBox_3 = new QGroupBox(ExportHTMLDialog);
        groupBox_3->setObjectName(QString::fromUtf8("groupBox_3"));
        groupBox_3->setMinimumSize(QSize(0, 100));
        layoutWidget1_ = new QWidget(groupBox_3);
        layoutWidget1_->setObjectName(QString::fromUtf8("layoutWidget1_"));
        layoutWidget1_->setGeometry(QRect(20, 30, 533, 78));
        gridLayout = new QGridLayout(layoutWidget1_);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 0, 0, 0);
        copyCssCheckBox = new QCheckBox(layoutWidget1_);
        copyCssCheckBox->setObjectName(QString::fromUtf8("copyCssCheckBox"));

        gridLayout->addWidget(copyCssCheckBox, 0, 0, 1, 1);

        lineEditCssSrc = new QLineEdit(layoutWidget1_);
        lineEditCssSrc->setObjectName(QString::fromUtf8("lineEditCssSrc"));
        lineEditCssSrc->setMinimumSize(QSize(300, 0));

        gridLayout->addWidget(lineEditCssSrc, 0, 1, 1, 1);

        lineEditCssDst = new QLineEdit(layoutWidget1_);
        lineEditCssDst->setObjectName(QString::fromUtf8("lineEditCssDst"));

        gridLayout->addWidget(lineEditCssDst, 1, 1, 1, 1);

        label = new QLabel(layoutWidget1_);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 1, 0, 1, 1);

        browseCssSrcButton = new QPushButton(layoutWidget1_);
        browseCssSrcButton->setObjectName(QString::fromUtf8("browseCssSrcButton"));
        sizePolicy3.setHeightForWidth(browseCssSrcButton->sizePolicy().hasHeightForWidth());
        browseCssSrcButton->setSizePolicy(sizePolicy3);

        gridLayout->addWidget(browseCssSrcButton, 0, 2, 1, 1);

        browseCssDstButton = new QPushButton(layoutWidget1_);
        browseCssDstButton->setObjectName(QString::fromUtf8("browseCssDstButton"));
        sizePolicy3.setHeightForWidth(browseCssDstButton->sizePolicy().hasHeightForWidth());
        browseCssDstButton->setSizePolicy(sizePolicy3);

        gridLayout->addWidget(browseCssDstButton, 1, 2, 1, 1);


        verticalLayout_2->addWidget(groupBox_3);

        groupBox_2 = new QGroupBox(ExportHTMLDialog);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        groupBox_2->setMinimumSize(QSize(0, 70));
        layoutWidget2 = new QWidget(groupBox_2);
        layoutWidget2->setObjectName(QString::fromUtf8("layoutWidget2"));
        layoutWidget2->setGeometry(QRect(10, 20, 519, 27));
        horizontalLayout = new QHBoxLayout(layoutWidget2);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setContentsMargins(11, 11, 11, 11);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        textLabel2_2 = new QLabel(layoutWidget2);
        textLabel2_2->setObjectName(QString::fromUtf8("textLabel2_2"));
        QSizePolicy sizePolicy6(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(textLabel2_2->sizePolicy().hasHeightForWidth());
        textLabel2_2->setSizePolicy(sizePolicy6);
        textLabel2_2->setMinimumSize(QSize(125, 0));
        textLabel2_2->setFrameShape(QFrame::NoFrame);
        textLabel2_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(textLabel2_2);

        lineEditPostScript = new QLineEdit(layoutWidget2);
        lineEditPostScript->setObjectName(QString::fromUtf8("lineEditPostScript"));
        lineEditPostScript->setMinimumSize(QSize(300, 0));

        horizontalLayout->addWidget(lineEditPostScript);

        browsePostExportButton = new QPushButton(layoutWidget2);
        browsePostExportButton->setObjectName(QString::fromUtf8("browsePostExportButton"));
        sizePolicy3.setHeightForWidth(browsePostExportButton->sizePolicy().hasHeightForWidth());
        browsePostExportButton->setSizePolicy(sizePolicy3);

        horizontalLayout->addWidget(browsePostExportButton);


        verticalLayout_2->addWidget(groupBox_2);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        spacerItem = new QSpacerItem(61, 21, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem);

        cancelButton = new QPushButton(ExportHTMLDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        hboxLayout1->addWidget(cancelButton);

        exportButton = new QPushButton(ExportHTMLDialog);
        exportButton->setObjectName(QString::fromUtf8("exportButton"));

        hboxLayout1->addWidget(exportButton);


        verticalLayout_2->addLayout(hboxLayout1);


        retranslateUi(ExportHTMLDialog);
        QObject::connect(cancelButton, SIGNAL(clicked()), ExportHTMLDialog, SLOT(reject()));
        QObject::connect(exportButton, SIGNAL(clicked()), ExportHTMLDialog, SLOT(accept()));

        exportButton->setDefault(true);


        QMetaObject::connectSlotsByName(ExportHTMLDialog);
    } // setupUi

    void retranslateUi(QDialog *ExportHTMLDialog)
    {
        ExportHTMLDialog->setWindowTitle(QCoreApplication::translate("ExportHTMLDialog", "Export HTML", nullptr));
        textLabel1->setText(QCoreApplication::translate("ExportHTMLDialog", "Export to directory:", nullptr));
        browseExportDirButton->setText(QCoreApplication::translate("ExportHTMLDialog", "Browse", nullptr));
        groupBox->setTitle(QCoreApplication::translate("ExportHTMLDialog", "Options", nullptr));
        imageCheckBox->setText(QCoreApplication::translate("ExportHTMLDialog", "Image of map", nullptr));
        includeImagesCheckBox->setText(QCoreApplication::translate("ExportHTMLDialog", "Include images", nullptr));
        TOCCheckBox->setText(QCoreApplication::translate("ExportHTMLDialog", "Table of contents", nullptr));
        taskFlagsCheckBox->setText(QCoreApplication::translate("ExportHTMLDialog", "Task flags", nullptr));
        userFlagsCheckBox->setText(QCoreApplication::translate("ExportHTMLDialog", "General flags", nullptr));
        numberingCheckBox->setText(QCoreApplication::translate("ExportHTMLDialog", "Number sections", nullptr));
        textColorCheckBox->setText(QCoreApplication::translate("ExportHTMLDialog", "Colored headings in text", nullptr));
        saveSettingsInMapCheckBox->setText(QCoreApplication::translate("ExportHTMLDialog", "Save settings in map", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("ExportHTMLDialog", "Stylesheets", nullptr));
        copyCssCheckBox->setText(QCoreApplication::translate("ExportHTMLDialog", "Copy CSS from ", nullptr));
        label->setText(QCoreApplication::translate("ExportHTMLDialog", "Use CSS from", nullptr));
        browseCssSrcButton->setText(QCoreApplication::translate("ExportHTMLDialog", "Browse", nullptr));
        browseCssDstButton->setText(QCoreApplication::translate("ExportHTMLDialog", "Browse", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("ExportHTMLDialog", "Scripts", nullptr));
        textLabel2_2->setText(QCoreApplication::translate("ExportHTMLDialog", "After Export:", nullptr));
        browsePostExportButton->setText(QCoreApplication::translate("ExportHTMLDialog", "Browse", nullptr));
        cancelButton->setText(QCoreApplication::translate("ExportHTMLDialog", "Cancel", nullptr));
        exportButton->setText(QCoreApplication::translate("ExportHTMLDialog", "Export", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExportHTMLDialog: public Ui_ExportHTMLDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORT_2D_HTML_2D_DIALOG_H
