/****************************************************************************
** Meta object code from reading C++ file 'vymview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/vymview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vymview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_VymView_t {
    QByteArrayData data[16];
    char stringdata0[212];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_VymView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_VymView_t qt_meta_stringdata_VymView = {
    {
QT_MOC_LITERAL(0, 0, 7), // "VymView"
QT_MOC_LITERAL(1, 8, 15), // "changeSelection"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 14), // "QItemSelection"
QT_MOC_LITERAL(4, 40, 6), // "newSel"
QT_MOC_LITERAL(5, 47, 6), // "delSel"
QT_MOC_LITERAL(6, 54, 22), // "updateDockWidgetTitles"
QT_MOC_LITERAL(7, 77, 9), // "expandAll"
QT_MOC_LITERAL(8, 87, 14), // "expandOneLevel"
QT_MOC_LITERAL(9, 102, 16), // "collapseOneLevel"
QT_MOC_LITERAL(10, 119, 18), // "collapseUnselected"
QT_MOC_LITERAL(11, 138, 13), // "showSelection"
QT_MOC_LITERAL(12, 152, 6), // "scaled"
QT_MOC_LITERAL(13, 159, 16), // "toggleTreeEditor"
QT_MOC_LITERAL(14, 176, 17), // "toggleSlideEditor"
QT_MOC_LITERAL(15, 194, 17) // "setFocusMapEditor"

    },
    "VymView\0changeSelection\0\0QItemSelection\0"
    "newSel\0delSel\0updateDockWidgetTitles\0"
    "expandAll\0expandOneLevel\0collapseOneLevel\0"
    "collapseUnselected\0showSelection\0"
    "scaled\0toggleTreeEditor\0toggleSlideEditor\0"
    "setFocusMapEditor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_VymView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   64,    2, 0x0a /* Public */,
       6,    0,   69,    2, 0x0a /* Public */,
       7,    0,   70,    2, 0x0a /* Public */,
       8,    0,   71,    2, 0x0a /* Public */,
       9,    0,   72,    2, 0x0a /* Public */,
      10,    0,   73,    2, 0x0a /* Public */,
      11,    1,   74,    2, 0x0a /* Public */,
      13,    0,   77,    2, 0x0a /* Public */,
      14,    0,   78,    2, 0x0a /* Public */,
      15,    0,   79,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    4,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void VymView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<VymView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->changeSelection((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 1: _t->updateDockWidgetTitles(); break;
        case 2: _t->expandAll(); break;
        case 3: _t->expandOneLevel(); break;
        case 4: _t->collapseOneLevel(); break;
        case 5: _t->collapseUnselected(); break;
        case 6: _t->showSelection((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->toggleTreeEditor(); break;
        case 8: _t->toggleSlideEditor(); break;
        case 9: _t->setFocusMapEditor(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject VymView::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_VymView.data,
    qt_meta_data_VymView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *VymView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VymView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_VymView.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int VymView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
