/****************************************************************************
** Meta object code from reading C++ file 'vymmodelwrapper.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/vymmodelwrapper.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vymmodelwrapper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_VymModelWrapper_t {
    QByteArrayData data[163];
    char stringdata0[1996];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_VymModelWrapper_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_VymModelWrapper_t qt_meta_stringdata_VymModelWrapper = {
    {
QT_MOC_LITERAL(0, 0, 15), // "VymModelWrapper"
QT_MOC_LITERAL(1, 16, 9), // "addBranch"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 15), // "addBranchBefore"
QT_MOC_LITERAL(4, 43, 12), // "addMapCenter"
QT_MOC_LITERAL(5, 56, 1), // "x"
QT_MOC_LITERAL(6, 58, 1), // "y"
QT_MOC_LITERAL(7, 60, 12), // "addMapInsert"
QT_MOC_LITERAL(8, 73, 8), // "filename"
QT_MOC_LITERAL(9, 82, 3), // "pos"
QT_MOC_LITERAL(10, 86, 13), // "contentFilter"
QT_MOC_LITERAL(11, 100, 13), // "addMapReplace"
QT_MOC_LITERAL(12, 114, 8), // "addSlide"
QT_MOC_LITERAL(13, 123, 8), // "addXLink"
QT_MOC_LITERAL(14, 132, 5), // "begin"
QT_MOC_LITERAL(15, 138, 3), // "end"
QT_MOC_LITERAL(16, 142, 5), // "width"
QT_MOC_LITERAL(17, 148, 5), // "color"
QT_MOC_LITERAL(18, 154, 8), // "penstyle"
QT_MOC_LITERAL(19, 163, 11), // "branchCount"
QT_MOC_LITERAL(20, 175, 11), // "centerCount"
QT_MOC_LITERAL(21, 187, 10), // "centerOnID"
QT_MOC_LITERAL(22, 198, 2), // "id"
QT_MOC_LITERAL(23, 201, 10), // "clearFlags"
QT_MOC_LITERAL(24, 212, 11), // "colorBranch"
QT_MOC_LITERAL(25, 224, 12), // "colorSubtree"
QT_MOC_LITERAL(26, 237, 4), // "copy"
QT_MOC_LITERAL(27, 242, 3), // "cut"
QT_MOC_LITERAL(28, 246, 9), // "cycleTask"
QT_MOC_LITERAL(29, 256, 5), // "depth"
QT_MOC_LITERAL(30, 262, 9), // "exportMap"
QT_MOC_LITERAL(31, 272, 14), // "getBranchIndex"
QT_MOC_LITERAL(32, 287, 11), // "getDestPath"
QT_MOC_LITERAL(33, 299, 10), // "getFileDir"
QT_MOC_LITERAL(34, 310, 11), // "getFileName"
QT_MOC_LITERAL(35, 322, 12), // "getFrameType"
QT_MOC_LITERAL(36, 335, 19), // "getHeadingPlainText"
QT_MOC_LITERAL(37, 355, 13), // "getHeadingXML"
QT_MOC_LITERAL(38, 369, 12), // "getMapAuthor"
QT_MOC_LITERAL(39, 382, 13), // "getMapComment"
QT_MOC_LITERAL(40, 396, 11), // "getMapTitle"
QT_MOC_LITERAL(41, 408, 16), // "getNotePlainText"
QT_MOC_LITERAL(42, 425, 10), // "getNoteXML"
QT_MOC_LITERAL(43, 436, 18), // "getSelectionString"
QT_MOC_LITERAL(44, 455, 20), // "getTaskPriorityDelta"
QT_MOC_LITERAL(45, 476, 12), // "getTaskSleep"
QT_MOC_LITERAL(46, 489, 16), // "getTaskSleepDays"
QT_MOC_LITERAL(47, 506, 6), // "getURL"
QT_MOC_LITERAL(48, 513, 10), // "getVymLink"
QT_MOC_LITERAL(49, 524, 13), // "getXLinkColor"
QT_MOC_LITERAL(50, 538, 13), // "getXLinkWidth"
QT_MOC_LITERAL(51, 552, 16), // "getXLinkPenStyle"
QT_MOC_LITERAL(52, 569, 18), // "getXLinkStyleBegin"
QT_MOC_LITERAL(53, 588, 16), // "getXLinkStyleEnd"
QT_MOC_LITERAL(54, 605, 13), // "hasActiveFlag"
QT_MOC_LITERAL(55, 619, 4), // "flag"
QT_MOC_LITERAL(56, 624, 7), // "hasNote"
QT_MOC_LITERAL(57, 632, 15), // "hasRichTextNote"
QT_MOC_LITERAL(58, 648, 7), // "hasTask"
QT_MOC_LITERAL(59, 656, 9), // "importDir"
QT_MOC_LITERAL(60, 666, 4), // "path"
QT_MOC_LITERAL(61, 671, 12), // "initIterator"
QT_MOC_LITERAL(62, 684, 5), // "iname"
QT_MOC_LITERAL(63, 690, 15), // "deepLevelsFirst"
QT_MOC_LITERAL(64, 706, 10), // "isScrolled"
QT_MOC_LITERAL(65, 717, 9), // "loadImage"
QT_MOC_LITERAL(66, 727, 8), // "loadNote"
QT_MOC_LITERAL(67, 736, 4), // "move"
QT_MOC_LITERAL(68, 741, 7), // "moveRel"
QT_MOC_LITERAL(69, 749, 8), // "moveDown"
QT_MOC_LITERAL(70, 758, 6), // "moveUp"
QT_MOC_LITERAL(71, 765, 13), // "moveSlideDown"
QT_MOC_LITERAL(72, 779, 1), // "n"
QT_MOC_LITERAL(73, 781, 11), // "moveSlideUp"
QT_MOC_LITERAL(74, 793, 12), // "nextIterator"
QT_MOC_LITERAL(75, 806, 3), // "nop"
QT_MOC_LITERAL(76, 810, 9), // "note2URLs"
QT_MOC_LITERAL(77, 820, 12), // "parseVymText"
QT_MOC_LITERAL(78, 833, 4), // "text"
QT_MOC_LITERAL(79, 838, 5), // "paste"
QT_MOC_LITERAL(80, 844, 4), // "redo"
QT_MOC_LITERAL(81, 849, 8), // "relinkTo"
QT_MOC_LITERAL(82, 858, 6), // "parent"
QT_MOC_LITERAL(83, 865, 3), // "num"
QT_MOC_LITERAL(84, 869, 6), // "remove"
QT_MOC_LITERAL(85, 876, 14), // "removeChildren"
QT_MOC_LITERAL(86, 891, 18), // "removeKeepChildren"
QT_MOC_LITERAL(87, 910, 11), // "removeSlide"
QT_MOC_LITERAL(88, 922, 17), // "repeatLastCommand"
QT_MOC_LITERAL(89, 940, 9), // "saveImage"
QT_MOC_LITERAL(90, 950, 8), // "saveNote"
QT_MOC_LITERAL(91, 959, 6), // "scroll"
QT_MOC_LITERAL(92, 966, 6), // "select"
QT_MOC_LITERAL(93, 973, 1), // "s"
QT_MOC_LITERAL(94, 975, 8), // "selectID"
QT_MOC_LITERAL(95, 984, 17), // "selectFirstBranch"
QT_MOC_LITERAL(96, 1002, 22), // "selectFirstChildBranch"
QT_MOC_LITERAL(97, 1025, 16), // "selectLastBranch"
QT_MOC_LITERAL(98, 1042, 21), // "selectLastChildBranch"
QT_MOC_LITERAL(99, 1064, 15), // "selectLastImage"
QT_MOC_LITERAL(100, 1080, 12), // "selectParent"
QT_MOC_LITERAL(101, 1093, 17), // "selectLatestAdded"
QT_MOC_LITERAL(102, 1111, 12), // "selectToggle"
QT_MOC_LITERAL(103, 1124, 12), // "selectString"
QT_MOC_LITERAL(104, 1137, 13), // "setFlagByName"
QT_MOC_LITERAL(105, 1151, 28), // "setHeadingConfluencePageName"
QT_MOC_LITERAL(106, 1180, 19), // "setHeadingPlainText"
QT_MOC_LITERAL(107, 1200, 13), // "setHideExport"
QT_MOC_LITERAL(108, 1214, 1), // "b"
QT_MOC_LITERAL(109, 1216, 21), // "setHideLinkUnselected"
QT_MOC_LITERAL(110, 1238, 28), // "setIncludeImagesHorizontally"
QT_MOC_LITERAL(111, 1267, 26), // "setIncludeImagesVertically"
QT_MOC_LITERAL(112, 1294, 15), // "setMapAnimCurve"
QT_MOC_LITERAL(113, 1310, 18), // "setMapAnimDuration"
QT_MOC_LITERAL(114, 1329, 12), // "setMapAuthor"
QT_MOC_LITERAL(115, 1342, 21), // "setMapBackgroundColor"
QT_MOC_LITERAL(116, 1364, 13), // "setMapComment"
QT_MOC_LITERAL(117, 1378, 18), // "setMapDefLinkColor"
QT_MOC_LITERAL(118, 1397, 15), // "setMapLinkStyle"
QT_MOC_LITERAL(119, 1413, 5), // "style"
QT_MOC_LITERAL(120, 1419, 14), // "setMapRotation"
QT_MOC_LITERAL(121, 1434, 1), // "a"
QT_MOC_LITERAL(122, 1436, 11), // "setMapTitle"
QT_MOC_LITERAL(123, 1448, 10), // "setMapZoom"
QT_MOC_LITERAL(124, 1459, 1), // "z"
QT_MOC_LITERAL(125, 1461, 16), // "setNotePlainText"
QT_MOC_LITERAL(126, 1478, 19), // "setFrameBorderWidth"
QT_MOC_LITERAL(127, 1498, 6), // "border"
QT_MOC_LITERAL(128, 1505, 18), // "setFrameBrushColor"
QT_MOC_LITERAL(129, 1524, 23), // "setFrameIncludeChildren"
QT_MOC_LITERAL(130, 1548, 15), // "setFramePadding"
QT_MOC_LITERAL(131, 1564, 7), // "padding"
QT_MOC_LITERAL(132, 1572, 16), // "setFramePenColor"
QT_MOC_LITERAL(133, 1589, 12), // "setFrameType"
QT_MOC_LITERAL(134, 1602, 4), // "type"
QT_MOC_LITERAL(135, 1607, 14), // "setScaleFactor"
QT_MOC_LITERAL(136, 1622, 1), // "f"
QT_MOC_LITERAL(137, 1624, 22), // "setSelectionBrushColor"
QT_MOC_LITERAL(138, 1647, 20), // "setSelectionPenColor"
QT_MOC_LITERAL(139, 1668, 20), // "setSelectionPenWidth"
QT_MOC_LITERAL(140, 1689, 20), // "setTaskPriorityDelta"
QT_MOC_LITERAL(141, 1710, 12), // "setTaskSleep"
QT_MOC_LITERAL(142, 1723, 6), // "setURL"
QT_MOC_LITERAL(143, 1730, 10), // "setVymLink"
QT_MOC_LITERAL(144, 1741, 13), // "setXLinkColor"
QT_MOC_LITERAL(145, 1755, 13), // "setXLinkStyle"
QT_MOC_LITERAL(146, 1769, 18), // "setXLinkStyleBegin"
QT_MOC_LITERAL(147, 1788, 16), // "setXLinkStyleEnd"
QT_MOC_LITERAL(148, 1805, 13), // "setXLinkWidth"
QT_MOC_LITERAL(149, 1819, 1), // "w"
QT_MOC_LITERAL(150, 1821, 5), // "sleep"
QT_MOC_LITERAL(151, 1827, 12), // "sortChildren"
QT_MOC_LITERAL(152, 1840, 15), // "toggleFlagByUid"
QT_MOC_LITERAL(153, 1856, 16), // "toggleFlagByName"
QT_MOC_LITERAL(154, 1873, 26), // "toggleFrameIncludeChildren"
QT_MOC_LITERAL(155, 1900, 12), // "toggleScroll"
QT_MOC_LITERAL(156, 1913, 12), // "toggleTarget"
QT_MOC_LITERAL(157, 1926, 10), // "toggleTask"
QT_MOC_LITERAL(158, 1937, 4), // "undo"
QT_MOC_LITERAL(159, 1942, 8), // "unscroll"
QT_MOC_LITERAL(160, 1951, 16), // "unscrollChildren"
QT_MOC_LITERAL(161, 1968, 11), // "unselectAll"
QT_MOC_LITERAL(162, 1980, 15) // "unsetFlagByName"

    },
    "VymModelWrapper\0addBranch\0\0addBranchBefore\0"
    "addMapCenter\0x\0y\0addMapInsert\0filename\0"
    "pos\0contentFilter\0addMapReplace\0"
    "addSlide\0addXLink\0begin\0end\0width\0"
    "color\0penstyle\0branchCount\0centerCount\0"
    "centerOnID\0id\0clearFlags\0colorBranch\0"
    "colorSubtree\0copy\0cut\0cycleTask\0depth\0"
    "exportMap\0getBranchIndex\0getDestPath\0"
    "getFileDir\0getFileName\0getFrameType\0"
    "getHeadingPlainText\0getHeadingXML\0"
    "getMapAuthor\0getMapComment\0getMapTitle\0"
    "getNotePlainText\0getNoteXML\0"
    "getSelectionString\0getTaskPriorityDelta\0"
    "getTaskSleep\0getTaskSleepDays\0getURL\0"
    "getVymLink\0getXLinkColor\0getXLinkWidth\0"
    "getXLinkPenStyle\0getXLinkStyleBegin\0"
    "getXLinkStyleEnd\0hasActiveFlag\0flag\0"
    "hasNote\0hasRichTextNote\0hasTask\0"
    "importDir\0path\0initIterator\0iname\0"
    "deepLevelsFirst\0isScrolled\0loadImage\0"
    "loadNote\0move\0moveRel\0moveDown\0moveUp\0"
    "moveSlideDown\0n\0moveSlideUp\0nextIterator\0"
    "nop\0note2URLs\0parseVymText\0text\0paste\0"
    "redo\0relinkTo\0parent\0num\0remove\0"
    "removeChildren\0removeKeepChildren\0"
    "removeSlide\0repeatLastCommand\0saveImage\0"
    "saveNote\0scroll\0select\0s\0selectID\0"
    "selectFirstBranch\0selectFirstChildBranch\0"
    "selectLastBranch\0selectLastChildBranch\0"
    "selectLastImage\0selectParent\0"
    "selectLatestAdded\0selectToggle\0"
    "selectString\0setFlagByName\0"
    "setHeadingConfluencePageName\0"
    "setHeadingPlainText\0setHideExport\0b\0"
    "setHideLinkUnselected\0"
    "setIncludeImagesHorizontally\0"
    "setIncludeImagesVertically\0setMapAnimCurve\0"
    "setMapAnimDuration\0setMapAuthor\0"
    "setMapBackgroundColor\0setMapComment\0"
    "setMapDefLinkColor\0setMapLinkStyle\0"
    "style\0setMapRotation\0a\0setMapTitle\0"
    "setMapZoom\0z\0setNotePlainText\0"
    "setFrameBorderWidth\0border\0"
    "setFrameBrushColor\0setFrameIncludeChildren\0"
    "setFramePadding\0padding\0setFramePenColor\0"
    "setFrameType\0type\0setScaleFactor\0f\0"
    "setSelectionBrushColor\0setSelectionPenColor\0"
    "setSelectionPenWidth\0setTaskPriorityDelta\0"
    "setTaskSleep\0setURL\0setVymLink\0"
    "setXLinkColor\0setXLinkStyle\0"
    "setXLinkStyleBegin\0setXLinkStyleEnd\0"
    "setXLinkWidth\0w\0sleep\0sortChildren\0"
    "toggleFlagByUid\0toggleFlagByName\0"
    "toggleFrameIncludeChildren\0toggleScroll\0"
    "toggleTarget\0toggleTask\0undo\0unscroll\0"
    "unscrollChildren\0unselectAll\0"
    "unsetFlagByName"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_VymModelWrapper[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     139,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  709,    2, 0x0a /* Public */,
       3,    0,  710,    2, 0x0a /* Public */,
       4,    2,  711,    2, 0x0a /* Public */,
       7,    3,  716,    2, 0x0a /* Public */,
       7,    2,  723,    2, 0x0a /* Public */,
       7,    1,  728,    2, 0x0a /* Public */,
      11,    1,  731,    2, 0x0a /* Public */,
      12,    0,  734,    2, 0x0a /* Public */,
      13,    5,  735,    2, 0x0a /* Public */,
      19,    0,  746,    2, 0x0a /* Public */,
      20,    0,  747,    2, 0x0a /* Public */,
      21,    1,  748,    2, 0x0a /* Public */,
      23,    0,  751,    2, 0x0a /* Public */,
      24,    1,  752,    2, 0x0a /* Public */,
      25,    1,  755,    2, 0x0a /* Public */,
      26,    0,  758,    2, 0x0a /* Public */,
      27,    0,  759,    2, 0x0a /* Public */,
      28,    0,  760,    2, 0x0a /* Public */,
      29,    0,  761,    2, 0x0a /* Public */,
      30,    0,  762,    2, 0x0a /* Public */,
      31,    0,  763,    2, 0x0a /* Public */,
      32,    0,  764,    2, 0x0a /* Public */,
      33,    0,  765,    2, 0x0a /* Public */,
      34,    0,  766,    2, 0x0a /* Public */,
      35,    0,  767,    2, 0x0a /* Public */,
      36,    0,  768,    2, 0x0a /* Public */,
      37,    0,  769,    2, 0x0a /* Public */,
      38,    0,  770,    2, 0x0a /* Public */,
      39,    0,  771,    2, 0x0a /* Public */,
      40,    0,  772,    2, 0x0a /* Public */,
      41,    0,  773,    2, 0x0a /* Public */,
      42,    0,  774,    2, 0x0a /* Public */,
      43,    0,  775,    2, 0x0a /* Public */,
      44,    0,  776,    2, 0x0a /* Public */,
      45,    0,  777,    2, 0x0a /* Public */,
      46,    0,  778,    2, 0x0a /* Public */,
      47,    0,  779,    2, 0x0a /* Public */,
      48,    0,  780,    2, 0x0a /* Public */,
      49,    0,  781,    2, 0x0a /* Public */,
      50,    0,  782,    2, 0x0a /* Public */,
      51,    0,  783,    2, 0x0a /* Public */,
      52,    0,  784,    2, 0x0a /* Public */,
      53,    0,  785,    2, 0x0a /* Public */,
      54,    1,  786,    2, 0x0a /* Public */,
      56,    0,  789,    2, 0x0a /* Public */,
      57,    0,  790,    2, 0x0a /* Public */,
      58,    0,  791,    2, 0x0a /* Public */,
      59,    1,  792,    2, 0x0a /* Public */,
      61,    2,  795,    2, 0x0a /* Public */,
      61,    1,  800,    2, 0x2a /* Public | MethodCloned */,
      64,    0,  803,    2, 0x0a /* Public */,
      65,    1,  804,    2, 0x0a /* Public */,
      66,    1,  807,    2, 0x0a /* Public */,
      67,    2,  810,    2, 0x0a /* Public */,
      68,    2,  815,    2, 0x0a /* Public */,
      69,    0,  820,    2, 0x0a /* Public */,
      70,    0,  821,    2, 0x0a /* Public */,
      71,    1,  822,    2, 0x0a /* Public */,
      71,    0,  825,    2, 0x0a /* Public */,
      73,    1,  826,    2, 0x0a /* Public */,
      73,    0,  829,    2, 0x0a /* Public */,
      74,    1,  830,    2, 0x0a /* Public */,
      75,    0,  833,    2, 0x0a /* Public */,
      76,    0,  834,    2, 0x0a /* Public */,
      77,    1,  835,    2, 0x0a /* Public */,
      79,    0,  838,    2, 0x0a /* Public */,
      80,    0,  839,    2, 0x0a /* Public */,
      81,    4,  840,    2, 0x0a /* Public */,
      81,    2,  849,    2, 0x0a /* Public */,
      81,    1,  854,    2, 0x0a /* Public */,
      84,    0,  857,    2, 0x0a /* Public */,
      85,    0,  858,    2, 0x0a /* Public */,
      86,    0,  859,    2, 0x0a /* Public */,
      87,    1,  860,    2, 0x0a /* Public */,
      88,    0,  863,    2, 0x0a /* Public */,
      89,    1,  864,    2, 0x0a /* Public */,
      90,    1,  867,    2, 0x0a /* Public */,
      91,    0,  870,    2, 0x0a /* Public */,
      92,    1,  871,    2, 0x0a /* Public */,
      94,    1,  874,    2, 0x0a /* Public */,
      95,    0,  877,    2, 0x0a /* Public */,
      96,    0,  878,    2, 0x0a /* Public */,
      97,    0,  879,    2, 0x0a /* Public */,
      98,    0,  880,    2, 0x0a /* Public */,
      99,    0,  881,    2, 0x0a /* Public */,
     100,    0,  882,    2, 0x0a /* Public */,
     101,    0,  883,    2, 0x0a /* Public */,
     102,    1,  884,    2, 0x0a /* Public */,
     104,    1,  887,    2, 0x0a /* Public */,
     105,    0,  890,    2, 0x0a /* Public */,
     106,    1,  891,    2, 0x0a /* Public */,
     107,    1,  894,    2, 0x0a /* Public */,
     109,    1,  897,    2, 0x0a /* Public */,
     110,    1,  900,    2, 0x0a /* Public */,
     111,    1,  903,    2, 0x0a /* Public */,
     112,    1,  906,    2, 0x0a /* Public */,
     113,    1,  909,    2, 0x0a /* Public */,
     114,    1,  912,    2, 0x0a /* Public */,
     115,    1,  915,    2, 0x0a /* Public */,
     116,    1,  918,    2, 0x0a /* Public */,
     117,    1,  921,    2, 0x0a /* Public */,
     118,    1,  924,    2, 0x0a /* Public */,
     120,    1,  927,    2, 0x0a /* Public */,
     122,    1,  930,    2, 0x0a /* Public */,
     123,    1,  933,    2, 0x0a /* Public */,
     125,    1,  936,    2, 0x0a /* Public */,
     126,    1,  939,    2, 0x0a /* Public */,
     128,    1,  942,    2, 0x0a /* Public */,
     129,    1,  945,    2, 0x0a /* Public */,
     130,    1,  948,    2, 0x0a /* Public */,
     132,    1,  951,    2, 0x0a /* Public */,
     133,    1,  954,    2, 0x0a /* Public */,
     135,    1,  957,    2, 0x0a /* Public */,
     137,    1,  960,    2, 0x0a /* Public */,
     138,    1,  963,    2, 0x0a /* Public */,
     139,    1,  966,    2, 0x0a /* Public */,
     140,    1,  969,    2, 0x0a /* Public */,
     141,    1,  972,    2, 0x0a /* Public */,
     142,    1,  975,    2, 0x0a /* Public */,
     143,    1,  978,    2, 0x0a /* Public */,
     144,    1,  981,    2, 0x0a /* Public */,
     145,    1,  984,    2, 0x0a /* Public */,
     146,    1,  987,    2, 0x0a /* Public */,
     147,    1,  990,    2, 0x0a /* Public */,
     148,    1,  993,    2, 0x0a /* Public */,
     150,    1,  996,    2, 0x0a /* Public */,
     151,    1,  999,    2, 0x0a /* Public */,
     151,    0, 1002,    2, 0x0a /* Public */,
     152,    1, 1003,    2, 0x0a /* Public */,
     153,    1, 1006,    2, 0x0a /* Public */,
     154,    0, 1009,    2, 0x0a /* Public */,
     155,    0, 1010,    2, 0x0a /* Public */,
     156,    0, 1011,    2, 0x0a /* Public */,
     157,    0, 1012,    2, 0x0a /* Public */,
     158,    0, 1013,    2, 0x0a /* Public */,
     159,    0, 1014,    2, 0x0a /* Public */,
     160,    0, 1015,    2, 0x0a /* Public */,
     161,    0, 1016,    2, 0x0a /* Public */,
     162,    1, 1017,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QReal, QMetaType::QReal,    5,    6,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::Int,    8,    9,   10,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    8,    9,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int, QMetaType::QString, QMetaType::QString,   14,   15,   16,   17,   18,
    QMetaType::Int,
    QMetaType::Int,
    QMetaType::Void, QMetaType::QString,   22,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Int,
    QMetaType::Bool,
    QMetaType::Int,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::Int,
    QMetaType::QString,
    QMetaType::Int,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::Int,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::Bool, QMetaType::QString,   55,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::QString,   60,
    QMetaType::Bool, QMetaType::QString, QMetaType::Bool,   62,   63,
    QMetaType::Bool, QMetaType::QString,   62,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QReal, QMetaType::QReal,    5,    6,
    QMetaType::Void, QMetaType::QReal, QMetaType::QReal,    5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   72,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   72,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString,   62,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString,   78,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString, QMetaType::Int, QMetaType::QReal, QMetaType::QReal,   82,   83,    5,    6,
    QMetaType::Bool, QMetaType::QString, QMetaType::Int,   82,   83,
    QMetaType::Bool, QMetaType::QString,   82,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   72,
    QMetaType::QVariant,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString,   93,
    QMetaType::Bool, QMetaType::QString,   93,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool, QMetaType::QString,  103,
    QMetaType::Void, QMetaType::QString,   93,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   93,
    QMetaType::Void, QMetaType::Bool,  108,
    QMetaType::Void, QMetaType::Bool,  108,
    QMetaType::Void, QMetaType::Bool,  108,
    QMetaType::Void, QMetaType::Bool,  108,
    QMetaType::Void, QMetaType::Int,   72,
    QMetaType::Void, QMetaType::Int,   72,
    QMetaType::Void, QMetaType::QString,   93,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, QMetaType::QString,   93,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, QMetaType::QString,  119,
    QMetaType::Void, QMetaType::Float,  121,
    QMetaType::Void, QMetaType::QString,   93,
    QMetaType::Void, QMetaType::Float,  124,
    QMetaType::Void, QMetaType::QString,   93,
    QMetaType::Void, QMetaType::Int,  127,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, QMetaType::Bool,  108,
    QMetaType::Void, QMetaType::Int,  131,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, QMetaType::QString,  134,
    QMetaType::Void, QMetaType::QReal,  136,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, QMetaType::QReal,    2,
    QMetaType::Void, QMetaType::Int,   72,
    QMetaType::Bool, QMetaType::QString,   93,
    QMetaType::Void, QMetaType::QString,   93,
    QMetaType::Void, QMetaType::QString,   93,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, QMetaType::QString,  119,
    QMetaType::Void, QMetaType::QString,  119,
    QMetaType::Void, QMetaType::QString,  119,
    QMetaType::Void, QMetaType::Int,  149,
    QMetaType::Void, QMetaType::Int,   72,
    QMetaType::Void, QMetaType::Bool,  108,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   93,
    QMetaType::Void, QMetaType::QString,   93,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   93,

       0        // eod
};

void VymModelWrapper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<VymModelWrapper *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->addBranch(); break;
        case 1: _t->addBranchBefore(); break;
        case 2: _t->addMapCenter((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2]))); break;
        case 3: _t->addMapInsert((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 4: _t->addMapInsert((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->addMapInsert((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->addMapReplace((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 7: _t->addSlide(); break;
        case 8: _t->addXLink((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5]))); break;
        case 9: { int _r = _t->branchCount();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 10: { int _r = _t->centerCount();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->centerOnID((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->clearFlags(); break;
        case 13: _t->colorBranch((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->colorSubtree((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->copy(); break;
        case 16: _t->cut(); break;
        case 17: _t->cycleTask(); break;
        case 18: { int _r = _t->depth();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->exportMap();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { int _r = _t->getBranchIndex();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 21: { QString _r = _t->getDestPath();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 22: { QString _r = _t->getFileDir();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 23: { QString _r = _t->getFileName();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 24: { QString _r = _t->getFrameType();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 25: { QString _r = _t->getHeadingPlainText();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 26: { QString _r = _t->getHeadingXML();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 27: { QString _r = _t->getMapAuthor();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 28: { QString _r = _t->getMapComment();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 29: { QString _r = _t->getMapTitle();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 30: { QString _r = _t->getNotePlainText();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 31: { QString _r = _t->getNoteXML();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 32: { QString _r = _t->getSelectionString();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 33: { int _r = _t->getTaskPriorityDelta();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 34: { QString _r = _t->getTaskSleep();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 35: { int _r = _t->getTaskSleepDays();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 36: { QString _r = _t->getURL();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 37: { QString _r = _t->getVymLink();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 38: { QString _r = _t->getXLinkColor();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 39: { int _r = _t->getXLinkWidth();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 40: { QString _r = _t->getXLinkPenStyle();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 41: { QString _r = _t->getXLinkStyleBegin();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 42: { QString _r = _t->getXLinkStyleEnd();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 43: { bool _r = _t->hasActiveFlag((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 44: { bool _r = _t->hasNote();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 45: { bool _r = _t->hasRichTextNote();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 46: { bool _r = _t->hasTask();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 47: _t->importDir((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 48: { bool _r = _t->initIterator((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 49: { bool _r = _t->initIterator((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 50: { bool _r = _t->isScrolled();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 51: _t->loadImage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 52: _t->loadNote((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 53: _t->move((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2]))); break;
        case 54: _t->moveRel((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2]))); break;
        case 55: _t->moveDown(); break;
        case 56: _t->moveUp(); break;
        case 57: _t->moveSlideDown((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 58: _t->moveSlideDown(); break;
        case 59: _t->moveSlideUp((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 60: _t->moveSlideUp(); break;
        case 61: { bool _r = _t->nextIterator((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 62: _t->nop(); break;
        case 63: _t->note2URLs(); break;
        case 64: { bool _r = _t->parseVymText((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 65: _t->paste(); break;
        case 66: _t->redo(); break;
        case 67: { bool _r = _t->relinkTo((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< qreal(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 68: { bool _r = _t->relinkTo((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 69: { bool _r = _t->relinkTo((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 70: _t->remove(); break;
        case 71: _t->removeChildren(); break;
        case 72: _t->removeKeepChildren(); break;
        case 73: _t->removeSlide((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 74: { QVariant _r = _t->repeatLastCommand();
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 75: _t->saveImage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 76: _t->saveNote((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 77: _t->scroll(); break;
        case 78: { bool _r = _t->select((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 79: { bool _r = _t->selectID((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 80: { bool _r = _t->selectFirstBranch();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 81: { bool _r = _t->selectFirstChildBranch();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 82: { bool _r = _t->selectLastBranch();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 83: { bool _r = _t->selectLastChildBranch();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 84: { bool _r = _t->selectLastImage();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 85: { bool _r = _t->selectParent();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 86: { bool _r = _t->selectLatestAdded();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 87: { bool _r = _t->selectToggle((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 88: _t->setFlagByName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 89: _t->setHeadingConfluencePageName(); break;
        case 90: _t->setHeadingPlainText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 91: _t->setHideExport((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 92: _t->setHideLinkUnselected((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 93: _t->setIncludeImagesHorizontally((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 94: _t->setIncludeImagesVertically((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 95: _t->setMapAnimCurve((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 96: _t->setMapAnimDuration((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 97: _t->setMapAuthor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 98: _t->setMapBackgroundColor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 99: _t->setMapComment((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 100: _t->setMapDefLinkColor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 101: _t->setMapLinkStyle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 102: _t->setMapRotation((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 103: _t->setMapTitle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 104: _t->setMapZoom((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 105: _t->setNotePlainText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 106: _t->setFrameBorderWidth((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 107: _t->setFrameBrushColor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 108: _t->setFrameIncludeChildren((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 109: _t->setFramePadding((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 110: _t->setFramePenColor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 111: _t->setFrameType((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 112: _t->setScaleFactor((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 113: _t->setSelectionBrushColor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 114: _t->setSelectionPenColor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 115: _t->setSelectionPenWidth((*reinterpret_cast< const qreal(*)>(_a[1]))); break;
        case 116: _t->setTaskPriorityDelta((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 117: { bool _r = _t->setTaskSleep((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 118: _t->setURL((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 119: _t->setVymLink((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 120: _t->setXLinkColor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 121: _t->setXLinkStyle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 122: _t->setXLinkStyleBegin((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 123: _t->setXLinkStyleEnd((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 124: _t->setXLinkWidth((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 125: _t->sleep((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 126: _t->sortChildren((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 127: _t->sortChildren(); break;
        case 128: _t->toggleFlagByUid((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 129: _t->toggleFlagByName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 130: _t->toggleFrameIncludeChildren(); break;
        case 131: _t->toggleScroll(); break;
        case 132: _t->toggleTarget(); break;
        case 133: _t->toggleTask(); break;
        case 134: _t->undo(); break;
        case 135: { bool _r = _t->unscroll();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 136: _t->unscrollChildren(); break;
        case 137: _t->unselectAll(); break;
        case 138: _t->unsetFlagByName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject VymModelWrapper::staticMetaObject = { {
    QMetaObject::SuperData::link<VymScriptContext::staticMetaObject>(),
    qt_meta_stringdata_VymModelWrapper.data,
    qt_meta_data_VymModelWrapper,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *VymModelWrapper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VymModelWrapper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_VymModelWrapper.stringdata0))
        return static_cast<void*>(this);
    return VymScriptContext::qt_metacast(_clname);
}

int VymModelWrapper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = VymScriptContext::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 139)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 139;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 139)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 139;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
