/****************************************************************************
** Meta object code from reading C++ file 'adaptormodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dbus/adaptormodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'adaptormodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AdaptorModel_t {
    QByteArrayData data[12];
    char stringdata0[147];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AdaptorModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AdaptorModel_t qt_meta_stringdata_AdaptorModel = {
    {
QT_MOC_LITERAL(0, 0, 12), // "AdaptorModel"
QT_MOC_LITERAL(1, 13, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 29, 32), // "org.insilmaril.vym.model.adaptor"
QT_MOC_LITERAL(3, 62, 7), // "crashed"
QT_MOC_LITERAL(4, 70, 0), // ""
QT_MOC_LITERAL(5, 71, 11), // "branchCount"
QT_MOC_LITERAL(6, 83, 12), // "QDBusVariant"
QT_MOC_LITERAL(7, 96, 7), // "execute"
QT_MOC_LITERAL(8, 104, 1), // "s"
QT_MOC_LITERAL(9, 106, 10), // "errorLevel"
QT_MOC_LITERAL(10, 117, 16), // "errorDescription"
QT_MOC_LITERAL(11, 134, 12) // "listCommands"

    },
    "AdaptorModel\0D-Bus Interface\0"
    "org.insilmaril.vym.model.adaptor\0"
    "crashed\0\0branchCount\0QDBusVariant\0"
    "execute\0s\0errorLevel\0errorDescription\0"
    "listCommands"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AdaptorModel[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       6,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   46,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   47,    4, 0x0a /* Public */,
       7,    1,   48,    4, 0x0a /* Public */,
       9,    0,   51,    4, 0x0a /* Public */,
      10,    0,   52,    4, 0x0a /* Public */,
      11,    0,   53,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    0x80000000 | 6,
    0x80000000 | 6, QMetaType::QString,    8,
    0x80000000 | 6,
    0x80000000 | 6,
    0x80000000 | 6,

       0        // eod
};

void AdaptorModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AdaptorModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->crashed(); break;
        case 1: { QDBusVariant _r = _t->branchCount();
            if (_a[0]) *reinterpret_cast< QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusVariant _r = _t->execute((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusVariant _r = _t->errorLevel();
            if (_a[0]) *reinterpret_cast< QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusVariant _r = _t->errorDescription();
            if (_a[0]) *reinterpret_cast< QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusVariant _r = _t->listCommands();
            if (_a[0]) *reinterpret_cast< QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AdaptorModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AdaptorModel::crashed)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AdaptorModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_meta_stringdata_AdaptorModel.data,
    qt_meta_data_AdaptorModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AdaptorModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AdaptorModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AdaptorModel.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int AdaptorModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void AdaptorModel::crashed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
