/********************************************************************************
** Form generated from reading UI file 'darktheme-settings-dialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.11
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DARKTHEME_2D_SETTINGS_2D_DIALOG_H
#define UI_DARKTHEME_2D_SETTINGS_2D_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DarkThemeSettingsDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_3;
    QVBoxLayout *verticalLayout;
    QRadioButton *systemUseDarkThemeButton;
    QRadioButton *alwaysUseDarkThemeButton;
    QRadioButton *neverUseDarkThemeButton;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *closeButton;
    QButtonGroup *buttonGroup;

    void setupUi(QDialog *DarkThemeSettingsDialog)
    {
        if (DarkThemeSettingsDialog->objectName().isEmpty())
            DarkThemeSettingsDialog->setObjectName(QString::fromUtf8("DarkThemeSettingsDialog"));
        DarkThemeSettingsDialog->resize(278, 181);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DarkThemeSettingsDialog->sizePolicy().hasHeightForWidth());
        DarkThemeSettingsDialog->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(DarkThemeSettingsDialog);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        groupBox = new QGroupBox(DarkThemeSettingsDialog);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy1);
        verticalLayout_3 = new QVBoxLayout(groupBox);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        systemUseDarkThemeButton = new QRadioButton(groupBox);
        buttonGroup = new QButtonGroup(DarkThemeSettingsDialog);
        buttonGroup->setObjectName(QString::fromUtf8("buttonGroup"));
        buttonGroup->addButton(systemUseDarkThemeButton);
        systemUseDarkThemeButton->setObjectName(QString::fromUtf8("systemUseDarkThemeButton"));

        verticalLayout->addWidget(systemUseDarkThemeButton);

        alwaysUseDarkThemeButton = new QRadioButton(groupBox);
        buttonGroup->addButton(alwaysUseDarkThemeButton);
        alwaysUseDarkThemeButton->setObjectName(QString::fromUtf8("alwaysUseDarkThemeButton"));

        verticalLayout->addWidget(alwaysUseDarkThemeButton);

        neverUseDarkThemeButton = new QRadioButton(groupBox);
        neverUseDarkThemeButton->setObjectName(QString::fromUtf8("neverUseDarkThemeButton"));

        verticalLayout->addWidget(neverUseDarkThemeButton);


        verticalLayout_3->addLayout(verticalLayout);


        verticalLayout_2->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 9, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        closeButton = new QDialogButtonBox(DarkThemeSettingsDialog);
        closeButton->setObjectName(QString::fromUtf8("closeButton"));
        closeButton->setOrientation(Qt::Horizontal);
        closeButton->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout_2->addWidget(closeButton);


        retranslateUi(DarkThemeSettingsDialog);
        QObject::connect(closeButton, SIGNAL(accepted()), DarkThemeSettingsDialog, SLOT(accept()));
        QObject::connect(closeButton, SIGNAL(rejected()), DarkThemeSettingsDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(DarkThemeSettingsDialog);
    } // setupUi

    void retranslateUi(QDialog *DarkThemeSettingsDialog)
    {
        DarkThemeSettingsDialog->setWindowTitle(QCoreApplication::translate("DarkThemeSettingsDialog", "Dark theme settings", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DarkThemeSettingsDialog", "Dark theme", nullptr));
        systemUseDarkThemeButton->setText(QCoreApplication::translate("DarkThemeSettingsDialog", "System settings for dark theme", nullptr));
        alwaysUseDarkThemeButton->setText(QCoreApplication::translate("DarkThemeSettingsDialog", "Always use dark theme", nullptr));
        neverUseDarkThemeButton->setText(QCoreApplication::translate("DarkThemeSettingsDialog", "Never use dark theme", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DarkThemeSettingsDialog: public Ui_DarkThemeSettingsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DARKTHEME_2D_SETTINGS_2D_DIALOG_H
