/****************************************************************************
** Meta object code from reading C++ file 'scripting.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.11)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/scripting.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scripting.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.11. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_VymScriptContext_t {
    QByteArrayData data[1];
    char stringdata0[17];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_VymScriptContext_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_VymScriptContext_t qt_meta_stringdata_VymScriptContext = {
    {
QT_MOC_LITERAL(0, 0, 16) // "VymScriptContext"

    },
    "VymScriptContext"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_VymScriptContext[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void VymScriptContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject VymScriptContext::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_VymScriptContext.data,
    qt_meta_data_VymScriptContext,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *VymScriptContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VymScriptContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_VymScriptContext.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QScriptable"))
        return static_cast< QScriptable*>(this);
    return QObject::qt_metacast(_clname);
}

int VymScriptContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_VymWrapper_t {
    QByteArrayData data[20];
    char stringdata0[214];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_VymWrapper_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_VymWrapper_t qt_meta_stringdata_VymWrapper = {
    {
QT_MOC_LITERAL(0, 0, 10), // "VymWrapper"
QT_MOC_LITERAL(1, 11, 12), // "clearConsole"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 26), // "isConfluenceAgentAvailable"
QT_MOC_LITERAL(4, 52, 10), // "currentMap"
QT_MOC_LITERAL(5, 63, 11), // "editHeading"
QT_MOC_LITERAL(6, 75, 7), // "loadMap"
QT_MOC_LITERAL(7, 83, 8), // "filename"
QT_MOC_LITERAL(8, 92, 8), // "mapCount"
QT_MOC_LITERAL(9, 101, 7), // "gotoMap"
QT_MOC_LITERAL(10, 109, 1), // "n"
QT_MOC_LITERAL(11, 111, 14), // "closeMapWithID"
QT_MOC_LITERAL(12, 126, 16), // "selectQuickColor"
QT_MOC_LITERAL(13, 143, 12), // "currentColor"
QT_MOC_LITERAL(14, 156, 12), // "currentMapID"
QT_MOC_LITERAL(15, 169, 16), // "toggleTreeEditor"
QT_MOC_LITERAL(16, 186, 8), // "loadFile"
QT_MOC_LITERAL(17, 195, 8), // "saveFile"
QT_MOC_LITERAL(18, 204, 1), // "s"
QT_MOC_LITERAL(19, 206, 7) // "version"

    },
    "VymWrapper\0clearConsole\0\0"
    "isConfluenceAgentAvailable\0currentMap\0"
    "editHeading\0loadMap\0filename\0mapCount\0"
    "gotoMap\0n\0closeMapWithID\0selectQuickColor\0"
    "currentColor\0currentMapID\0toggleTreeEditor\0"
    "loadFile\0saveFile\0s\0version"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_VymWrapper[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   89,    2, 0x0a /* Public */,
       3,    0,   90,    2, 0x0a /* Public */,
       4,    0,   91,    2, 0x0a /* Public */,
       5,    0,   92,    2, 0x0a /* Public */,
       6,    1,   93,    2, 0x0a /* Public */,
       8,    0,   96,    2, 0x0a /* Public */,
       9,    1,   97,    2, 0x0a /* Public */,
      11,    1,  100,    2, 0x0a /* Public */,
      12,    1,  103,    2, 0x0a /* Public */,
      13,    0,  106,    2, 0x0a /* Public */,
      14,    0,  107,    2, 0x0a /* Public */,
      15,    0,  108,    2, 0x0a /* Public */,
      16,    1,  109,    2, 0x0a /* Public */,
      17,    2,  112,    2, 0x0a /* Public */,
      19,    0,  117,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::QObjectStar,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString,    7,
    QMetaType::Int,
    QMetaType::Void, QMetaType::UInt,   10,
    QMetaType::Bool, QMetaType::UInt,   10,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::QString,
    QMetaType::UInt,
    QMetaType::Void,
    QMetaType::QString, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    7,   18,
    QMetaType::QString,

       0        // eod
};

void VymWrapper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<VymWrapper *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->clearConsole(); break;
        case 1: { bool _r = _t->isConfluenceAgentAvailable();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { QObject* _r = _t->currentMap();
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        case 3: _t->editHeading(); break;
        case 4: { bool _r = _t->loadMap((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { int _r = _t->mapCount();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->gotoMap((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 7: { bool _r = _t->closeMapWithID((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->selectQuickColor((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: { QString _r = _t->currentColor();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { uint _r = _t->currentMapID();
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->toggleTreeEditor(); break;
        case 12: { QString _r = _t->loadFile((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->saveFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 14: { QString _r = _t->version();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject VymWrapper::staticMetaObject = { {
    QMetaObject::SuperData::link<VymScriptContext::staticMetaObject>(),
    qt_meta_stringdata_VymWrapper.data,
    qt_meta_data_VymWrapper,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *VymWrapper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VymWrapper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_VymWrapper.stringdata0))
        return static_cast<void*>(this);
    return VymScriptContext::qt_metacast(_clname);
}

int VymWrapper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = VymScriptContext::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}
struct qt_meta_stringdata_Selection_t {
    QByteArrayData data[6];
    char stringdata0[45];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Selection_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Selection_t qt_meta_stringdata_Selection = {
    {
QT_MOC_LITERAL(0, 0, 9), // "Selection"
QT_MOC_LITERAL(1, 10, 4), // "test"
QT_MOC_LITERAL(2, 15, 0), // ""
QT_MOC_LITERAL(3, 16, 8), // "setModel"
QT_MOC_LITERAL(4, 25, 16), // "VymModelWrapper*"
QT_MOC_LITERAL(5, 42, 2) // "mw"

    },
    "Selection\0test\0\0setModel\0VymModelWrapper*\0"
    "mw"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Selection[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x0a /* Public */,
       3,    1,   25,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,

       0        // eod
};

void Selection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Selection *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->test(); break;
        case 1: _t->setModel((*reinterpret_cast< VymModelWrapper*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Selection::staticMetaObject = { {
    QMetaObject::SuperData::link<VymScriptContext::staticMetaObject>(),
    qt_meta_stringdata_Selection.data,
    qt_meta_data_Selection,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Selection::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Selection::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Selection.stringdata0))
        return static_cast<void*>(this);
    return VymScriptContext::qt_metacast(_clname);
}

int Selection::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = VymScriptContext::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
